@@ -147,6 +147,7 @@ void CoordinateSystem::chooseAxes()
 		axes[i].setScaling(false);
 		axes[i].setNumbers(false);
 		axes[i].setLabel(false);
+		axes[i].setDecorate(false);
 	}
 
 	vector<unsigned> idx;
@@ -313,6 +314,7 @@ void CoordinateSystem::autoDecorateExposedAxis(Axis& ax, bool left)
 	ax.setScaling(true);
 	ax.setNumbers(true);
 	ax.setLabel(true);
+	ax.setDecorate(true);
 
 	const double SQRT_2 = 0.7071067;
 	double sina = fabs(diff.y / s);
