@@ -27,9 +27,10 @@
  *                                                                         *
  ***************************************************************************/
 #include "Correlation.h"
-#include "../ColorBox.h"
-#include "../plot2D/MultiLayer.h"
-#include "../plot2D/PlotCurve.h"
+#include <MultiLayer.h>
+#include <PlotCurve.h>
+#include <ColorBox.h>
+
 #include <QMessageBox>
 #include <QLocale>
 
@@ -64,10 +65,8 @@ bool Correlation::setDataFromTable(Table *t, const QString& colName1, const QStr
 		return false;
 	}
 
-    if (d_n > 0){//delete previousely allocated memory
-		delete[] d_x;
-		delete[] d_y;
-	}
+    if (d_n > 0)//delete previousely allocated memory
+		freeMemory();
 
     startRow--; endRow--;
 	if (startRow < 0 || startRow >= t->numRows())
@@ -83,8 +82,18 @@ bool Correlation::setDataFromTable(Table *t, const QString& colName1, const QStr
 	while (d_n < rows)
 		d_n *= 2;
 
-    d_x = new double[d_n];
-	d_y = new double[d_n];
+    d_x = (double *)malloc(d_n*sizeof(double));
+	if (!d_x){
+		memoryErrorMessage();
+		return false;
+	};
+
+	d_y = (double *)malloc(d_n*sizeof(double));
+	if (!d_y){
+		memoryErrorMessage();
+		free(d_x);
+		return false;
+	};
 
     if(d_y && d_x){
 		memset( d_x, 0, d_n * sizeof( double ) ); // zero-pad the two arrays...
@@ -95,9 +104,7 @@ bool Correlation::setDataFromTable(Table *t, const QString& colName1, const QStr
 			d_y[i] = d_table->cell(j, col2);
 		}
 	} else {
-		QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot") + " - " + tr("Error"),
-                        tr("Could not allocate memory, operation aborted!"));
-        d_init_err = true;
+		memoryErrorMessage();
 		d_n = 0;
 		return false;
 	}
@@ -150,11 +157,7 @@ void Correlation::addResultCurve()
 	d_table->addCol();
 	int n = d_n/2;
 
-#ifdef Q_CC_MSVC
-    QVarLengthArray<double> x_temp(d_n), y_temp(d_n);
-#else
     double x_temp[d_n], y_temp[d_n];
-#endif
 	for (int i = 0; i<d_n; i++){
 	    double x = i - n;
         x_temp[i] = x;
@@ -181,15 +184,11 @@ void Correlation::addResultCurve()
 
 	if (d_graphics_display){
 		if (!d_output_graph)
-			d_output_graph = createOutputGraph()->activeLayer();
+			createOutputGraph();
 
     	DataCurve *c = new DataCurve(d_table, d_table->colName(cols), d_table->colName(cols2));
-#ifdef Q_CC_MSVC
-		c->setData(x_temp.data(), y_temp.data(), d_n);
-#else
 		c->setData(x_temp, y_temp, d_n);
-#endif
-    	c->setPen(QPen(ColorBox::color(d_curveColorIndex), 1));
+		c->setPen(QPen(d_curveColor, 1));
 		d_output_graph->insertPlotItem(c, Graph::Line);
 		d_output_graph->updatePlot();
 	}
