@@ -66,9 +66,10 @@ class Filter : public QObject
 		//! Sets the tolerance used by the GSL routines
 		void setTolerance(double eps){d_tolerance = eps;};
 
+		//! Obsolete: sets the color of the output fit curve.
+		void setColor(int colorId);
 		//! Sets the color of the output fit curve.
-		void setColor(int colorId){d_curveColorIndex = colorId;};
-
+		void setColor(const QColor& color){d_curveColor = color;};
         //! Sets the color of the output fit curve. Provided for convenience. To be used in scripts only!
         void setColor(const QString& colorName);
 
@@ -161,7 +162,7 @@ class Filter : public QObject
 		int d_points;
 
 		//! Color index of the result curve
-		int d_curveColorIndex;
+		QColor d_curveColor;
 
 		//! Maximum number of iterations per fit
 		int d_max_iterations;
