@@ -71,7 +71,7 @@ void Fit::init()
 	d_x = 0;
 	d_y = 0;
 	d_w = 0;
-	d_curveColorIndex = 1;
+	d_curveColor = Qt::red;
 	d_solver = ScaledLevenbergMarquardt;
 	d_tolerance = 1e-4;
 	d_gen_function = true;
@@ -312,8 +312,7 @@ QString Fit::logFitInfo(int iterations, int status)
 		info +="\n";
 
 	info += tr("Weighting Method") + ": ";
-	switch(d_weighting)
-	{
+	switch(d_weighting){
 		case NoWeighting:
 			info += tr("No weighting");
 			break;
@@ -346,7 +345,8 @@ QString Fit::logFitInfo(int iterations, int status)
 	}
 
 	info += tr("From x")+" = "+locale.toString(d_x[0], 'e', d_prec)+" "+tr("to x")+" = "+locale.toString(d_x[d_n-1], 'e', d_prec)+"\n";
-	double chi_2_dof = chi_2/(d_n - d_p);
+	int dof = d_n - d_p;
+	double chi_2_dof = chi_2/dof;
 	for (int i=0; i<d_p; i++){
 		info += d_param_names[i];
 		if (!d_param_explain[i].isEmpty())
@@ -360,8 +360,10 @@ QString Fit::logFitInfo(int iterations, int status)
 	info += "--------------------------------------------------------------------------------------\n";
 	info += "Chi^2/doF = " + locale.toString(chi_2_dof, 'e', d_prec) + "\n";
 	info += tr("R^2") + " = " + locale.toString(rSquare(), 'g', d_prec) + "\n";
-	info += tr("Adjusted R^2") + " = " + locale.toString(d_adjusted_r_square, 'g', d_prec) + "\n";
-	info += tr("RMSE (Root Mean Squared Error)") + " = " + locale.toString(sqrt(d_rss/(d_n - d_p)), 'g', d_prec) + "\n";
+	if (dof > 1)
+		info += tr("Adjusted R^2") + " = " + locale.toString(d_adjusted_r_square, 'g', d_prec) + "\n";
+	if (dof >= 1)
+		info += tr("RMSE (Root Mean Squared Error)") + " = " + locale.toString(sqrt(d_rss/dof), 'g', d_prec) + "\n";
 	info += tr("RSS (Residual Sum of Squares)") + " = " + locale.toString(d_rss, 'g', d_prec) + "\n";
 	info += "---------------------------------------------------------------------------------------\n";
 	if (is_non_linear){
@@ -406,18 +408,18 @@ QString Fit::legendInfo()
 	else
 		dataSet = d_y_col_name;
 
-	QString info = tr("Dataset") + ": " + dataSet + "\n";
-	info += tr("Function") + ": " + d_formula + "\n\n";
+	QString info = "<b>" + tr("Dataset") + "</b>: " + dataSet + "\n";
+	info += "<b>" + tr("Function") + "</b>: " + d_formula + "\n";
 
 	ApplicationWindow *app = (ApplicationWindow *)parent();
 	QLocale locale = app->locale();
 
 	double chi_2_dof = chi_2/(d_n - d_p);
-	info += "Chi^2/doF = " + locale.toString(chi_2_dof, 'e', d_prec) + "\n";
-	info += tr("R^2") + " = " + locale.toString(rSquare(), 'g', d_prec) + "\n";
+	info += "<b>" + tr("Chi^2/doF") + "</b> = " + locale.toString(chi_2_dof, 'e', d_prec) + "\n";
+	info += "<b>" + tr("R^2") + "</b> = " + locale.toString(rSquare(), 'g', d_prec) + "\n";
 
 	for (int i = 0; i < d_p; i++){
-		info += d_param_names[i] + " = " + locale.toString(d_results[i], 'e', d_prec) + " +/- ";
+		info += "<b>" + d_param_names[i] + "</b> = " + locale.toString(d_results[i], 'e', d_prec) + " +/- ";
 		if (d_scale_errors)
 			info += locale.toString(sqrt(chi_2_dof*gsl_matrix_get(covar, i, i)), 'e', d_prec);
 		else
@@ -530,7 +532,7 @@ bool Fit::setWeightingData(WeightingMethod w, const QString& colName)
 
 				if (t->numRows() < d_n){
   	            	QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Error"),
-  	                tr("The column %1 has less points than the fitted data set. Please choose another column!.").arg(colName));
+  	                tr("The column %1 has less points than the fitted data set. Please choose another column!").arg(colName));
   	                return false;
   	            }
 
@@ -556,6 +558,9 @@ Table* Fit::parametersTable(const QString& tableName)
 	}
 
 	d_param_table->setHeader(QStringList() << tr("Parameter") << tr("Value") << tr ("Error"));
+	
+	d_param_table->setColumnType(0, Table::Text);
+	d_param_table->setColPlotDesignation(0, Table::None);
 	d_param_table->setColPlotDesignation(2, Table::yErr);
 	d_param_table->setHeaderColType();
 
@@ -618,15 +623,15 @@ Matrix* Fit::covarianceMatrix(const QString& matrixName)
 
 double *Fit::errors()
 {
-	if (!d_errors) {
+	if (!d_errors)
 		d_errors = new double[d_p];
-		double chi_2_dof = chi_2/(d_n - d_p);
-                for (int i = 0; i < d_p; i++){
-			if (d_scale_errors)
-				d_errors[i] = sqrt(chi_2_dof*gsl_matrix_get(covar,i,i));
-			else
-				d_errors[i] = sqrt(gsl_matrix_get(covar,i,i));
-		}
+
+	double chi_2_dof = chi_2/(d_n - d_p);
+	for (int i = 0; i < d_p; i++){
+		if (d_scale_errors)
+			d_errors[i] = sqrt(chi_2_dof*gsl_matrix_get(covar,i,i));
+		else
+			d_errors[i] = sqrt(gsl_matrix_get(covar,i,i));
 	}
 	return d_errors;
 }
@@ -675,7 +680,7 @@ QwtPlotCurve* Fit::showResiduals()
 	QString tableName = outputTable->objectName();
 	DataCurve *c = new DataCurve(outputTable, tableName + "_1", tableName + "_residue");
 	c->setData(d_x, d_residuals, d_n);
-    c->setPen(QPen(ColorBox::color(d_curveColorIndex + 1), 1));
+	c->setPen(QPen(ColorBox::color(ColorBox::colorIndex(d_curveColor) + 1), 1));
 
 	d_output_graph->insertPlotItem(c, Graph::Line);
     d_output_graph->updatePlot();
@@ -768,12 +773,12 @@ void Fit::showConfidenceLimits(double confidenceLevel)
 	QString tableName = outputTable->objectName();
 	DataCurve *c = new DataCurve(outputTable, tableName + "_1", tableName + "_LCL");
 	c->setData(X, lcl, points);
-    c->setPen(QPen(ColorBox::color(d_curveColorIndex + 2), 1));
+	c->setPen(QPen(ColorBox::color(ColorBox::colorIndex(d_curveColor) + 2), 1));
 	d_output_graph->insertPlotItem(c, Graph::Line);
 
 	c = new DataCurve(outputTable, tableName + "_1", tableName + "_UCL");
 	c->setData(X, ucl, points);
-    c->setPen(QPen(ColorBox::color(d_curveColorIndex + 2), 1));
+	c->setPen(QPen(ColorBox::color(ColorBox::colorIndex(d_curveColor) + 2), 1));
 	d_output_graph->insertPlotItem(c, Graph::Line);
 
     d_output_graph->updatePlot();
@@ -885,12 +890,12 @@ void Fit::showPredictionLimits(double confidenceLevel)
 	QString tableName = outputTable->objectName();
 	DataCurve *c = new DataCurve(outputTable, tableName + "_1", tableName + "_LPL");
 	c->setData(X, lcl, points);
-    c->setPen(QPen(ColorBox::color(d_curveColorIndex + 3), 1));
+	c->setPen(QPen(ColorBox::color(ColorBox::colorIndex(d_curveColor) + 3), 1));
 	d_output_graph->insertPlotItem(c, Graph::Line);
 
 	c = new DataCurve(outputTable, tableName + "_1", tableName + "_UPL");
 	c->setData(X, ucl, points);
-    c->setPen(QPen(ColorBox::color(d_curveColorIndex + 3), 1));
+	c->setPen(QPen(ColorBox::color(ColorBox::colorIndex(d_curveColor) + 3), 1));
 	d_output_graph->insertPlotItem(c, Graph::Line);
 
     d_output_graph->updatePlot();
@@ -926,9 +931,10 @@ void Fit::fit()
 		return;
 	}
 
-	QApplication::setOverrideCursor(Qt::WaitCursor);
+	if (!removeDataSingularities())
+		return;
 
-	removeDataSingularities();
+	QApplication::setOverrideCursor(Qt::WaitCursor);
 
 	struct FitData d_data = {d_n, d_p, d_x, d_y, d_w, this};
 
@@ -1019,11 +1025,15 @@ FunctionCurve * Fit::insertFitFunctionCurve(const QString& name, int penWidth, b
 {
 	QString title = d_output_graph->generateFunctionName(name);
 	FunctionCurve *c = new FunctionCurve(FunctionCurve::Normal, title);
-	c->setPen(QPen(ColorBox::color(d_curveColorIndex), penWidth));
-	c->setRange(d_from, d_to);
+	c->setPen(QPen(d_curveColor, penWidth));
+	c->setRange(QMIN(d_from, d_to), QMAX(d_from, d_to));
 	c->setFormula(d_formula);
+	if (d_curve){
+		c->setCurveType(d_curve->curveType());
+		c->setAxis(d_curve->xAxis(), d_curve->yAxis());
+	}
 
-	for (int j=0; j<d_p; j++)
+	for (int j = 0; j < d_p; j++)
 		c->setConstant(d_param_names[j], d_results[j]);
 
 	if (updateData)
