@@ -27,9 +27,9 @@
  *                                                                         *
  ***************************************************************************/
 #include "FilterDialog.h"
-#include "../FFTFilter.h"
-#include "../../plot2D/Graph.h"
-#include <ColorBox.h>
+#include "FFTFilter.h"
+#include <Graph.h>
+#include <ColorButton.h>
 #include <DoubleSpinBox.h>
 
 #include <QGroupBox>
@@ -46,6 +46,7 @@ FilterDialog::FilterDialog(int type, QWidget* parent, Qt::WFlags fl )
 	setObjectName( "FilterDialog" );
 	setWindowTitle(tr("QtiPlot - Filter options"));
 	setSizeGripEnabled( true );
+	setAttribute(Qt::WA_DeleteOnClose);
 
     QGroupBox *gb1 = new QGroupBox();
     QGridLayout *gl1 = new QGridLayout(gb1);
@@ -68,8 +69,8 @@ FilterDialog::FilterDialog(int type, QWidget* parent, Qt::WFlags fl )
 	boxStart->setMinimum(0.0);
 	gl1->addWidget(boxStart, 1, 1);
 
-	boxColor = new ColorBox(false);
-	boxColor->setColor(QColor(Qt::red));
+	boxColor = new ColorButton();
+	boxColor->setColor(Qt::red);
 	if (type >= FFTFilter::BandPass){
 	    gl1->addWidget(new QLabel(tr("High Frequency (Hz)")), 2, 0);
 
@@ -112,7 +113,7 @@ FilterDialog::FilterDialog(int type, QWidget* parent, Qt::WFlags fl )
     hb->addLayout(vl);
 
 	connect( buttonFilter, SIGNAL( clicked() ), this, SLOT( filter() ) );
-    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
+	connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( close() ) );
 }
 
 void FilterDialog::filter()
@@ -139,13 +140,15 @@ void FilterDialog::filter()
     } else
     	f->setCutoff(from);
 
-	f->setColor(boxColor->currentIndex());
+	f->setColor(boxColor->color());
 	f->run();
 	delete f;
 }
 
 void FilterDialog::setGraph(Graph *g)
 {
+	if (!g)
+		return;
 	graph = g;
 	boxName->addItems (g->analysableCurvesList());
 }
