@@ -2,7 +2,7 @@
     File                 : FitDialog.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2004-2007 by Ion Vasilief
+	Copyright            : (C) 2004-2010 by Ion Vasilief
     Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Fit Wizard
 
@@ -37,10 +37,10 @@
 #include <LogisticFit.h>
 #include <MyParser.h>
 #include <ApplicationWindow.h>
-#include <ColorBox.h>
+#include <ColorButton.h>
 #include <DoubleSpinBox.h>
 #include <FunctionCurve.h>
-#include <pixmaps.h>
+#include <ScriptEdit.h>
 
 #include <QListWidget>
 #include <QTableWidget>
@@ -66,29 +66,6 @@
 
 using namespace std;
 
-/* XPM */
-static const char * param_range_btn_xpm[] = {
-"18 14 5 1",
-" 	c None",
-".	c #000000",
-"+	c #FFFFFF",
-"@	c #808000",
-"#	c #FFFF00",
-"..+            ..+",
-"..+            ..+",
-"..+  @@+  @@+  ..+",
-"..+ @#@+  @#@+ ..+",
-"..+@##@+  @##@+..+",
-"..@###@@@@@###@..+",
-"..#############..+",
-"..@###@@@@@###@..+",
-"..+@##@+  @##@+..+",
-"..++@#@+  @#@++..+",
-"..+ +@@+  @@++ ..+",
-"..+  +++  +++  ..+",
-"..+            ..+",
-"+++            +++"};
-
 FitDialog::FitDialog(Graph *g, QWidget* parent, Qt::WFlags fl )
 : QDialog( parent, fl )
 {
@@ -168,10 +145,56 @@ void FitDialog::initFitPage()
 
     hbox0->addWidget(lblFunction);
 
-    QGridLayout *gl1 = new QGridLayout();
+	QGroupBox *gb1 = new QGroupBox(tr("Data Set"));
+    QGridLayout *gl1 = new QGridLayout(gb1);
     gl1->addWidget(new QLabel(tr("Curve")), 0, 0);
 	boxCurve = new QComboBox();
     gl1->addWidget(boxCurve, 0, 1);
+    gl1->setColumnStretch(1, 1);
+
+    gl1->addWidget(new QLabel( tr("Color")), 1, 0);
+	boxColor = new ColorButton();
+	boxColor->setColor(Qt::red);
+    gl1->addWidget(boxColor, 1, 1);
+
+    gl1->addWidget(new QLabel(tr("From x=")), 0, 2);
+
+	boxFrom = new DoubleSpinBox();
+    boxFrom->setLocale(app->locale());
+    boxFrom->setDecimals(app->d_decimal_digits);
+    connect(boxFrom, SIGNAL(valueChanged(double)), this, SLOT(updatePreview()));
+    gl1->addWidget(boxFrom, 0, 3);
+
+	gl1->addWidget(new QLabel( tr("To x=")), 1, 2);
+
+	boxTo = new DoubleSpinBox();
+    boxTo->setLocale(app->locale());
+    boxTo->setDecimals(app->d_decimal_digits);
+    connect(boxTo, SIGNAL(valueChanged(double)), this, SLOT(updatePreview()));
+    gl1->addWidget(boxTo, 1, 3);
+    gl1->setColumnStretch(3, 1);
+
+	gl1->addWidget(new QLabel(tr("Weighting")), 2, 0);
+	boxWeighting = new QComboBox();
+	boxWeighting->addItem(tr("No weighting"));
+	boxWeighting->addItem(tr("Instrumental"));
+	boxWeighting->addItem(tr("Statistical"));
+	boxWeighting->addItem(tr("Arbitrary Dataset"));
+	boxWeighting->addItem(tr("Direct Weighting"));
+    gl1->addWidget(boxWeighting, 2, 1);
+	gl1->addWidget(new QLabel(), 2, 2);
+
+	QHBoxLayout *hbox2 = new QHBoxLayout();
+
+	tableNamesBox = new QComboBox();
+	tableNamesBox->setVisible(false);
+    hbox2->addWidget(tableNamesBox);
+
+	colNamesBox = new QComboBox();
+	colNamesBox->setVisible(false);
+    hbox2->addWidget(colNamesBox);
+
+	gl1->addLayout(hbox2, 2, 3);
 
 	boxFunction = new QTextEdit();
 	boxFunction->setReadOnly(true);
@@ -182,19 +205,31 @@ void FitDialog::initFitPage()
 	boxFunction->setMaximumHeight(50);
 
     QVBoxLayout *vb = new QVBoxLayout();
-    vb->addWidget(new QLabel(tr("Initial guesses")));
     btnSaveGuesses = new QPushButton(tr( "&Save" ));
-    btnSaveGuesses->setPixmap(QPixmap(filesave_xpm));
+	btnSaveGuesses->setPixmap(QPixmap(":/filesave.png"));
     connect(btnSaveGuesses, SIGNAL(clicked()), this, SLOT(saveInitialGuesses()));
     vb->addWidget(btnSaveGuesses);
-    btnParamRange = new QPushButton();
-    btnParamRange->setIcon(QIcon(QPixmap(param_range_btn_xpm)));
+
+    btnLoadGuesses = new QPushButton(tr("Re&load" ));
+	btnLoadGuesses->setPixmap(QPixmap(":/reload.png"));
+    connect(btnLoadGuesses, SIGNAL(clicked()), this, SLOT(loadInitialGuesses()));
+    vb->addWidget(btnLoadGuesses);
+
+	btnGuess = new QPushButton(tr( "&Guess" ));
+	btnGuess->setPixmap(QPixmap(":/help.png"));
+    connect(btnGuess, SIGNAL(clicked()), this, SLOT(guessInitialValues()));
+    vb->addWidget(btnGuess);
+
+    btnParamRange = new QPushButton(tr("&Range" ));
+	btnParamRange->setIcon(QIcon(":/param_range_btn.png"));
     btnParamRange->setCheckable(true);
     connect(btnParamRange, SIGNAL(toggled(bool)), this, SLOT(showParameterRange(bool)));
     vb->addWidget(btnParamRange);
 
     vb->addStretch();
-	gl1->addLayout(vb, 3, 0);
+
+	QGroupBox *gb0 = new QGroupBox(tr("Initial guesses"));
+	QGridLayout *gl0 = new QGridLayout(gb0);
 
 	boxParams = new QTableWidget();
     boxParams->setColumnCount(6);
@@ -212,101 +247,53 @@ void FitDialog::initFitPage()
     boxParams->verticalHeader()->hide();
     boxParams->hideColumn(1);
     boxParams->hideColumn(3);
-    gl1->addWidget(boxParams, 3, 1);
 
-	gl1->addWidget(new QLabel( tr("Algorithm")), 4, 0 );
+    gl0->addWidget(boxParams, 0, 0);
+	gl0->addLayout(vb, 0, 1);
+
+	QGroupBox *gb3 = new QGroupBox(tr("Algorithm"));
+    QGridLayout *gl3 = new QGridLayout(gb3);
+
 	boxAlgorithm = new QComboBox();
 	boxAlgorithm->addItem(tr("Scaled Levenberg-Marquardt"));
 	boxAlgorithm->addItem(tr("Unscaled Levenberg-Marquardt"));
 	boxAlgorithm->addItem(tr("Nelder-Mead Simplex"));
-    gl1->addWidget(boxAlgorithm, 4, 1);
+    gl3->addWidget(boxAlgorithm, 0, 0);
 
-	gl1->addWidget(new QLabel( tr("Color")), 5, 0);
-	boxColor = new ColorBox( false );
-	boxColor->setColor(QColor(Qt::red));
-    gl1->addWidget(boxColor, 5, 1);
-
-    QGroupBox *gb1 = new QGroupBox();
-    gb1->setLayout(gl1);
-
-    QGridLayout *gl2 = new QGridLayout();
-    gl2->addWidget(new QLabel(tr("From x=")), 0, 0);
-
-	boxFrom = new DoubleSpinBox();
-    boxFrom->setLocale(app->locale());
-    boxFrom->setDecimals(app->d_decimal_digits);
-    connect(boxFrom, SIGNAL(valueChanged(double)), this, SLOT(updatePreview()));
-    gl2->addWidget(boxFrom, 0, 1);
-
-	gl2->addWidget(new QLabel( tr("To x=")), 1, 0);
-
-	boxTo = new DoubleSpinBox();
-    boxTo->setLocale(app->locale());
-    boxTo->setDecimals(app->d_decimal_digits);
-    connect(boxTo, SIGNAL(valueChanged(double)), this, SLOT(updatePreview()));
-    gl2->addWidget(boxTo, 1, 1);
-
-    QGroupBox *gb2 = new QGroupBox();
-    gb2->setLayout(gl2);
-
-    QGridLayout *gl3 = new QGridLayout();
-    gl3->addWidget(new QLabel(tr("Iterations")), 0, 0);
+    gl3->addWidget(new QLabel(tr("Iterations")), 0, 1);
 	boxPoints = new QSpinBox();
     boxPoints->setRange(10, 10000);
 	boxPoints->setSingleStep(50);
 	boxPoints->setValue(1000);
-    gl3->addWidget(boxPoints, 0, 1);
-	gl3->addWidget(new QLabel( tr("Tolerance")), 1, 0);
+    gl3->addWidget(boxPoints, 0, 2);
+    gl3->setColumnStretch(2, 1);
 
+	gl3->addWidget(new QLabel( tr("Tolerance")), 0, 3);
 	boxTolerance = new DoubleSpinBox();
 	boxTolerance->setRange(0.0, 1.0);
 	boxTolerance->setSingleStep(1e-4);
     boxTolerance->setLocale(app->locale());
     boxTolerance->setDecimals(13);
     boxTolerance->setValue(1e-4);
-	gl3->addWidget(boxTolerance, 1, 1);
-
-    QGroupBox *gb3 = new QGroupBox();
-    gb3->setLayout(gl3);
-
-    QHBoxLayout *hbox1 = new QHBoxLayout();
-    hbox1->addWidget(gb2);
-    hbox1->addWidget(gb3);
-
-    QHBoxLayout *hbox2 = new QHBoxLayout();
-	hbox2->addWidget(new QLabel(tr( "Weighting Method" )));
-	boxWeighting = new QComboBox();
-	boxWeighting->addItem(tr("No weighting"));
-	boxWeighting->addItem(tr("Instrumental"));
-	boxWeighting->addItem(tr("Statistical"));
-	boxWeighting->addItem(tr("Arbitrary Dataset"));
-	boxWeighting->addItem(tr("Direct Weighting"));
-    hbox2->addWidget(boxWeighting);
-    QGroupBox *gb4 = new QGroupBox();
-    gb4->setLayout(hbox2);
-
-	tableNamesBox = new QComboBox();
-	tableNamesBox->setEnabled(false);
-    hbox2->addWidget(tableNamesBox);
-	colNamesBox = new QComboBox();
-	colNamesBox->setEnabled(false);
-    hbox2->addWidget(colNamesBox);
+	gl3->addWidget(boxTolerance, 0, 4);
+	gl3->setColumnStretch(4, 1);
 
     QHBoxLayout *hbox3 = new QHBoxLayout();
-	buttonCancel1 = new QPushButton(tr( "&Close" ));
-	buttonCancel1->setPixmap(QPixmap(delete_xpm));
-    hbox3->addWidget(buttonCancel1);
-	hbox3->addStretch();
 	previewBox = new QCheckBox(tr("&Preview"));
 	previewBox->setChecked(false);
     connect(previewBox, SIGNAL(clicked(bool)), this, SLOT(showPreview(bool)));
     hbox3->addWidget(previewBox);
-
+	hbox3->addStretch();
 	btnDeleteFitCurves = new QPushButton(tr( "&Delete Fit Curves" ));
     hbox3->addWidget(btnDeleteFitCurves);
-    btnDeleteFitCurves->setPixmap(QPixmap(close_xpm));
+	btnDeleteFitCurves->setPixmap(QPixmap(":/close.png"));
+
+	buttonCancel1 = new QPushButton(tr( "&Close" ));
+	buttonCancel1->setPixmap(QPixmap(":/delete.png"));
+	hbox3->addWidget(buttonCancel1);
+
     buttonOk = new QPushButton(tr( "&Fit" ) );
-    buttonOk->setPixmap(QPixmap(play_xpm));
+	buttonOk->setPixmap(QPixmap(":/play.png"));
 	buttonOk->setDefault( true );
     hbox3->addWidget(buttonOk);
 
@@ -328,7 +315,8 @@ void FitDialog::initFitPage()
 	frameLayout->addSpacing (space);
 
 	buttonEdit = new QPushButton();
-	buttonEdit->setIcon(QPixmap(prev_xpm));
+	buttonEdit->setIcon(QIcon(":/prev.png"));
+	buttonEdit->setIconSize(QSize(16, 17));
 	buttonEdit->setToolTip(tr("Select Function"));
 
 	QFrame *frame2 = new QFrame();
@@ -350,7 +338,8 @@ void FitDialog::initFitPage()
 	frameLayout->addSpacing (space);
 
 	buttonAdvanced = new QPushButton();
-	buttonAdvanced->setIcon(QPixmap(next_xpm));
+	buttonAdvanced->setIcon(QIcon(":/next.png"));
+	buttonAdvanced->setIconSize(QSize(16, 17));
 	buttonAdvanced->setToolTip(tr("Custom Output"));
 
 	QFrame *frame3 = new QFrame();
@@ -369,7 +358,8 @@ void FitDialog::initFitPage()
 	frameLayout->addWidget(advancedLbl);
 	frameLayout->addSpacing (space);
 
-	QHBoxLayout *hbox4 = new QHBoxLayout();
+	QGroupBox *g = new QGroupBox();
+	QHBoxLayout *hbox4 = new QHBoxLayout(g);
 	hbox4->addStretch();
 	hbox4->addWidget(frame1);
 	hbox4->addWidget(buttonEdit);
@@ -381,23 +371,13 @@ void FitDialog::initFitPage()
     QVBoxLayout *vbox1 = new QVBoxLayout();
     vbox1->addLayout(hbox0);
     vbox1->addWidget(boxFunction);
-    vbox1->addSpacing(20);
+	vbox1->addWidget(gb0);
     vbox1->addWidget(gb1);
-    vbox1->addLayout(hbox1);
-    vbox1->addWidget(gb4);
+    vbox1->addWidget(gb3);
     vbox1->addSpacing(10);
     vbox1->addLayout(hbox3);
-    vbox1->addSpacing(10);
-
-    QFrame *frame4 = new QFrame();
-	frame4->setFrameShape(QFrame::HLine);
-	frame4->setLineWidth(10);
-	palette = this->palette();
-	palette.setColor(QPalette::WindowText, Qt::darkGray);
-	frame4->setPalette(palette);
-	vbox1->addWidget(frame4);
 	vbox1->addSpacing(10);
-    vbox1->addLayout(hbox4);
+	vbox1->addWidget(g);
 
     fitPage = new QWidget();
     fitPage->setLayout(vbox1);
@@ -412,7 +392,7 @@ void FitDialog::initFitPage()
 	connect( buttonAdvanced, SIGNAL(clicked()), this, SLOT(showAdvancedPage() ) );
     connect( tableNamesBox, SIGNAL( activated(int) ), this, SLOT( selectSrcTable(int) ) );
 
-	connect(boxColor, SIGNAL(activated(int)), this, SLOT(updatePreviewColor(int)));
+	connect(boxColor, SIGNAL(colorChanged(const QColor &)), this, SLOT(updatePreviewColor(const QColor &)));
 	setFocusProxy(boxFunction);
 }
 
@@ -474,6 +454,7 @@ void FitDialog::initEditPage()
     hbox1->addWidget(polynomOrderBox);
 
 	buttonPlugins = new QPushButton(tr( "Choose plug&ins folder..." ) );
+	buttonPlugins->setIcon(QIcon(":/folder_open.png"));
     hbox1->addWidget(buttonPlugins);
 	buttonPlugins->hide();
 
@@ -482,20 +463,21 @@ void FitDialog::initEditPage()
 	boxName = new QLineEdit(tr("user1"));
     gl2->addWidget(boxName, 0, 1);
 	btnAddFunc = new QPushButton(tr( "&Save" ));
+	btnAddFunc->setPixmap(QPixmap(":/filesave.png"));
     gl2->addWidget(btnAddFunc, 0, 2);
     gl2->addWidget(new QLabel(tr("Parameters")), 1, 0);
 	boxParam = new QLabel();
 	boxParam->setFrameStyle(QFrame::Box | QFrame::Sunken);
     gl2->addWidget(boxParam, 1, 1);
 	btnDelFunc = new QPushButton( tr( "&Remove" ));
+	btnDelFunc->setPixmap(QPixmap(":/close.png"));
     gl2->addWidget(btnDelFunc, 1, 2);
 
     QGroupBox *gb = new QGroupBox();
     gb->setLayout(gl2);
 
-	editBox = new QTextEdit();
-	editBox->setTextFormat(Qt::PlainText);
-	editBox->setAcceptRichText(false);
+	editBox = new ScriptEdit(((ApplicationWindow *)parent())->scriptingEnv());
+	editBox->enableShortcuts();
 	connect(editBox->document(), SIGNAL(contentsChange(int,int,int)), this, SLOT(guessParameters()));
 	editBox->setFocus();
 
@@ -512,7 +494,7 @@ void FitDialog::initEditPage()
     vbox1->addWidget(buttonClear);
 	vbox1->addStretch();
 	buttonCancel2 = new QPushButton(tr( "&Close" ));
-	buttonCancel2->setPixmap(QPixmap(delete_xpm));
+	buttonCancel2->setPixmap(QPixmap(":/delete.png"));
     vbox1->addWidget(buttonCancel2);
 
 	QVBoxLayout *vb = new QVBoxLayout();
@@ -542,7 +524,8 @@ void FitDialog::initEditPage()
 	frameLayout->addSpacing (space);
 
 	btnContinue = new QPushButton();
-	btnContinue->setIcon(QPixmap(next_xpm));
+	btnContinue->setIcon(QIcon(":/next.png"));
+	btnContinue->setIconSize(QSize(16, 17));
 	btnContinue->setToolTip(tr("Start Fitting Session"));
 
 	QFrame *frame2 = new QFrame();
@@ -561,7 +544,8 @@ void FitDialog::initEditPage()
 	frameLayout->addWidget(fittingSessionLbl);
 	frameLayout->addSpacing (space);
 
-	QHBoxLayout *hbox3 = new QHBoxLayout();
+	QGroupBox *g = new QGroupBox();
+	QHBoxLayout *hbox3 = new QHBoxLayout(g);
 	hbox3->addStretch();
 	hbox3->addWidget(frame1);
     hbox3->addWidget(btnContinue);
@@ -573,17 +557,7 @@ void FitDialog::initEditPage()
     vbox2->addWidget(gb);
     vbox2->addLayout(hbox2);
 	vbox2->addSpacing (10);
-
-    QFrame *frame3 = new QFrame();
-	frame3->setFrameShape(QFrame::HLine);
-	frame3->setLineWidth(5);
-	palette = this->palette();
-	palette.setColor(QPalette::WindowText, Qt::darkGray);
-	frame3->setPalette(palette);
-	vbox2->addWidget(frame3);
-
-	vbox2->addSpacing (10);
-    vbox2->addLayout(hbox3);
+	vbox2->addWidget(g);
 
     editPage = new QWidget();
     editPage->setLayout(vbox2);
@@ -602,7 +576,6 @@ void FitDialog::initEditPage()
 	connect( buttonCancel2, SIGNAL(clicked()), this, SLOT(close()));
 }
 
-
 void FitDialog::initAdvancedPage()
 {
 	ApplicationWindow *app = (ApplicationWindow *)this->parent();
@@ -695,10 +668,10 @@ void FitDialog::initAdvancedPage()
 	connect( plotLabelBox, SIGNAL(stateChanged (int)), this, SLOT(enableApplyChanges(int)));
 
     QHBoxLayout *hbox1 = new QHBoxLayout();
+	hbox1->addStretch();
 	buttonCancel3 = new QPushButton(tr( "&Close" ));
-	buttonCancel3->setPixmap(QPixmap(delete_xpm));
+	buttonCancel3->setPixmap(QPixmap(":/delete.png"));
     hbox1->addWidget(buttonCancel3);
-    hbox1->addStretch();
 	btnApply = new QPushButton(tr( "&Apply" ));
 	btnApply->setEnabled(false);
 	connect( btnApply, SIGNAL(clicked()), this, SLOT(applyChanges()));
@@ -737,7 +710,8 @@ void FitDialog::initAdvancedPage()
 
 	btnBack = new QPushButton();
 	connect( btnBack, SIGNAL(clicked()), this, SLOT(returnToFitPage()));
-	btnBack->setIcon(QPixmap(prev_xpm));
+	btnBack->setIcon(QIcon(":/prev.png"));
+	btnBack->setIconSize(QSize(16, 17));
 	btnBack->setToolTip(tr("Fitting Session"));
 
 	QFrame *frame2 = new QFrame();
@@ -758,24 +732,15 @@ void FitDialog::initAdvancedPage()
 	frameLayout->addWidget(fittingSessionLbl);
 	frameLayout->addSpacing (space);
 
-	QHBoxLayout *hbox2 = new QHBoxLayout();
+	QGroupBox *g = new QGroupBox();
+	QHBoxLayout *hbox2 = new QHBoxLayout(g);
 	hbox2->addWidget(frame1);
 	hbox2->addWidget(btnBack);
 	hbox2->addWidget(frame2);
 	hbox2->addStretch();
 
 	vbox1->addSpacing (10);
-
-	QFrame *frame3 = new QFrame();
-	frame3->setFrameShape(QFrame::HLine);
-	frame3->setLineWidth(10);
-	palette = this->palette();
-	palette.setColor(QPalette::WindowText, Qt::darkGray);
-	frame3->setPalette(palette);
-	vbox1->addWidget(frame3);
-
-	vbox1->addSpacing (10);
-	vbox1->addLayout(hbox2);
+	vbox1->addWidget(g);
 
     advancedPage = new QWidget();
 	advancedPage->setLayout(vbox1);
@@ -869,12 +834,11 @@ void FitDialog::setGraph(Graph *g)
 	boxCurve->clear();
 	boxCurve->addItems(d_graph->analysableCurvesList());
 
-    QString selectedCurve = g->selectedCurveTitle();
-	if (!selectedCurve.isEmpty()){
-	    int index = boxCurve->findText (selectedCurve);
-		boxCurve->setCurrentItem(index);
-	}
-    activateCurve(boxCurve->currentText());
+	QString selectedCurve = g->selectedCurveTitle();
+	if (!selectedCurve.isEmpty())
+		boxCurve->setCurrentIndex(boxCurve->findText (selectedCurve));
+
+	activateCurve(boxCurve->currentText());
 
 	connect (d_graph, SIGNAL(closedGraph()), this, SLOT(close()));
 	connect (d_graph, SIGNAL(dataRangeChanged()), this, SLOT(changeDataRange()));
@@ -942,7 +906,7 @@ void FitDialog::saveUserFunction()
 		QString filter = tr("QtiPlot fit model")+" (*.fit);;";
 		filter += tr("All files")+" (*.*)";
 		QString fn = ApplicationWindow::getFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
-                                app->fitModelsPath + "/" + name, filter);
+								app->fitModelsPath + "/" + name, filter, 0, true, app->d_confirm_overwrite);
 		if (!fn.isEmpty()){
             QFileInfo fi(fn);
             app->fitModelsPath = fi.dir().absolutePath();
@@ -1031,11 +995,13 @@ void FitDialog::showFitPage()
         boxAlgorithm->setEnabled(false);
 		boxPoints->setEnabled(false);
 		boxTolerance->setEnabled(false);
+		btnGuess->setVisible(false);
     } else {
         btnParamRange->setEnabled(true);
         boxAlgorithm->setEnabled(true);
 		boxPoints->setEnabled(true);
 		boxTolerance->setEnabled(true);
+		btnGuess->setVisible(d_current_fit->type() == Fit::BuiltIn);
     }
 
 	QStringList paramList = d_current_fit->parameterNames();
@@ -1111,7 +1077,7 @@ void FitDialog::showFitPage()
 	boxFunction->setText(formula);
 	lblFunction->setText(boxName->text() +" (x, " + boxParam->text().simplified() + ")");
 
-	tw->setCurrentWidget (fitPage);
+	tw->setCurrentWidget(fitPage);
 	if (previewBox->isChecked())
 		updatePreview();
 }
@@ -1415,11 +1381,12 @@ void FitDialog::accept()
 	QStringList parameters = QStringList();
 	MyParser parser;
 	bool error = false;
-#ifdef Q_CC_MSVC
-    QVarLengthArray<double> paramsInit(n), paramRangeLeft(n), paramRangeRight(n);
-#else
-	double paramsInit[n], paramRangeLeft[n], paramRangeRight[n];
-#endif
+	double *paramsInit = (double *)malloc(n*sizeof(double));
+	double *paramRangeLeft = (double *)malloc(n*sizeof(double));
+	double *paramRangeRight = (double *)malloc(n*sizeof(double));
+	if (!paramsInit || !paramRangeLeft || !paramRangeRight)
+		return;
+
 	QString formula = boxFunction->text();
 	NonLinearFit *nlf = qobject_cast<NonLinearFit *>(d_current_fit);
 	if (nlf)
@@ -1469,22 +1436,14 @@ void FitDialog::accept()
 
 	if (!error){
 		if (d_current_fit->type() == Fit::BuiltIn)
-#ifdef Q_CC_MSVC
-			modifyGuesses (paramsInit.data());
-#else
 			modifyGuesses (paramsInit);
-#endif
 
 		if (nlf){
 			if (!nlf->setParametersList(parameters)) return;
 			if (!nlf->setFormula(formula, false)) return;
 		}
 
-#ifdef Q_CC_MSVC
-		d_current_fit->setInitialGuesses(paramsInit.data());
-#else
 		d_current_fit->setInitialGuesses(paramsInit);
-#endif
 
 		if (!d_current_fit->setDataFromCurve(curve, start, end) ||
 			!d_current_fit->setWeightingData ((Fit::WeightingMethod)boxWeighting->currentIndex(),
@@ -1498,7 +1457,7 @@ void FitDialog::accept()
 		d_current_fit->setTolerance(eps);
 		d_current_fit->setOutputPrecision(app->fit_output_precision);
 		d_current_fit->setAlgorithm((Fit::Algorithm)boxAlgorithm->currentIndex());
-		d_current_fit->setColor(boxColor->currentIndex());
+		d_current_fit->setColor(boxColor->color());
 		d_current_fit->generateFunction(generatePointsBtn->isChecked(), generatePointsBox->value());
 		d_current_fit->setMaximumIterations(boxPoints->value());
 		if (!d_current_fit->isA("PolynomialFit") && !d_current_fit->isA("LinearFit") && !d_current_fit->isA("LinearSlopeFit"))
@@ -1529,6 +1488,10 @@ void FitDialog::accept()
 
 		if (previewBox->isChecked())
 			updatePreview();
+
+		free (paramsInit);
+		free (paramRangeLeft);
+		free (paramRangeRight);
 	}
 }
 
@@ -1583,13 +1546,10 @@ void FitDialog::selectSrcTable(int tabnr)
 
 void FitDialog::enableWeightingParameters(int index)
 {
-	if (index == Fit::Dataset || index == Fit::Direct){
-		tableNamesBox->setEnabled(true);
-		colNamesBox->setEnabled(true);
-	} else {
-		tableNamesBox->setEnabled(false);
-		colNamesBox->setEnabled(false);
-	}
+	bool on = (index == Fit::Dataset || index == Fit::Direct);
+
+	tableNamesBox->setVisible(on);
+	colNamesBox->setVisible(on);
 }
 
 void FitDialog::closeEvent (QCloseEvent * e)
@@ -1754,7 +1714,7 @@ void FitDialog::saveInitialGuesses()
 		QString filter = tr("QtiPlot fit model") + " (*.fit);;";
 		filter += tr("All files") + " (*.*)";
 		QString fn = ApplicationWindow::getFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
-                                app->fitModelsPath + "/" + d_current_fit->objectName(), filter);
+								app->fitModelsPath + "/" + d_current_fit->objectName(), filter, 0, true, app->d_confirm_overwrite);
 		if (!fn.isEmpty()){
             QFileInfo fi(fn);
             QString baseName = fi.fileName();
@@ -1767,6 +1727,42 @@ void FitDialog::saveInitialGuesses()
     }
 }
 
+void FitDialog::loadInitialGuesses()
+{
+	if (!d_current_fit)
+		return;
+	if (!d_current_fit->load(d_current_fit->fileName()))
+		return;
+
+	for (int i = 0; i < d_current_fit->numParameters(); i++){
+		((DoubleSpinBox*)boxParams->cellWidget(i, 2))->setValue(d_current_fit->initialGuess(i));
+		boxParams->item(i, boxParams->columnCount() - 1)->setText("--");
+	}
+
+	if (previewBox->isChecked())
+		updatePreview();
+}
+
+void FitDialog::guessInitialValues()
+{
+	if (!d_current_fit)
+		return;
+
+	if (d_current_fit->type() == Fit::BuiltIn){
+		d_current_fit->setDataFromCurve(boxCurve->currentText(), boxFrom->value(), boxTo->value());
+		d_current_fit->guessInitialValues();
+		//modifyGuesses (paramsInit);
+	}
+
+	for (int i = 0; i < d_current_fit->numParameters(); i++){
+		((DoubleSpinBox*)boxParams->cellWidget(i, 2))->setValue(d_current_fit->initialGuess(i));
+		boxParams->item(i, boxParams->columnCount() - 1)->setText("--");
+	}
+
+	if (previewBox->isChecked())
+		updatePreview();
+}
+
 QStringList FitDialog::plugInNames()
 {
 	QStringList lst;
@@ -1800,7 +1796,7 @@ void FitDialog::updatePreview()
 		d_preview_curve = new FunctionCurve();
 		d_preview_curve->attach(d_graph);
 		d_preview_curve->setRenderHint(QwtPlotItem::RenderAntialiased, d_graph->antialiasing());
-		d_preview_curve->setPen(QPen(ColorBox::color(boxColor->currentIndex()), 1));
+		d_preview_curve->setPen(QPen(boxColor->color(), 1));
 	}
 
 	bool changedVar = false;
@@ -1844,12 +1840,12 @@ void FitDialog::updatePreview()
 	d_graph->replot();
 }
 
-void FitDialog::updatePreviewColor(int colorIndex)
+void FitDialog::updatePreviewColor(const QColor &c)
 {
     if (!d_preview_curve)
 		return;
 
-    d_preview_curve->setPen(QPen(ColorBox::color(colorIndex), 1));
+	d_preview_curve->setPen(QPen(c, 1));
 	d_graph->replot();
 }
 
