@@ -27,9 +27,9 @@
  *                                                                         *
  ***************************************************************************/
 #include "InterpolationDialog.h"
-#include "../Interpolation.h"
-#include "../../plot2D/Graph.h"
-#include <ColorBox.h>
+#include <Interpolation.h>
+#include <Graph.h>
+#include <ColorButton.h>
 #include <DoubleSpinBox.h>
 
 #include <QGroupBox>
@@ -64,7 +64,7 @@ InterpolationDialog::InterpolationDialog( QWidget* parent, Qt::WFlags fl )
 
 	gl1->addWidget(new QLabel(tr("Points")), 2, 0);
 	boxPoints = new QSpinBox();
-	boxPoints->setRange(3,100000);
+	boxPoints->setRange(3, 100000);
 	boxPoints->setSingleStep(10);
 	boxPoints->setValue(1000);
 	gl1->addWidget(boxPoints, 2, 1);
@@ -85,8 +85,8 @@ InterpolationDialog::InterpolationDialog( QWidget* parent, Qt::WFlags fl )
 
 	gl1->addWidget(new QLabel(tr("Color")), 5, 0);
 
-	boxColor = new ColorBox(false);
-	boxColor->setColor(QColor(Qt::red));
+	boxColor = new ColorButton();
+	boxColor->setColor(Qt::red);
 	gl1->addWidget(boxColor, 5, 1);
 	gl1->setRowStretch(6, 1);
 	gl1->setColumnStretch(1, 1);
@@ -106,7 +106,7 @@ InterpolationDialog::InterpolationDialog( QWidget* parent, Qt::WFlags fl )
 
 	connect( boxName, SIGNAL(activated(const QString&)), this, SLOT( activateCurve(const QString&)));
 	connect( buttonFit, SIGNAL( clicked() ), this, SLOT( interpolate() ) );
-    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
+	connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( close() ) );
 }
 
 void InterpolationDialog::interpolate()
@@ -129,35 +129,34 @@ void InterpolationDialog::interpolate()
 		return;
 	}
 
-	Interpolation *i = new Interpolation((ApplicationWindow *)this->parent(), graph, curve,
-                                      from, to, boxMethod->currentIndex());
+	Interpolation *i = new Interpolation((ApplicationWindow *)this->parent(), graph, curve, from, to, boxMethod->currentIndex());
 	i->setOutputPoints(boxPoints->value());
-	i->setColor(boxColor->currentIndex());
+	i->setColor(boxColor->color());
 	i->run();
 	delete i;
 }
 
 void InterpolationDialog::setGraph(Graph *g)
 {
+	if (!g)
+		return;
+
 	graph = g;
 	boxName->addItems(g->analysableCurvesList());
 
-    QString selectedCurve = g->selectedCurveTitle();
+	QString selectedCurve = g->selectedCurveTitle();
 	if (!selectedCurve.isEmpty())
-	{
-	    int index = boxName->findText (selectedCurve);
-		boxName->setCurrentItem(index);
-	}
+		boxName->setCurrentIndex(boxName->findText(selectedCurve));
 
-    activateCurve(boxName->currentText());
+	activateCurve(boxName->currentText());
 
-	connect (graph, SIGNAL(closedGraph()), this, SLOT(close()));
+	connect (graph, SIGNAL(destroyed()), this, SLOT(close()));
 	connect (graph, SIGNAL(dataRangeChanged()), this, SLOT(changeDataRange()));
 };
 
 void InterpolationDialog::activateCurve(const QString& curveName)
 {
-	QwtPlotCurve *c = graph->curve(curveName);
+	PlotCurve *c = graph->curve(curveName);
 	if (!c)
 		return;
 
