@@ -28,10 +28,11 @@
  *                                                                         *
  ***************************************************************************/
 #include "SmoothCurveDialog.h"
-#include "../SmoothFilter.h"
+#include "SmoothFilter.h"
 #include <Graph.h>
+#include <PlotCurve.h>
 #include <MyParser.h>
-#include <ColorBox.h>
+#include <ColorButton.h>
 
 #include <QGroupBox>
 #include <QSpinBox>
@@ -49,6 +50,7 @@ SmoothCurveDialog::SmoothCurveDialog(int method, QWidget* parent, Qt::WFlags fl
     setObjectName( "SmoothCurveDialog" );
 	setWindowTitle(tr("QtiPlot - Smoothing Options"));
 	setSizeGripEnabled( true );
+	setAttribute(Qt::WA_DeleteOnClose);
 
     QGroupBox *gb1 = new QGroupBox();
     QGridLayout *gl1 = new QGridLayout(gb1);
@@ -57,8 +59,8 @@ SmoothCurveDialog::SmoothCurveDialog(int method, QWidget* parent, Qt::WFlags fl
     boxName = new QComboBox();
 	gl1->addWidget(boxName, 0, 1);
 
-	boxColor = new ColorBox();
-	boxColor->setColor(QColor(Qt::red));
+	boxColor = new ColorButton();
+	boxColor->setColor(Qt::red);
 
 	if (method == SmoothFilter::SavitzkyGolay)
 		{
@@ -84,8 +86,7 @@ SmoothCurveDialog::SmoothCurveDialog(int method, QWidget* parent, Qt::WFlags fl
 		gl1->addWidget(boxColor, 4, 1);
         gl1->setRowStretch(5, 1);
 		}
-	else
-	if (method == SmoothFilter::Lowess)
+	else if (method == SmoothFilter::Lowess)
 		{
         gl1->addWidget(new QLabel(tr("f")), 1, 0);
 		boxF = new QDoubleSpinBox();
@@ -133,7 +134,7 @@ SmoothCurveDialog::SmoothCurveDialog(int method, QWidget* parent, Qt::WFlags fl
     hb->addLayout(vl);
 
 	connect( btnSmooth, SIGNAL(clicked()), this, SLOT( smooth()));
-    connect( buttonCancel, SIGNAL(clicked()), this, SLOT( reject()));
+	connect( buttonCancel, SIGNAL(clicked()), this, SLOT( close()));
 	connect( boxName, SIGNAL(activated(const QString&)), this, SLOT(activateCurve(const QString&)));
 }
 
@@ -154,26 +155,28 @@ void SmoothCurveDialog::smooth()
     else
         sf->setSmoothPoints(boxPointsLeft->value());
 
-    sf->setColor(boxColor->currentIndex());
-    sf->run();
-    delete sf;
+	sf->setColor(boxColor->color());
+	sf->run();
+	delete sf;
 }
 
 void SmoothCurveDialog::setGraph(Graph *g)
 {
-    graph = g;
-    boxName->addItems (g->analysableCurvesList());
-    activateCurve(boxName->currentText());
+	if (!g)
+		return;
+
+	graph = g;
+	boxName->addItems (g->analysableCurvesList());
+	activateCurve(boxName->currentText());
 }
 
 void SmoothCurveDialog::activateCurve(const QString& curveName)
 {
-    if (smooth_method == SmoothFilter::Average)
-	{
-	QwtPlotCurve *c = graph->curve(curveName);
-	if (!c || c->rtti() != QwtPlotItem::Rtti_PlotCurve)
-		return;
+	if (smooth_method == SmoothFilter::Average){
+		PlotCurve *c = graph->curve(curveName);
+		if (!c || c->rtti() != QwtPlotItem::Rtti_PlotCurve)
+			return;
 
-	boxPointsLeft->setMaxValue(c->dataSize()/2);
+		boxPointsLeft->setMaxValue(c->dataSize()/2);
 	}
 }
