@@ -10,8 +10,6 @@ double **matrix(long nrl, long nrh, long ncl, long nch);
 void free_matrix(double **m, long nrl, long nrh, long ncl, long nch);
 double *vector(long nl, long nh);
 void free_vector(double *v, long nl, long nh);
-size_t *ivector(long nl, long nh);
-void free_ivector(size_t *v, long nl, long nh);
 int *intvector(long nl, long nh);
 void free_intvector(int*v, long nl, long nh);
 
