@@ -2,11 +2,9 @@
     File                 : ApplicationWindow.h
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief,
-                           Tilman Hoener zu Siederdissen,
-                           Knut Franke
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net,
-                           knut.franke*gmx.de
+	Copyright            : (C) 2004 - 2010 by Ion Vasilief,
+						   (C) 2006 - june 2007 Tilman Hoener zu Siederdissen, Knut Franke
+	Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : QtiPlot's main window
 
  ***************************************************************************/
@@ -40,7 +38,9 @@
 #include <QDesktopServices>
 #include <QBuffer>
 #include <QLineEdit>
+#include <QMessageBox>
 
+#include <MultiLayer.h>
 #include <Graph.h>
 #include <Table.h>
 #include <ScriptingEnv.h>
@@ -49,6 +49,20 @@
 
 #include <qwt_color_map.h>
 
+#ifdef BROWSER_PLUGIN
+#include <qtbrowserplugin.h>
+#endif
+
+#ifdef QAXSERVER
+#include <ActiveQt/QAxBindable>
+#include <ActiveQt/QAxFactory>
+#include <qt_windows.h>
+#endif
+
+#ifdef HAS_EXCEL
+	#include <QAxObject>
+#endif
+
 class QPixmap;
 class QCloseEvent;
 class QDropEvent;
@@ -74,7 +88,6 @@ class Table;
 class ScalePicker;
 class Graph3D;
 class Note;
-class MultiLayer;
 class FunctionDialog;
 class Folder;
 class FolderListItem;
@@ -87,6 +100,8 @@ class LegendWidget;
 class ArrowMarker;
 class TextEditor;
 class AssociationsDialog;
+class ScriptEdit;
+class ExportDialog;
 
 /**
  * \brief QtiPlot's main window.
@@ -116,10 +131,25 @@ class AssociationsDialog;
  * in ApplicationWindow after the above reorganizations. Think about whether a Model/View approach can be
  * used for Project/ProjectExplorer.
  */
+
 class ApplicationWindow: public QMainWindow, public scripted
+#ifdef BROWSER_PLUGIN
+	, public QtNPBindable
+#endif
+#ifdef QAXSERVER
+	, public QAxBindable
+#endif
 {
     Q_OBJECT
 
+#ifdef BROWSER_PLUGIN
+    Q_CLASSINFO("ClassID", "{2e5b2715-46b2-4831-ba9b-6a3b195d5ec8}")
+    Q_CLASSINFO("InterfaceID", "{94581136-3c0c-46cc-97a1-066061356d43}")
+    Q_CLASSINFO("EventsID", "{8c191b77-1894-45c7-9d6b-201dede95410}")
+
+    Q_CLASSINFO("MIME", "application/x-qtiplot")
+#endif
+
 public:
     ApplicationWindow(bool factorySettings = false);
 	ApplicationWindow(const QStringList& l);
@@ -187,7 +217,7 @@ public:
 	static QString imageFilter();
 	static QString getFileName(QWidget *parent = 0, const QString & caption = QString(),
 		const QString & dir = QString(), const QString & filter = QString(),
-		QString * selectedFilter = 0, bool save = true);
+		QString * selectedFilter = 0, bool save = true, bool confirmOverwrite = true);
 
 	static void memoryAllocationError();
 	QColor readColorFromProject(const QString& name);
@@ -196,6 +226,18 @@ public:
 	bool isMdiAreaEnabled(){return d_mdi_windows_area;};
 
 	void displayInfo(const QString& text){info->setText(text);};
+	QLineEdit *displayInfoLineEdit(){return info;};
+
+	QList<QColor> indexedColors(){return d_indexed_colors;};
+	void setIndexedColors(const QList<QColor>& lst){d_indexed_colors = lst;};
+
+	QList<int> indexedSymbols(){return d_symbols_list;};
+	void setIndexedSymbols(const QList<int>& lst){d_symbols_list = lst;};
+
+	QStringList indexedColorNames(){return d_indexed_color_names;};
+	void setIndexedColorNames(const QStringList& lst){d_indexed_color_names = lst;};
+
+	void connectScriptEditor(ScriptEdit *);
 
 public slots:
 	//! \name Projects and Project Files
@@ -203,7 +245,10 @@ public slots:
 	void open();
 	ApplicationWindow* open(const QString& fn, bool factorySettings = false, bool newProject = true);
 	ApplicationWindow* openProject(const QString& fn, bool factorySettings = false, bool newProject = true);
+#ifdef OPJ_IMPORT
 	ApplicationWindow* importOPJ(const QString& fn, bool factorySettings = false, bool newProject = true);
+#endif
+	void closeProject();
 
 	/**
 	 * \brief Create a new project from a data file.
@@ -220,7 +265,7 @@ public slots:
 	 * \param fn is read as a Python script file and loaded in the command script window.
 	 * \param execute specifies if the script should be executed after opening.
 	 */
-	ApplicationWindow * loadScript(const QString& fn, bool execute = false, bool noGui = false, bool factorySettings = false);
+	ApplicationWindow * loadScript(const QString& fn, bool execute = false, bool noGui = false);
 
 	QList<MdiSubWindow *> windowsList();
 	void updateWindowLists(MdiSubWindow *w);
@@ -231,6 +276,8 @@ public slots:
 
 	void saveProjectAs(const QString& fileName = QString(), bool compress = false);
 	bool saveProject(bool compress = false);
+	void saveWindowAs(const QString& fileName = QString(), bool compress = false);
+	bool saveWindow(MdiSubWindow *w, const QString& fileName = QString(), bool compress = false);
 
 	//! Set the project status to modifed
 	void modifiedProject();
@@ -257,12 +304,10 @@ public slots:
 	//@{
 	//! Creates a new empty multilayer plot
 	MultiLayer* newGraph(const QString& caption = tr("Graph"));
-	MultiLayer* multilayerPlot(int c, int r, int style);
+	MultiLayer* multilayerPlot(int c, int r, int style, const MultiLayer::AlignPolicy& align = MultiLayer::AlignLayers);
 	MultiLayer* multilayerPlot(Table* w, const QStringList& colList, int style, int startRow = 0, int endRow = -1);
 	//! used when restoring a plot from a project file
 	MultiLayer* multilayerPlot(const QString& caption, int layers = 1, int rows = 1, int cols = 1);
-	//! used by the plot wizard
-	MultiLayer* multilayerPlot(const QStringList& colList);
 	MultiLayer* waterfallPlot();
 	MultiLayer* waterfallPlot(Table *t, const QStringList& list);
 	void connectMultilayerPlot(MultiLayer *g);
@@ -279,6 +324,7 @@ public slots:
   	MultiLayer* plotContour(Matrix *m = 0);
   	MultiLayer* plotColorMap(Matrix *m = 0);
   	MultiLayer* plotImage(Matrix *m = 0);
+  	MultiLayer* plotImageProfiles(Matrix *m = 0);
 
 	//! Rearrange the layersin order to fit to the size of the plot window
   	void autoArrangeLayers();
@@ -288,6 +334,13 @@ public slots:
 	void plot4Layers();
 	void plotStackedLayers();
 	void plotStackedHistograms();
+
+	void plotStackSharedAxisLayers();
+	void plotVerticalSharedAxisLayers();
+	void plotHorizontalSharedAxisLayers();
+	void plotSharedAxesLayers();
+	void plotCustomLayout(bool sharedAxes = false);
+	void plotCustomLayoutSharedAxes(){plotCustomLayout(true);};
 	//@}
 
 	//! \name 3D Data Plots
@@ -338,13 +391,9 @@ public slots:
 	MultiLayer * newFunctionPlot(QStringList &formulas, double start, double end, int points = 100, const QString& var = "x", int type = 0);
 
 	FunctionDialog* functionDialog();
-	FunctionDialog* showFunctionDialog();
 	FunctionDialog* showFunctionDialog(Graph * g, int curve);
 	void addFunctionCurve();
-	void clearSurfaceFunctionsList();
 	void clearLogInfo();
-	void clearParamFunctionsList();
-	void clearPolarFunctionsList();
 	void updateFunctionLists(int type, QStringList &formulas);
 	void updateSurfaceFuncList(const QString& s);
 	//@}
@@ -394,7 +443,6 @@ public slots:
 	//! Used when loading a table from a project file
 	Table* newTable(const QString& caption,int r, int c);
 	Table* newTable(int r, int c, const QString& name = QString(),const QString& legend = QString());
-	Table* newTable(const QString& caption, int r, int c, const QString& text);
 	/**
 	 * \brief Create a Table which is initially hidden; used to return the result of an analysis operation.
 	 *
@@ -419,12 +467,26 @@ public slots:
 	void connectTable(Table* w);
 	void initTable(Table* w, const QString& caption);
 	void customTable(Table* w);
+	Table* importOdfSpreadsheet(const QString& = QString::null, int sheet = -1);
 
-	void importWaveFile();
+	Table* importExcel(const QString& = QString::null, int sheet = -1);
+#ifdef HAS_EXCEL
+	static bool isExcelInstalled();
+	Table* importUsingExcel(const QString& = QString::null, int sheet = -1);
+	void importExcelCharts(QAxObject* ws, const QString& fn = QString::null);
+	MultiLayer *importExcelChart(QAxObject* chart, const QString& fn = QString::null, const QString& name = QString::null);
+#endif
+#ifdef XLS_IMPORT
+	Table* importExcelCrossplatform(const QString& = QString::null, int sheet = -1);
+	void exportExcel();
+#endif
+
+	Table* importWaveFile();
 	void importASCII();
 	void importASCII(const QStringList& files, int import_mode, const QString& local_column_separator, int local_ignored_lines, bool local_rename_columns,
         bool local_strip_spaces, bool local_simplify_spaces, bool local_import_comments,
-        QLocale local_separators, const QString& local_comment_string, bool import_read_only, int endLineChar);
+		QLocale local_separators, const QString& local_comment_string, bool import_read_only, int endLineChar,
+		const QList<int>& colTypes = QList<int>(), const QStringList& colFormats = QStringList());
 	void exportAllTables(const QString& dir, const QString& filter, const QString& sep, bool colNames, bool colComments, bool expSelection);
 
 	//! recalculate selected cells of current table
@@ -490,6 +552,7 @@ public slots:
 	void exportPDF();
 	void print();
 	void printPreview();
+	void setPrintPreviewOptions(QPrinter *);
 	void printAllPlots();
 	//@}
 
@@ -531,7 +594,7 @@ public slots:
 	//@}
 
 	//! Show about dialog
-	static void about();
+	static QMessageBox* about(bool dialog = true);
 	//! Return a version string ("QtiPlot x.y.z")
 	static QString versionString();
 	void removeCurves(const QString& name);
@@ -564,7 +627,7 @@ public slots:
 	void changeMatrixName(const QString& oldName, const QString& newName);
 	void updateCurves(Table *t, const QString& name);
 
-	void showTable(const QString& curve);
+	void showTable(Table *, const QString& curve);
 	void showTable(int i);
 
 	void addColToTable();
@@ -607,6 +670,9 @@ public slots:
 
 	//! \name Plot Tools
 	//@{
+	void graphSelectionChanged(SelectionMoveResizer *);
+	void lowerActiveEnrichment();
+	void raiseActiveEnrichment(bool = true);
 	void newLegend();
 	void addTimeStamp();
 	void drawLine();
@@ -619,7 +685,7 @@ public slots:
 	void addImage();
 	void zoomIn();
 	void zoomOut();
-	void magnify();
+	void magnify(int mode = 0);
 	void setAutoScale();
 	void showRangeSelectors();
 	void showCursor();
@@ -685,7 +751,7 @@ public slots:
 	void showEnrichementDialog();
 	void showLineDialog();
 	void showTitleDialog();
-	void showExportASCIIDialog();
+	ExportDialog* showExportASCIIDialog();
 	void showCurvesDialog();
 	void showCurveRangeDialog();
 	CurveRangeDialog* showCurveRangeDialog(Graph *g, int curve);
@@ -698,6 +764,7 @@ public slots:
 	void showColsDialog();
 	void showColMenu(int c);
 	void showColumnValuesDialog();
+	void showExtractDataDialog();
 
 	void showGraphContextMenu();
 	void showTableContextMenu(bool selection);
@@ -753,6 +820,9 @@ public slots:
 	void showColorMapDialog();
 	//@}
 
+	void baselineDialog();
+	void subtractReferenceData();
+	void subtractStraightLine();
 	void translateCurveHor();
 	void translateCurve(TranslateCurveTool::Direction direction = TranslateCurveTool::Vertical);
 
@@ -819,7 +889,7 @@ public slots:
 	void custom3DGrids(int grids);
 	//@}
 
-	void updateRecentProjectsList();
+	void updateRecentProjectsList(const QString& fn = QString::null);
 
 	//!  connected to the done(bool) signal of the http object
 	void receivedVersionFile(bool error);
@@ -880,6 +950,8 @@ public slots:
 	void execute();
 	void executeAll();
 	void evaluate();
+	void commentSelection();
+	void uncommentSelection();
 	//@}
 
 	//! \name Folders
@@ -1026,8 +1098,8 @@ private:
 	//! \name Initialization
 	//@{
 	void init(bool factorySettings = false);
-    void initCompleter();
-    void initGlobalConstants();
+	void initCompleter();
+	void initGlobalConstants();
 	void createActions();
 	void initMainMenu();
 	void initToolBars();
@@ -1038,6 +1110,9 @@ private:
 	//@}
 	virtual QMenu * createPopupMenu(){return NULL;};
 	void updateCompleter(const QString& windowName, bool remove = false, const QString& newName = QString::null);
+	QMessageBox::StandardButton showSaveProjectMessage();
+	void goToParentFolder();
+	bool isProjectFile(const QString& fn);
 
 private slots:
 	void disableActions();
@@ -1047,6 +1122,7 @@ private slots:
 	void customMenu(QMdiSubWindow* w);
 	void windowActivated(QMdiSubWindow *w);
 	void custom2DPlotTools(MultiLayer *);
+	void updateExplorerWindowLayout(Qt::DockWidgetArea);
 
 	void analysisMenuAboutToShow();
 	void scriptingMenuAboutToShow();
@@ -1090,7 +1166,14 @@ private slots:
 
 // TODO: a lot of this stuff should be private
 public:
+	bool d_force_muParser;
+	bool d_keep_aspect_ration;
+	QPrinter::PaperSize d_print_paper_size;
+	QPrinter::Orientation d_printer_orientation;
+	bool d_muparser_c_locale;
 	Graph::LegendDisplayMode d_graph_legend_display;
+	Graph::AxisTitlePolicy d_graph_axis_labeling;
+	bool d_synchronize_graph_scales;
 	int d_latex_compiler;
 	QString d_latex_compiler_path;
 	//! Last selected filter in open project dialog
@@ -1099,6 +1182,7 @@ public:
 	int d_frame_geometry_unit;
 	//! Default geometry unit to be displayed in the PlotDialog.
 	int d_layer_geometry_unit;
+	int d_layer_canvas_width, d_layer_canvas_height;
     int d_notes_tab_length;
     QFont d_notes_font;
 	//! End of line convention used for copy/paste operations and when exporting tables/matrices to ASCII files.
@@ -1147,6 +1231,7 @@ public:
 	bool d_ASCII_import_comments, d_ASCII_import_read_only, d_ASCII_import_preview;
 	int d_ASCII_import_mode, d_preview_lines;
 	int d_ASCII_import_first_row_role;
+	QSize d_import_ASCII_dialog_size;
 	//! Specifies if only the Tables/Matrices in the current folder should be displayed in the Add/remove curve dialog.
 	bool d_show_current_folder;
 	bool d_scale_plots_on_print, d_print_cropmarks;
@@ -1157,7 +1242,8 @@ public:
 	bool d_extended_open_dialog;
 	bool generateUniformFitPoints;
 	bool generatePeakCurves;
-	int peakCurvesColor;
+	QColor peakCurvesColor;
+	bool d_multi_peak_messages;
 	//! User defined size for the Add/Remove curves dialog
 	QSize d_add_curves_dialog_size;
 
@@ -1200,7 +1286,7 @@ public:
 	QString templatesDir;
 	bool autoScaleFonts, autoResizeLayers, autoSearchUpdates;
 	bool confirmCloseTable, confirmCloseMatrix, confirmClosePlot2D, confirmClosePlot3D;
-	bool confirmCloseFolder, confirmCloseNotes;
+	bool confirmCloseFolder, confirmCloseNotes, d_confirm_overwrite;
 	bool titleOn, autoSave, autoscale2DPlots, antialiasing2DPlots;
 	//! \name User preferences for axes in new 2D plot layers
 	//@{
@@ -1210,6 +1296,8 @@ public:
 	QVector<bool> d_show_axes_labels;
 	//! Draw axes backbones
 	bool drawBackbones;
+	//! Default distance between the ticks and the tick labels
+	int d_graph_tick_labels_dist;
 	//! Distance between the axes title and the backbone
 	int d_graph_axes_labels_dist;
 	//! Default axes linewidth
@@ -1217,9 +1305,10 @@ public:
 	//@}
 	int majTicksStyle, minTicksStyle, legendFrameStyle, autoSaveTime, canvasFrameWidth;
 	QColor legendBackground, legendTextColor, defaultArrowColor;
-	int defaultArrowHeadLength, defaultArrowHeadAngle, d_legend_default_angle;
-	double defaultArrowLineWidth, defaultCurveLineWidth;
-	bool defaultArrowHeadFill;
+	int defaultArrowHeadLength, defaultArrowHeadAngle, d_legend_default_angle, d_symbol_style;
+	double defaultArrowLineWidth, defaultCurveLineWidth, defaultSymbolEdge;
+	bool defaultArrowHeadFill, d_fill_symbols, d_indexed_symbols;
+	int defaultCurveBrush, defaultCurveAlpha, d_curve_line_style;
 	Qt::PenStyle defaultArrowLineStyle;
 	QPen d_frame_widget_pen;
 	int majTicksLength, minTicksLength, defaultPlotMargin;
@@ -1234,7 +1323,7 @@ public:
 	QStringList recentProjects;
 	bool saved;
 	QStringList locales;
-	QStringList functions; //user-defined functions;
+	QStringList d_recent_functions; //user-defined functions;
 	QStringList xFunctions, yFunctions, rFunctions, thetaFunctions; // user functions for parametric and polar plots
 	QStringList surfaceFunc; //user-defined surface functions;
     QStringList d_param_surface_func; //user-defined parametric surface functions;
@@ -1323,11 +1412,17 @@ private:
 	QMenu *tableMenu, *fillMenu, *normMenu, *newMenu, *exportPlotMenu, *smoothMenu, *filterMenu, *decayMenu, *importMenu;
 
 	QAction *actionEditCurveRange, *actionCurveFullRange, *actionShowAllCurves, *actionHideCurve, *actionHideOtherCurves;
-	QAction *actionEditFunction, *actionRemoveCurve, *actionShowCurveWorksheet, *actionShowCurvePlotDialog;
+	QAction *actionRemoveCurve, *actionShowCurveWorksheet, *actionShowCurvePlotDialog;
     QAction *actionNewProject, *actionAppendProject, *actionNewNote, *actionNewTable, *actionNewFunctionPlot;
     QAction *actionNewSurfacePlot, *actionNewMatrix, *actionNewGraph, *actionNewFolder;
     QAction *actionOpen, *actionLoadImage, *actionSaveProject, *actionSaveProjectAs, *actionImportImage;
-    QAction *actionLoad, *actionUndo, *actionRedo, *actionImportSound;
+	QAction *actionLoad, *actionUndo, *actionRedo, *actionImportSound;
+#ifdef ODS_IMPORT
+	QAction *actionOpenOds;
+#endif
+#ifdef XLS_IMPORT
+	QAction *actionOpenExcel, *actionExportExcel;
+#endif
     QAction *actionCopyWindow, *actionShowAllColumns, *actionHideSelectedColumns;
     QAction *actionCutSelection, *actionCopySelection, *actionPasteSelection, *actionClearSelection;
     QAction *actionShowExplorer, *actionShowLog, *actionAddLayer, *actionShowLayerDialog, *actionAutomaticLayout;
@@ -1335,9 +1430,9 @@ private:
     QAction *actionShowConsole;
 #endif
 #ifdef SCRIPTING_PYTHON
-    QAction *actionOpenQtDesignerUi, *actionShowScriptWindow;
+	QAction *actionOpenQtDesignerUi, *actionCommentSelection, *actionUncommentSelection;
 #endif
-    QAction *actionSwapColumns, *actionMoveColRight, *actionMoveColLeft, *actionMoveColFirst, *actionMoveColLast;
+    QAction *actionSwapColumns, *actionMoveColRight, *actionMoveColLeft, *actionMoveColFirst, *actionMoveColLast, *actionShowScriptWindow;
     QAction *actionExportGraph, *actionExportAllGraphs, *actionPrint, *actionPrintAllPlots, *actionShowExportASCIIDialog;
     QAction *actionExportPDF, *actionReadOnlyCol, *actionStemPlot;
     QAction *actionCloseAllWindows, *actionCloseProject, *actionClearLogInfo, *actionShowPlotWizard, *actionShowConfigureDialog;
@@ -1378,7 +1473,7 @@ private:
 	QAction *actionBoxPlot, *actionMultiPeakGauss, *actionMultiPeakLorentz, *actionCheckUpdates;
 	QAction *actionDonate, *actionHomePage, *actionDownloadManual, *actionTechnicalSupport, *actionTranslations;
 	QAction *actionHelpForums, *actionHelpBugReports;
-	QAction *actionShowPlotDialog, *actionShowScaleDialog, *actionOpenTemplate, *actionSaveTemplate;
+	QAction *actionShowPlotDialog, *actionShowScaleDialog, *actionOpenTemplate, *actionSaveTemplate, *actionSaveWindow;
 	QAction *actionNextWindow, *actionPrevWindow;
 	QAction *actionScriptingLang, *actionRestartScripting, *actionClearTable, *actionGoToRow, *actionGoToColumn;
 	QAction *actionNoteExecute, *actionNoteExecuteAll, *actionNoteEvaluate, *actionSaveNote, *actionFrequencyCount;
@@ -1389,7 +1484,7 @@ private:
 	QAction *actionFlipMatrixVertically, *actionFlipMatrixHorizontally, *actionRotateMatrix;
 	QAction *actionViewMatrixImage, *actionViewMatrix, *actionExportMatrix, *actionMatrixDefaultScale;
     QAction *actionMatrixGrayScale, *actionMatrixRainbowScale, *actionMatrixCustomScale, *actionRotateMatrixMinus;
-    QAction *actionMatrixXY, *actionMatrixColumnRow, *actionImagePlot, *actionToolBars;
+    QAction *actionMatrixXY, *actionMatrixColumnRow, *actionImagePlot, *actionToolBars, *actionImageProfilesPlot;
     QAction *actionMatrixFFTDirect, *actionMatrixFFTInverse;
 	QAction *actionFontBold, *actionFontItalic, *actionFontBox, *actionFontSize;
 	QAction *actionSuperscript, *actionSubscript, *actionUnderline, *actionGreekSymbol, *actionCustomActionDialog;
@@ -1399,13 +1494,21 @@ private:
     QAction *wireframe, *hiddenline, *polygon, *filledmesh, *pointstyle, *barstyle, *conestyle, *crossHairStyle;
     QAction *actionShowUndoStack, *actionShowNoteLineNumbers, *actionAddFormula, *actionAddRectangle, *actionAddEllipse;
     QActionGroup *coord, *floorstyle, *grids, *plotstyle, *dataTools;
-    QAction *actionMagnify, *actionFindWindow, *actionWaterfallPlot;
+	QAction *actionMagnify, *actionFindWindow, *actionWaterfallPlot, *actionMagnifyHor, *actionMagnifyVert;
     QAction *actionMoveRowUp, *actionMoveRowDown, *actionAdjustColumnWidth;
     QAction *actionPresentationODF, *actionRenameNoteTab, *actionAddNoteTab, *actionCloseNoteTab;
     QAction *actionIncreaseIndent, *actionDecreaseIndent, *actionFind, *actionFindNext, *actionFindPrev, *actionReplace;
     QAction *actionIncreasePrecision, *actionDecreasePrecision, *actionPrintPreview;
+	QAction *actionVertSharedAxisLayers, *actionHorSharedAxisLayers, *actionSharedAxesLayers, *actionStackSharedAxisLayers;
+	QAction *actionCustomSharedAxisLayers, *actionCustomLayout, *actionExtractTableData, *actionSubtractLine, *actionSubtractReference;
+	QAction *actionRaiseEnrichment, *actionLowerEnrichment, *actionBaseline;
+
     QList<QAction *> d_user_actions;
     QUndoView *d_undo_view;
     QList<QMenu *> d_user_menus;
+
+	QList<QColor> d_indexed_colors;
+	QStringList d_indexed_color_names;
+	QList<int> d_symbols_list;
 };
 #endif
