@@ -2,7 +2,8 @@
     File                 : ImportASCIIDialog.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006,2007 by Ion Vasilief, Knut Franke
+	Copyright            : (C) 2004 - 2010 by Ion Vasilief,
+						   (C) 2006 - June 2007 by Knut Franke
     Email (use @ for *)  : ion_vasilief*yahoo.fr, knut.franke*gmx.de
     Description          : Import ASCII file(s) dialog
 
@@ -44,6 +45,7 @@
 #include <QProgressDialog>
 #include <QStackedWidget>
 #include <QHeaderView>
+#include <QInputDialog>
 
 #include <gsl/gsl_math.h>
 
@@ -51,14 +53,6 @@ ImportASCIIDialog::ImportASCIIDialog(bool new_windows_only, QWidget * parent, bo
 : ExtensibleFileDialog(parent, extended, flags )
 {
 	setWindowTitle(tr("QtiPlot - Import ASCII File(s)"));
-
-	QStringList filters;
-	filters << tr("All files") + " (*)";
-	filters << tr("Text files") + " (*.TXT *.txt)";
-	filters << tr("Data files") + " (*.DAT *.dat)";
-	filters << tr("Comma Separated Values") + " (*.CSV *.csv)";
-	setFilters( filters );
-
 	setFileMode( QFileDialog::ExistingFiles );
 
 	d_current_path = QString::null;
@@ -127,9 +121,8 @@ void ImportASCIIDialog::initAdvancedOptions()
 
 	advanced_layout->addWidget(new QLabel(tr("Import each file as: ")), 0, 0);
 	d_import_mode = new QComboBox();
-	// Important: Keep this in sync with the ImportMode enum.
 	d_import_mode->addItem(tr("New Table"));
-	d_import_mode->addItem(tr("New Matrice"));
+	d_import_mode->addItem(tr("New Matrix"));
 	d_import_mode->addItem(tr("New Columns"));
 	d_import_mode->addItem(tr("New Rows"));
 	d_import_mode->addItem(tr("Overwrite Current Window"));
@@ -236,9 +229,16 @@ void ImportASCIIDialog::initAdvancedOptions()
 	d_preview_lines_box->setSpecialValueText(tr("All"));
 	advanced_layout->addWidget(d_preview_lines_box, 6, 1);
 
+	QHBoxLayout *btnsLayout = new QHBoxLayout();
+	d_col_types_button = new QPushButton(tr("Column T&ypes..."));
+	connect(d_col_types_button, SIGNAL(clicked()), this, SLOT(showColTypeDialog()));
+	d_col_types_button->hide();
+	btnsLayout->addWidget(d_col_types_button);
+
 	d_help_button = new QPushButton(tr("&Help"));
 	connect(d_help_button, SIGNAL(clicked()), this, SLOT(displayHelp()));
-	advanced_layout->addWidget(d_help_button, 6, 2);
+	btnsLayout->addWidget(d_help_button);
+	advanced_layout->addLayout(btnsLayout, 6, 2);
 
 	d_preview_table = NULL;
 	d_preview_matrix = NULL;
@@ -246,6 +246,14 @@ void ImportASCIIDialog::initAdvancedOptions()
 	main_layout->addWidget(d_preview_stack);
 }
 
+void ImportASCIIDialog::showColTypeDialog()
+{
+	if (d_preview_table){
+		d_preview_table->setSelectedColumn(0);
+		d_preview_table->showColTypeDialog();
+	}
+}
+
 void ImportASCIIDialog::initPreview(int previewMode)
 {
 	if (previewMode < NewTables || previewMode > Overwrite)
@@ -270,6 +278,7 @@ void ImportASCIIDialog::initPreview(int previewMode)
 			d_preview_table = new PreviewTable(30, 2, this);
 			d_preview_table->setNumericPrecision(app->d_decimal_digits);
 			d_preview_stack->addWidget(d_preview_table);
+			connect(d_preview_table, SIGNAL(modifiedColumnType()), this, SLOT(preview()));
 			enableTableOptions(true);
 		break;
 
@@ -287,9 +296,10 @@ void ImportASCIIDialog::initPreview(int previewMode)
 				return;
 
 			if (w->inherits("Table")){
-				d_preview_table = new PreviewTable(30, 2, this);
+				d_preview_table = new PreviewTable(30, ((Table*)w)->numCols(), this);
 				d_preview_table->setNumericPrecision(app->d_decimal_digits);
 				d_preview_stack->addWidget(d_preview_table);
+				connect(d_preview_table, SIGNAL(modifiedColumnType()), this, SLOT(preview()));
 				enableTableOptions(true);
 			} else if (w->isA("Matrix")){
 				d_preview_matrix = new PreviewMatrix(app, (Matrix *)w);
@@ -306,6 +316,7 @@ void ImportASCIIDialog::enableTableOptions(bool on)
 	d_rename_columns->setEnabled(on);
 	d_import_comments->setEnabled(on && d_rename_columns->isChecked());
 	d_read_only->setEnabled(on);
+	d_col_types_button->setVisible(on);
 }
 
 void ImportASCIIDialog::setColumnSeparator(const QString& sep)
@@ -372,12 +383,17 @@ void ImportASCIIDialog::displayHelp()
 
 void ImportASCIIDialog::updateImportMode(int mode)
 {
+	setUpdatesEnabled(false);
+
 	if (mode == Overwrite)
 		setFileMode( QFileDialog::ExistingFile );
 	else
 		setFileMode( QFileDialog::ExistingFiles );
 
 	initPreview(mode);
+
+	setUpdatesEnabled(true);
+	repaint();
 }
 
 void ImportASCIIDialog::closeEvent(QCloseEvent* e)
@@ -385,10 +401,11 @@ void ImportASCIIDialog::closeEvent(QCloseEvent* e)
 	ApplicationWindow *app = (ApplicationWindow *)this->parent();
 	if (app){
 		app->d_extended_import_ASCII_dialog = this->isExtended();
-		app->d_ASCII_file_filter = this->selectedFilter();
+		app->d_ASCII_file_filter = this->selectedNameFilter();
 		app->d_ASCII_import_preview = d_preview_button->isChecked();
 		app->d_preview_lines = d_preview_lines_box->value();
 		app->d_ASCII_import_first_row_role = d_first_line_role->currentIndex();
+		app->d_import_ASCII_dialog_size = size();
 	}
 
 	e->accept();
@@ -443,8 +460,8 @@ void ImportASCIIDialog::previewTable()
 	if (d_current_path.trimmed().isEmpty()){
 		d_preview_table->clear();
 		d_preview_table->resetHeader();
-        return;
-    }
+		return;
+	}
 
 	int importMode = d_import_mode->currentIndex();
 	if (importMode == NewTables)
@@ -452,14 +469,14 @@ void ImportASCIIDialog::previewTable()
 	else
 		importMode -= 2;
 
+	d_preview_table->clear();
 	d_preview_table->resetHeader();
-	d_preview_table->importASCII(d_current_path, columnSeparator(), d_ignored_lines->value(),
-							renameColumns(), d_strip_spaces->isChecked(),
-							d_simplify_spaces->isChecked(), importComments(),
-                            d_comment_string->text(), (Table::ImportMode)importMode,
-                            boxEndLine->currentIndex(), d_preview_lines_box->value());
 
-	d_preview_table->updateDecimalSeparators(decimalSeparators());
+	d_preview_table->importASCII(d_current_path, columnSeparator(), d_ignored_lines->value(),
+						renameColumns(), d_strip_spaces->isChecked(),
+						d_simplify_spaces->isChecked(), importComments(),
+						d_comment_string->text(), (Table::ImportMode)importMode, decimalSeparators(),
+						boxEndLine->currentIndex(), d_preview_lines_box->value());
 
     if (!d_preview_table->isVisible())
         d_preview_table->show();
@@ -493,6 +510,11 @@ void ImportASCIIDialog::changePreviewFile(const QString& path)
 	if (path.isEmpty())
 		return;
 
+	if (d_current_path.isEmpty()){//avoid importing the first file which is by default selected in the file dialog
+		d_current_path = path;
+		return;
+	}
+
 	QFileInfo fi(path);
 	if (!fi.exists() || fi.isDir() || !fi.isFile())
 		return;
@@ -534,6 +556,23 @@ void ImportASCIIDialog::enableComments()
 								!d_first_line_role->currentIndex());
 }
 
+void ImportASCIIDialog::selectFilter(const QString & filter)
+{
+	QStringList filters;
+	filters << tr("All files") + " (*)";
+	filters << tr("Text files") + " (*.TXT *.txt)";
+	filters << tr("Data files") + " (*.DAT *.dat)";
+	filters << tr("Comma Separated Values") + " (*.CSV *.csv)";
+
+	if (!filters.contains(filter))
+		filters << filter;
+
+	setFilters(filters);
+	setEditableFilter();
+
+	QFileDialog::selectFilter(filter);
+}
+
 /*****************************************************************************
  *
  * Class PreviewTable
@@ -549,11 +588,18 @@ PreviewTable::PreviewTable(int numRows, int numCols, QWidget * parent, const cha
 	setRowMovingEnabled(false);
 	setColumnMovingEnabled(false);
 	verticalHeader()->setResizeEnabled(false);
+	horizontalHeader()->installEventFilter(this);
+	horizontalHeader()->setResizeEnabled(true);
+	horizontalHeader()->setMovingEnabled (false);
 
-	for (int i=0; i<numCols; i++){
+	for (int i = 0; i < numCols; i++){
 		comments << "";
-		col_label << QString::number(i+1);
+		col_label << QString::number(i + 1);
+		colTypes << Table::Numeric;
+		d_col_format << QString();
 	}
+
+	d_start_col = numCols;
 	setHeader();
 	setMinimumHeight(2*horizontalHeader()->height());
 	connect(horizontalHeader(), SIGNAL(sizeChange(int, int, int)), this, SLOT(setHeader()));
@@ -561,7 +607,7 @@ PreviewTable::PreviewTable(int numRows, int numCols, QWidget * parent, const cha
 
 void PreviewTable::importASCII(const QString &fname, const QString &sep, int ignoredLines, bool renameCols,
     bool stripSpaces, bool simplifySpaces, bool importComments, const QString& commentString,
-	int importMode, int endLine, int maxRows)
+	int importMode, const QLocale& importLocale, int endLine, int maxRows)
 {
 	int rows;
 	QString name = MdiSubWindow::parseAsciiFile(fname, commentString, endLine, ignoredLines, maxRows, rows);
@@ -574,6 +620,9 @@ void PreviewTable::importASCII(const QString &fname, const QString &sep, int ign
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
+	QLocale locale = this->locale();
+	bool updateDecimalSeparators = (importLocale != locale) ? true : false;
+
 	QTextStream t(&f);
 	QString s = t.readLine();//read first line
 	if (simplifySpaces)
@@ -587,7 +636,7 @@ void PreviewTable::importASCII(const QString &fname, const QString &sep, int ign
 	bool allNumbers = true;
 	for (int i=0; i<cols; i++)
 	{//verify if the strings in the line used to rename the columns are not all numbers
-		locale().toDouble(line[i], &allNumbers);
+		locale.toDouble(line[i], &allNumbers);
 		if (!allNumbers)
 			break;
 	}
@@ -613,8 +662,11 @@ void PreviewTable::importASCII(const QString &fname, const QString &sep, int ign
 			}
 		break;
 		case Table::NewColumns:
-			startCol = c;
-			addColumns(cols);
+			startCol = d_start_col;
+			//d_start_col += cols;
+			if (abs(c - d_start_col) < cols)
+				addColumns(cols);
+
 			if (r < rows)
 				setNumRows(rows);
 		break;
@@ -662,8 +714,14 @@ void PreviewTable::importASCII(const QString &fname, const QString &sep, int ign
 
 	if ((!renameCols || allNumbers)&& !importComments && rows > 0){
 		//put values in the first line of the table
-		for (int i = 0; i<cols; i++)
-			setText(startRow, startCol + i, line[i]);
+		for (int i = 0; i<cols; i++){
+			bool ok;
+			double val = importLocale.toDouble(line[i], &ok);
+			if (colTypes[startCol + i] == Table::Numeric && (ok || updateDecimalSeparators))
+				setText(startRow, startCol + i, locale.toString(val, 'g', d_numeric_precision));
+			else
+				setText(startRow, startCol + i, line[i]);
+		}
 		startRow++;
 	}
 
@@ -686,8 +744,14 @@ void PreviewTable::importASCII(const QString &fname, const QString &sep, int ign
 			addColumns(lc - cols);
 			cols = lc;
 		}
-		for (int j=0; j<cols && j<lc; j++)
-			setText(row, startCol + j, line[j]);
+		for (int j = 0; j < cols && j < lc; j++){
+			bool ok;
+			double val = importLocale.toDouble(line[j], &ok);
+			if (colTypes[startCol + j] == Table::Numeric && (ok || updateDecimalSeparators))
+				setText(row, startCol + j, locale.toString(val, 'g', d_numeric_precision));
+			else
+				setText(row, startCol + j, line[j]);
+		}
 
 		row++;
 		qApp->processEvents(QEventLoop::ExcludeUserInput);
@@ -698,10 +762,23 @@ void PreviewTable::importASCII(const QString &fname, const QString &sep, int ign
 
 void PreviewTable::resetHeader()
 {
-	for (int i=0; i<numCols(); i++){
-	    comments[i] = QString::null;
+	int cols = numCols();
+	for (int i = 0; i < cols; i++){
+		comments[i] = QString::null;
 		col_label[i] = QString::number(i+1);
 	}
+
+	int size = colTypes.size();
+	if (size > cols){
+		for (int i = 0; i < (size - cols); i++)
+			colTypes.pop_back();
+	}
+
+	size = d_col_format.size();
+	if (size > cols){
+		for (int i = 0; i < (size - cols); i++)
+			d_col_format.pop_back();
+	}
 }
 
 void PreviewTable::clear()
@@ -712,22 +789,6 @@ void PreviewTable::clear()
 	}
 }
 
-void PreviewTable::updateDecimalSeparators(const QLocale& oldSeparators)
-{
-	QLocale locale = ((QWidget *)parent())->locale();
-	if (locale == oldSeparators)
-        return;
-
-	for (int i=0; i<numCols(); i++){
-        for (int j=0; j<numRows(); j++){
-            if (!text(j, i).isEmpty()){
-				double val = oldSeparators.toDouble(text(j, i));
-                setText(j, i, locale.toString(val, 'g', d_numeric_precision));
-			}
-		}
-	}
-}
-
 void PreviewTable::setHeader()
 {
 	Q3Header *head = horizontalHeader();
@@ -750,12 +811,199 @@ void PreviewTable::addColumns(int c)
 	}
 	max++;
 	insertColumns(cols, c);
-	for (int i=0; i<c; i++){
+	for (int i = 0; i < c; i++){
 		comments << QString();
-		col_label<< QString::number(max+i);
+		col_label << QString::number(max+i);
+		colTypes << Table::Numeric;
+		d_col_format << QString();
 	}
 }
 
+bool PreviewTable::eventFilter(QObject *object, QEvent *e)
+{
+	Q3Header *hheader = horizontalHeader();
+	if (e->type() == QEvent::MouseButtonPress && object == (QObject*)hheader) {
+		const QMouseEvent *me = (const QMouseEvent *)e;
+		clearSelection();
+		int col = hheader->sectionAt (me->pos().x() + hheader->offset());
+		if (col >= 0 && col < numCols()){
+			QRect rect = hheader->sectionRect(col);
+			rect.setLeft(rect.left() + 2);
+			rect.setRight(rect.right() - 2);
+			if (rect.contains (me->pos())){
+				selectColumn(col);
+				d_selected_column = col;
+				showColTypeDialog();
+				return true;
+			}
+		}
+	}
+	return Q3Table::eventFilter(object, e);
+}
+
+void PreviewTable::setSelectedColumn(int col)
+{
+	selectColumn(col);
+	d_selected_column = col;
+}
+
+void PreviewTable::setColumnFormat(const QString& format)
+{
+	d_col_format[d_selected_column] = format;
+}
+
+void PreviewTable::setColumnType(int type)
+{
+	bool vis = type > Table::Text;
+	formatBox->setVisible(vis);
+	formatLabel->setVisible(vis);
+
+	formatBox->blockSignals(true);
+	formatBox->clear();
+	formatBox->setEditable(false);
+	switch (type){
+		case Table::Date:
+			formatBox->addItem("dd/MM/yyyy");
+			formatBox->addItem("dd/MM/yyyy HH:mm");
+			formatBox->addItem("dd/MM/yyyy HH:mm:ss");
+
+			formatBox->addItem("dd.MM.yyyy");
+			formatBox->addItem("dd.MM.yyyy HH:mm");
+			formatBox->addItem("dd.MM.yyyy HH:mm:ss");
+
+			formatBox->addItem("dd MM yyyy");
+			formatBox->addItem("dd MM yyyy HH:mm");
+			formatBox->addItem("dd MM yyyy HH:mm:ss");
+
+			formatBox->addItem("yyyy-MM-dd");
+			formatBox->addItem("yyyy-MM-dd HH:mm");
+			formatBox->addItem("yyyy-MM-dd HH:mm:ss");
+
+			formatBox->addItem("yyyyMMdd");
+			formatBox->addItem("yyyyMMdd HH:mm");
+			formatBox->addItem("yyyyMMdd HH:mm:ss");
+
+			formatBox->setEditable(true);
+			formatBox->setEditText(d_col_format[d_selected_column]);
+		break;
+
+		case Table::Time:
+			formatBox->addItem("h");
+			formatBox->addItem("h ap");
+			formatBox->addItem("h AP");
+			formatBox->addItem("h:mm");
+			formatBox->addItem("h:mm ap");
+			formatBox->addItem("hh:mm");
+			formatBox->addItem("h:mm:ss");
+			formatBox->addItem("h:mm:ss.zzz");
+			formatBox->addItem("mm:ss");
+			formatBox->addItem("mm:ss.zzz");
+			formatBox->addItem("hmm");
+			formatBox->addItem("hmmss");
+			formatBox->addItem("hhmmss");
+
+			formatBox->setEditable(true);
+			formatBox->setEditText(d_col_format[d_selected_column]);
+		break;
+
+		default:
+		break;
+	}
+
+	colTypes[d_selected_column] = type;
+	formatBox->blockSignals(false);
+	modifiedColumnType();
+}
+
+void PreviewTable::showColTypeDialog()
+{
+	QDialog *colTypeDialog = new QDialog(this);
+	colTypeDialog->setAttribute(Qt::WA_DeleteOnClose);
+	colTypeDialog->setSizeGripEnabled(true);
+	colTypeDialog->setMinimumWidth(300);
+	colTypeDialog->setWindowTitle(tr("Choose column type"));
+
+	gb1 = new QGroupBox();
+	QGridLayout *hl1 = new QGridLayout(gb1);
+	hl1->addWidget(new QLabel(tr("Type")), 0, 0);
+	typesBox = new QComboBox();
+	typesBox->addItems(QStringList() << tr("Numeric") << tr("Text") << tr("Date") << tr("Time"));
+	hl1->addWidget(typesBox, 0, 1);
+
+	formatLabel = new QLabel(tr("Format"));
+	hl1->addWidget(formatLabel, 1, 0);
+	formatLabel->hide();
+
+	formatBox = new QComboBox();
+	hl1->addWidget(formatBox, 1, 1);
+	formatBox->hide();
+	hl1->setColumnStretch(1, 1);
+	hl1->setRowStretch(2, 1);
+
+	buttonPrev = new QPushButton("&<<");
+	buttonPrev->setAutoDefault(false);
+	buttonPrev->setMaximumWidth(40);
+
+	buttonNext = new QPushButton("&>>");
+	buttonNext->setAutoDefault(false);
+	buttonNext->setMaximumWidth(40);
+
+	QPushButton *closeBtn = new QPushButton(tr("&Close"));
+	connect(closeBtn, SIGNAL(clicked()), colTypeDialog, SLOT(reject()));
+
+	QHBoxLayout *hl2 = new QHBoxLayout();
+	hl2->addStretch();
+	hl2->addWidget(buttonPrev);
+	hl2->addWidget(buttonNext);
+	hl2->addWidget(closeBtn);
+
+	QVBoxLayout *vl = new QVBoxLayout(colTypeDialog);
+	vl->addWidget(gb1);
+	vl->addLayout(hl2);
+
+	updateColumn(d_selected_column);
+
+	connect(typesBox, SIGNAL(currentIndexChanged (int)), this, SLOT(setColumnType(int)));
+	connect(formatBox, SIGNAL(currentIndexChanged(const QString&)), this, SLOT(setColumnFormat(const QString&)));
+	connect(formatBox, SIGNAL(editTextChanged(const QString&)), this, SLOT(setColumnFormat(const QString&)));
+	connect(buttonPrev, SIGNAL(clicked()), this, SLOT(prevColumn()));
+	connect(buttonNext, SIGNAL(clicked()), this, SLOT(nextColumn()));
+	colTypeDialog->exec();
+}
+
+void PreviewTable::prevColumn()
+{
+	updateColumn(--d_selected_column);
+}
+
+void PreviewTable::nextColumn()
+{
+	updateColumn(++d_selected_column);
+}
+
+void PreviewTable::updateColumn(int sc)
+{
+	if (sc < 0 || sc >= numCols())
+		return;
+
+	typesBox->setCurrentIndex(colTypes[sc]);
+	setColumnType(typesBox->currentIndex());
+
+	gb1->setTitle(tr("Column") + " " + QString::number(sc + 1));
+
+	if (!sc)
+		buttonPrev->setEnabled(false);
+	else
+		buttonPrev->setEnabled(true);
+
+	if (sc >= numCols() - 1)
+		buttonNext->setEnabled(false);
+	else
+		buttonNext->setEnabled(true);
+
+	clearSelection();
+	selectColumn(sc);
+}
 /*****************************************************************************
  *
  * Class PreviewMatrix
@@ -802,6 +1050,7 @@ void PreviewMatrix::importASCII(const QString &fname, const QString &sep, int ig
 {
 	d_matrix_model->importASCII(fname, sep, ignoredLines, stripSpaces,
 		simplifySpaces, commentString, importAs, locale, endLine, maxRows);
+	reset();
 }
 
 void PreviewMatrix::clear()
