@@ -70,6 +70,11 @@
 #include <EmfEngine.h>
 #endif
 
+#ifdef XLS_IMPORT
+    #include <ExcelFormat.h>
+    using namespace ExcelFormat;
+#endif
+
 Matrix::Matrix(ScriptingEnv *env, int r, int c, const QString& label, ApplicationWindow* parent, const QString& name, Qt::WFlags f)
 : MdiSubWindow(label, parent, name, f), scripted(env)
 {
@@ -140,6 +145,11 @@ void Matrix::initImage(const QImage& image)
     } else
 		imageLabel->resize(500, 500);
 
+	x_start = 0.0;
+	x_end = w - 1.0;
+	y_start = 0.0;
+	y_end = h - 1.0;
+
 	displayImage(image);
 }
 
@@ -1100,6 +1110,16 @@ void Matrix::exportEMF(const QString& fileName)
 }
 #endif
 
+bool Matrix::isEmpty()
+{
+	double min, max;
+	range(&min, &max);
+	if (gsl_isnan(min) && gsl_isnan(max))
+		return true;
+
+	return false;
+}
+
 void Matrix::range(double *min, double *max)
 {
 	double d_min = cell(0, 0);
@@ -1251,14 +1271,14 @@ void Matrix::setViewType(ViewType type, bool renderImage)
 	if (d_view_type == ImageView){
 		delete d_table_view;
 		delete d_select_all_shortcut;
-	    initImageView();
+		initImageView();
 		if (renderImage)
 			displayImage(d_matrix_model->renderImage());
 		d_stack->setCurrentWidget(imageLabel);
 	} else if (d_view_type == TableView){
 		delete imageLabel;
-	    initTableView();
-	    d_stack->setCurrentWidget(d_table_view);
+		initTableView();
+		d_stack->setCurrentWidget(d_table_view);
 	}
 	emit modifiedWindow(this);
 }
@@ -1319,6 +1339,11 @@ void Matrix::importImage(const QString& fn)
     if (image.isNull())
         return;
 
+	x_start = 0.0;
+	x_end = image.width() - 1.0;
+	y_start = 0.0;
+	y_end = image.height() - 1.0;
+
 	double *buffer = d_matrix_model->dataCopy();
 	if (buffer){
     	d_undo_stack->push(new MatrixSetImageCommand(d_matrix_model, image, d_view_type, 0,
@@ -1332,6 +1357,7 @@ void Matrix::importImage(const QString& fn)
 		emit modifiedWindow(this);
 		modifiedData(this);
 	}
+
 	setWindowLabel(fn);
 }
 
@@ -1496,24 +1522,26 @@ bool Matrix::exportODF(const QString& fname, bool exportSelection)
 	int rightCol = numCols() - 1;
 
 	if (exportSelection && d_view_type == TableView){
-        QModelIndexList selectedIndexes = d_table_view->selectionModel()->selectedIndexes();
-        topRow = selectedIndexes[0].row();
-        bottomRow = topRow;
-        leftCol = selectedIndexes[0].column();
-        rightCol = leftCol;
-        foreach(QModelIndex index, selectedIndexes){
-            int row = index.row();
-            if (row < topRow)
-                topRow = row;
-            if (row > bottomRow)
-                bottomRow = row;
+            QModelIndexList selectedIndexes = d_table_view->selectionModel()->selectedIndexes();
+            if (!selectedIndexes.isEmpty()){
+                topRow = selectedIndexes[0].row();
+                bottomRow = topRow;
+                leftCol = selectedIndexes[0].column();
+                rightCol = leftCol;
+            }
+            foreach(QModelIndex index, selectedIndexes){
+                int row = index.row();
+                if (row < topRow)
+                    topRow = row;
+                if (row > bottomRow)
+                    bottomRow = row;
 
-            int col = index.column();
-            if (col < leftCol)
-                leftCol = col;
-            if (col > rightCol)
-                rightCol = col;
-        }
+                int col = index.column();
+                if (col < leftCol)
+                    leftCol = col;
+                if (col > rightCol)
+                    rightCol = col;
+            }
 	}
 
 	QTextDocument *document = new QTextDocument();
@@ -1545,6 +1573,61 @@ bool Matrix::exportODF(const QString& fname, bool exportSelection)
 	return true;
 }
 
+#ifdef XLS_IMPORT
+bool Matrix::exportExcel(const QString& fname, bool exportSelection)
+{
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	int topRow = 0;
+	int bottomRow = numRows() - 1;
+	int leftCol = 0;
+	int rightCol = numCols() - 1;
+
+	if (exportSelection && d_view_type == TableView){
+		QModelIndexList selectedIndexes = d_table_view->selectionModel()->selectedIndexes();
+		if (!selectedIndexes.isEmpty()){
+			topRow = selectedIndexes[0].row();
+			bottomRow = topRow;
+			leftCol = selectedIndexes[0].column();
+			rightCol = leftCol;
+		}
+
+		foreach(QModelIndex index, selectedIndexes){
+			int row = index.row();
+			if (row < topRow)
+				topRow = row;
+			if (row > bottomRow)
+				bottomRow = row;
+
+			int col = index.column();
+			if (col < leftCol)
+				leftCol = col;
+			if (col > rightCol)
+				rightCol = col;
+		}
+	}
+
+	BasicExcel xls;
+	xls.New(1);
+	BasicExcelWorksheet* sheet = xls.GetWorksheet((size_t)0);
+
+	rightCol = QMIN(rightCol, 256);
+	bottomRow = QMIN(bottomRow, 65536);
+	for (int i = topRow; i <= bottomRow; i++){
+		for (int j = leftCol; j <= rightCol; j++){
+			if (d_matrix_model->text(i, j).isEmpty())
+				continue;
+			sheet->Cell(i, j)->Set(d_matrix_model->cell(i, j));
+		}
+	}
+
+	xls.SaveAs(fname);
+
+	QApplication::restoreOverrideCursor();
+	return true;
+}
+#endif
+
 bool Matrix::exportASCII(const QString& fname, const QString& separator, bool exportSelection)
 {
 	QFile f(fname);
@@ -1557,7 +1640,13 @@ bool Matrix::exportASCII(const QString& fname, const QString& separator, bool ex
 	if (fname.endsWith(".odf") || fname.endsWith(".html")){
 		f.close();
 		return exportODF(fname, exportSelection);
-	}
+        }
+#ifdef XLS_IMPORT
+        else if (fname.endsWith(".xls")){
+            f.close();
+            return exportExcel(fname, exportSelection);
+        }
+#endif
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
@@ -1578,24 +1667,26 @@ bool Matrix::exportASCII(const QString& fname, const QString& separator, bool ex
 	}
 
 	if (exportSelection && d_view_type == TableView){
-        QModelIndexList selectedIndexes = d_table_view->selectionModel()->selectedIndexes();
-        topRow = selectedIndexes[0].row();
-        bottomRow = topRow;
-        leftCol = selectedIndexes[0].column();
-        rightCol = leftCol;
-        foreach(QModelIndex index, selectedIndexes){
-            int row = index.row();
-            if (row < topRow)
-                topRow = row;
-            if (row > bottomRow)
-                bottomRow = row;
+            QModelIndexList selectedIndexes = d_table_view->selectionModel()->selectedIndexes();
+            if (!selectedIndexes.isEmpty()){
+                topRow = selectedIndexes[0].row();
+                bottomRow = topRow;
+                leftCol = selectedIndexes[0].column();
+                rightCol = leftCol;
+            }
+            foreach(QModelIndex index, selectedIndexes){
+                int row = index.row();
+                if (row < topRow)
+                    topRow = row;
+                if (row > bottomRow)
+                    bottomRow = row;
 
-            int col = index.column();
-            if (col < leftCol)
-                leftCol = col;
-            if (col > rightCol)
-                rightCol = col;
-        }
+                int col = index.column();
+                if (col < leftCol)
+                    leftCol = col;
+                if (col > rightCol)
+                    rightCol = col;
+            }
 	}
 
 	if (exportTex){
