@@ -28,12 +28,13 @@
  ***************************************************************************/
 #include "DataPickerTool.h"
 #include "Graph.h"
-#include <cursors.h>
 #include "FunctionCurve.h"
 #include "PlotCurve.h"
 #include "QwtErrorPlotCurve.h"
 #include <ApplicationWindow.h>
 #include <MultiLayer.h>
+#include <LegendWidget.h>
+
 #include <QClipboard>
 
 #include <qwt_symbol.h>
@@ -62,7 +63,7 @@ DataPickerTool::DataPickerTool(Graph *graph, ApplicationWindow *app, Mode mode,
 		d_graph->canvas()->setCursor(Qt::PointingHandCursor);
 	} else {
 		setSelectionFlags(QwtPicker::PointSelection | QwtPicker::ClickSelection);
-		d_graph->canvas()->setCursor(QCursor(QPixmap(vizor_xpm), -1, -1));
+		d_graph->canvas()->setCursor(QCursor(QPixmap(":/vizor.png"), -1, -1));
 	}
 
 	if (status_target)
@@ -251,10 +252,10 @@ bool DataPickerTool::keyEventFilter(QKeyEvent *ke)
 		case Qt::Key_Plus:
 			if (d_selected_curve) {
 				int n_points = d_selected_curve->dataSize();
-				if (ke->modifiers () == Qt::ShiftModifier)
-                    setSelection(d_selected_curve, (d_selected_point + 10) % n_points);
-                else if (ke->modifiers () == Qt::NoModifier)
-                    setSelection(d_selected_curve, (d_selected_point + 1) % n_points);
+				if (ke->modifiers () & Qt::ShiftModifier)
+					setSelection(d_selected_curve, (d_selected_point + 10) % n_points);
+				else
+					setSelection(d_selected_curve, (d_selected_point + 1) % n_points);
 				d_graph->replot();
 			} else
 				setSelection(d_graph->curve(0), 0);
@@ -264,10 +265,10 @@ bool DataPickerTool::keyEventFilter(QKeyEvent *ke)
 		case Qt::Key_Minus:
 			if (d_selected_curve) {
 				int n_points = d_selected_curve->dataSize();
-				if (ke->modifiers () == Qt::ShiftModifier)
-                    setSelection(d_selected_curve, (d_selected_point - 10 + n_points) % n_points);
-                else if (ke->modifiers () == Qt::NoModifier)
-                    setSelection(d_selected_curve, (d_selected_point - 1 + n_points) % n_points);
+				if (ke->modifiers () & Qt::ShiftModifier)
+					setSelection(d_selected_curve, (d_selected_point - 10 + n_points) % n_points);
+				else
+					setSelection(d_selected_curve, (d_selected_point - 1 + n_points) % n_points);
 				d_graph->replot();
 			} else
 				setSelection(d_graph->curve(d_graph->curveCount()-1), 0);
@@ -485,49 +486,64 @@ void DataPickerTool::copySelection()
 	QApplication::clipboard()->setText(text);
 }
 
+void DataPickerTool::pasteSelectionAsLayerText()
+{
+	double x = d_selected_curve->x(d_selected_point);
+	double y = d_selected_curve->y(d_selected_point);
+
+	QString text = d_app->locale().toString(x, 'G', 16) + "/" + d_app->locale().toString(y, 'G', 16);
+	LegendWidget *l = d_graph->newLegend(text);
+	l->setAttachPolicy(FrameWidget::Scales);
+	l->setFrameStyle(FrameWidget::None);
+	l->setOriginCoord(x, y);
+}
+
 void DataPickerTool::pasteSelection()
 {
+	if (!d_selected_curve)
+		return;
+
 	QString text = QApplication::clipboard()->text();
 	if (text.isEmpty())
 		return;
 
-    if (((PlotCurve *)d_selected_curve)->type() == Graph::Function)
-        return;
+	if (((PlotCurve *)d_selected_curve)->type() == Graph::Function)
+		return;
 
-    Table *t = ((DataCurve*)d_selected_curve)->table();
-    if (!t)
-        return;
+	Table *t = ((DataCurve*)d_selected_curve)->table();
+	if (!t)
+		return;
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
 	QTextStream ts( &text, QIODevice::ReadOnly );
-    int row = ((DataCurve*)d_selected_curve)->tableRow(d_selected_point);
-    int col = t->colIndex(d_selected_curve->title().text());
+	int row = ((DataCurve*)d_selected_curve)->tableRow(d_selected_point);
+	int col = t->colIndex(d_selected_curve->title().text());
 
-    int prec; char f;
-    t->columnNumericFormat(col, &f, &prec);
-    QLocale locale = d_app->locale();
-    QStringList cellTexts = ts.readLine().split("\t");
-    if (cellTexts.count() >= 2){
-        bool numeric;
-        double value = locale.toDouble(cellTexts[1], &numeric);
-        if (numeric){
-            t->setText(row, col, locale.toString(value, f, prec));
+	int prec; char f;
+	t->columnNumericFormat(col, &f, &prec);
+	QLocale locale = d_app->locale();
+	QStringList cellTexts = ts.readLine().split("\t");
+	if (cellTexts.count() >= 2){
+		bool numeric;
+		double value = locale.toDouble(cellTexts[1], &numeric);
+		if (numeric){
+			t->setText(row, col, locale.toString(value, f, prec));
 
-            double x_val = d_selected_curve->x(d_selected_point);
-            d_selection_marker.setValue(x_val, value);
-            if (d_selection_marker.plot() == NULL)
-                d_selection_marker.attach(d_graph);
+			double x_val = d_selected_curve->x(d_selected_point);
+			d_selection_marker.setValue(x_val, value);
+			if (d_selection_marker.plot() == NULL)
+				d_selection_marker.attach(d_graph);
 
-            t->notifyChanges();
+			t->notifyChanges();
 
-            emit statusText(QString("%1[%2]: x=%3; y=%4")
-                            .arg(d_selected_curve->title().text())
-                            .arg(row + 1)
-                            .arg(locale.toString(x_val, 'G', d_app->d_decimal_digits))
-                            .arg(cellTexts[1]));
-        }
-    }
+			emit statusText(QString("%1[%2]: x=%3; y=%4")
+							.arg(d_selected_curve->title().text())
+							.arg(row + 1)
+							.arg(locale.toString(x_val, 'G', d_app->d_decimal_digits))
+							.arg(cellTexts[1]));
+		}
+	}
 
 	QApplication::restoreOverrideCursor();
 }
@@ -573,3 +589,17 @@ int DataPickerTool::findClosestPoint(QwtPlotCurve *c, double x, bool up)
     else
         return index_right;
 }
+
+BaselineTool::BaselineTool(QwtPlotCurve *c, Graph *graph, ApplicationWindow *app)
+	: DataPickerTool(graph, app, DataPickerTool::Move, app->displayInfoLineEdit(), SLOT(setText(const QString&)))
+{
+	setSelectedCurve(c);
+}
+
+void BaselineTool::setSelection(QwtPlotCurve *curve, int point_index)
+{
+	if (d_selected_curve && curve != d_selected_curve)
+		return;
+
+	DataPickerTool::setSelection(curve, point_index);
+}
