@@ -80,7 +80,7 @@ typedef struct{
   QColor lCol;     //!< line color
   float lWidth;    //!< line width
   int lStyle;      //!< line style
-  int filledArea;  //!< flag: toggles area filling under curve
+  double filledArea;  //!< flag: toggles area filling under curve; if > 0 it containts the alphaF value of the brush color
   QColor aCol;     //!< curve area color
   int aStyle;      //!< area filling style
   QColor symCol;   //!< symbol outline color
@@ -128,7 +128,7 @@ class Graph: public QwtPlot
 		~Graph();
 
 		enum Axis{Left, Right, Bottom, Top};
-		enum Scale{Linear, Log10};
+		enum Scale{Linear, Log10, Ln, Log2, Reciprocal, Probability, Logit};
 		enum Ticks{NoTicks = 0, Out = 1, InOut = 2, In = 3};
 		enum MarkerType{None = -1, Text = 0, Arrow = 1, Image = 2};
 		enum CurveType{Line, Scatter, LineSymbols, VerticalBars, Area, Pie, VerticalDropLines,
@@ -136,6 +136,7 @@ class Graph: public QwtPlot
 			Box, VectXYAM, VerticalSteps, ColorMap, GrayScale, Contour, Function, ImagePlot,
 			StackBar, StackColumn};
 		enum LegendDisplayMode{ColumnName, ColumnComment, TableName, TableLegend};
+		enum AxisTitlePolicy{Default, ColName, ColComment, NameAndComment};
 
 		//! Returns a pointer to the parent MultiLayer object.
 		MultiLayer *multiLayer();
@@ -154,11 +155,14 @@ class Graph: public QwtPlot
 
 		FrameWidget *activeEnrichment(){return d_active_enrichment;};
 		QList <FrameWidget *> enrichmentsList(){return d_enrichments;};
+		QList <FrameWidget *> increasingAreaEnrichmentsList();
 
 		bool hasSeletedItems();
 		void deselect();
 		void deselect(QWidget *);
 
+		QPointer<SelectionMoveResizer> selectionMoveResizer(){return d_markers_selector;};
+
 		QwtPlotItem* selectedCurveLabels();
         //! Used when restoring DataCurve curveID from a project file
         void restoreCurveLabels(int curveID, const QStringList& lst);
@@ -193,7 +197,12 @@ class Graph: public QwtPlot
 
     	void print(QPainter *, const QRect &rect, const QwtPlotPrintFilter & = QwtPlotPrintFilter());
 		void updateLayout();
-		void adjustGeometryToCanvas(const QRect &canvasRect);
+		void setCanvasGeometry(const QRect &canvasRect);
+		//!Convenience function for scripts
+		void setCanvasGeometry(int x, int y, int w, int h){setCanvasGeometry(QRect(x, y, w, h));};
+		void setCanvasSize(const QSize &size);
+		//!Convenience function for scripts
+		void setCanvasSize(int w, int h){setCanvasSize(QSize(w, h));};
 
     	void updateCurveLabels();
 
@@ -207,8 +216,9 @@ class Graph: public QwtPlot
 		void addLegendItem();
 		bool isPrinting(){return d_is_printing;};
 
-		void enablePanningMagnifier(bool on = true);
-		bool hasPanningMagnifierEnabled(){if (d_magnifier && d_panner) return true; return false;}
+		void enablePanningMagnifier(bool on = true, int mode = 0);
+		bool hasPanningMagnifierEnabled(){if (d_magnifier && d_panner) return true; return false;};
+		QwtPlotMagnifier* magnifyTool(){return d_magnifier;};
 
 		static QString escapeTeXSpecialCharacters(const QString &);
 		static QString texSuperscripts(const QString &);
@@ -219,6 +229,10 @@ class Graph: public QwtPlot
 		int speedModeMaxPoints(){return d_speed_mode_points;};
 		double getDouglasPeukerTolerance(){return d_Douglas_Peuker_tolerance;};
 
+		AxisTitlePolicy axisTitlePolicy(){return d_axis_title_policy;};
+		void setAxisTitlePolicy(const AxisTitlePolicy& policy){d_axis_title_policy = policy;};
+		void setSynchronizedScaleDivisions(bool on){d_synchronize_scales = on;};
+
 	public slots:
 		void copy(Graph* g);
 		void copyCurves(Graph* g);
@@ -275,12 +289,13 @@ class Graph: public QwtPlot
 		//! Map curve pointer to index.
 		int curveIndex(QwtPlotItem *c){return d_curves.indexOf(c);};
 		//! map curve title to index
-  	    int curveIndex(const QString &title){return plotItemsList().findIndex(title);}
-  	    //! get curve by index
-  	    QwtPlotCurve* curve(int index);
-  	    //! get curve by name
-  	    QwtPlotCurve* curve(const QString &title){return curve(curveIndex(title));};
-  	    //! get curve title string by inde (convenience function for scripts)
+		int curveIndex(const QString &title){return plotItemsList().findIndex(title);}
+		DataCurve* dataCurve(int index);
+		//! get curve by index
+		PlotCurve* curve(int index);
+		//! get curve by name
+		PlotCurve* curve(const QString &title){return curve(curveIndex(title));};
+		//! get curve title string by inde (convenience function for scripts)
 		QString curveTitle(int index);
 
 		//! Returns the names of all the curves suitable for data analysis, as a string list. The list excludes error bars and spectrograms.
@@ -299,6 +314,7 @@ class Graph: public QwtPlot
 		void setCurveStyle(int index, int s);
 		void setCurveFullRange(int curveIndex);
 		void setCurveLineColor(int curveIndex, int colorIndex);
+		void setCurveLineColor(int curveIndex, QColor qColor);
 		void setCurveLineStyle(int curveIndex, Qt::PenStyle style);
 		void setCurveLineWidth(int curveIndex, double width);
 		void setGrayScale();
@@ -308,11 +324,8 @@ class Graph: public QwtPlot
 		//! \name Output: Copy/Export/Print
 		//@{
 		void print();
-		void setScaleOnPrint(bool on){d_scale_on_print = on;};
-		void printCropmarks(bool on){d_print_cropmarks = on;};
-
 		void copyImage();
-		QPixmap graphPixmap(const QSize& size = QSize(), double scaleFontsFactor = 1.0);
+		QPixmap graphPixmap(const QSize& size = QSize(), double scaleFontsFactor = 1.0, bool transparent = false);
 		//! Provided for convenience in scripts
 		void exportToFile(const QString& fileName);
 		void exportSVG(const QString& fname, const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
@@ -332,6 +345,7 @@ class Graph: public QwtPlot
 
 		bool isExportingTeX(){return d_is_exporting_tex;};
 		void setTeXExportingMode(bool on = true){d_is_exporting_tex = on;};
+		bool escapeTeXStrings(){return d_tex_escape_strings;};
 		void setEscapeTeXStringsMode(bool on = true){d_tex_escape_strings = on;};
 		//@}
 
@@ -414,6 +428,9 @@ class Graph: public QwtPlot
 		QString saveAxesLabelsType();
 		QString saveAxesBaseline();
 		QString saveAxesFormulas();
+		QString saveAxesBackbones();
+		QString saveTickLabelsSpace();
+		QString saveLabelsPrefixAndSuffix();
 		//@}
 
 		//! \name Texts
@@ -445,7 +462,8 @@ class Graph: public QwtPlot
 
 		//! Used when opening a project file
 		void addArrow(QStringList list, int fileVersion);
-		QList<QwtPlotMarker *> linesList(){return d_lines;};
+		QList<ArrowMarker *> arrowsList(){return d_lines;};
+		int numArrows(){return d_lines.count();};
 
 		//!Draws a line/arrow depending on the value of "arrow"
 		void drawLine(bool on, bool arrow = false);
@@ -468,16 +486,16 @@ class Graph: public QwtPlot
 		//@{
 		void removeMarker();
 		//! Keep the markers on screen each time the scales are modified by adding/removing curves
-		void updateMarkersBoundingRect();
+		void updateMarkersBoundingRect(bool rescaleEvent = true);
 
-		/*!\brief Set the selected marker.
-		 * \param mrk key of the marker to be selected.
-		 * \param add whether the marker is to be added to an existing selection.
+		/*!\brief Set the selected arrow.
+		 * \param mrk key of the arrow to be selected.
+		 * \param add whether the arrow is to be added to an existing selection.
 		 * If <i>add</i> is false (the default) or there is no existing selection, a new SelectionMoveResizer is
 		 * created and stored in #d_markers_selector.
 		 */
-		void setSelectedMarker(QwtPlotMarker* mrk, bool add = false);
-		QwtPlotMarker* selectedMarker(){return d_selected_marker;};
+		void setSelectedArrow(ArrowMarker* mrk, bool add = false);
+		ArrowMarker* selectedArrow(){return d_selected_arrow;};
 		bool markerSelected();
 		//! Reset any selection states on markers.
 		void deselectMarker();
@@ -497,8 +515,9 @@ class Graph: public QwtPlot
 		void setRightAxisTitle(const QString& text);
 		void setTopAxisTitle(const QString& text);
 
-		QString axisTitleString(int axis){return axisTitle(axis).text();};
+		QString axisTitleString(int axis);
 		void setAxisTitle(int axis, const QString& text);
+		void updateAxesTitles();
 		//! TODO: eliminate this function in version 0.9.1 (used only when restoring project files)
 		void setScaleTitle(int axis, const QString& text);
 
@@ -525,7 +544,9 @@ class Graph: public QwtPlot
 
 		void showAxis(int axis, int type, const QString& formatInfo, Table *table, bool axisOn,
 				int majTicksType, int minTicksType, bool labelsOn, const QColor& c, int format,
-                int prec, int rotation, int baselineDist, const QString& formula, const QColor& labelsColor);
+				int prec, int rotation, int baselineDist, const QString& formula, const QColor& labelsColor,
+				int spacing = 4, bool backbone = true, const ScaleDraw::ShowTicksPolicy& showTicks = ScaleDraw::ShowAll,
+				const QString& prefix = QString::null, const QString& suffix = QString::null);
 
 		void enableAxis(int axis, bool on = true);
 		void enableAxisLabels(int axis, bool on = true);
@@ -539,7 +560,7 @@ class Graph: public QwtPlot
 		void drawAxesBackbones(bool yes);
 		bool axesBackbones(){return drawAxesBackbone;};
 		//! used when opening a project file
-		void loadAxesOptions(const QString& s);
+		void loadAxesOptions(const QStringList& lst);
 
 		void setAxisMargin(int axis, int margin);
 
@@ -702,6 +723,8 @@ class Graph: public QwtPlot
 		void showAxisDialog();
 		void showScaleDialog();
 
+		//! Returns a pointer to the spectrogram which data source is matrix m (the pointer can be NULL)
+		Spectrogram* spectrogram(Matrix *m);
 		//! Add a spectrogram to the graph
   		Spectrogram* plotSpectrogram(Matrix *m, CurveType type);
 		//! Restores a spectrogram. Used when opening a project file.
@@ -725,7 +748,6 @@ signals:
 		void drawLineEnded(bool);
 		void cursorInfo(const QString&);
 		void showPlotDialog(int);
-		void createTable(const QString&,int,int,const QString&);
 
 		void viewLineDialog();
 		void viewTitleDialog();
@@ -746,12 +768,18 @@ signals:
 		void currentFontChanged(const QFont&);
         void enableTextEditor(Graph *);
         void axisDivChanged(Graph *, int);
+		void updatedLayout(Graph *);
+		void selectionChanged(SelectionMoveResizer *);
+
+	private slots:
+		void selectorDeleted();
 
 	private:
+		QString parseAxisTitle(int axis);
 		QList<FrameWidget*> stackingOrderEnrichmentsList();
-        //! Finds bounding interval of the plot data.
-        QwtDoubleInterval axisBoundingInterval(int axis);
-        void deselectCurves();
+		//! Finds bounding interval of the plot data.
+		QwtDoubleInterval axisBoundingInterval(int axis);
+		void deselectCurves();
 
 		void dropEvent(QDropEvent*);
 		void dragEnterEvent(QDragEnterEvent*);
@@ -781,16 +809,15 @@ signals:
 		//! Render hint for plot items.
 		bool d_antialiasing;
 		bool autoScaleFonts;
-		bool d_scale_on_print, d_print_cropmarks;
 		bool drawLineOn, drawArrowOn, drawAxesBackbone;
 		//! Flag telling if we are performing a print operation
 		bool d_is_printing;
 		//! Stores the step the user specified for the four scale. If step = 0.0, the step will be calculated automatically by the Qwt scale engine.
 		QVector<double> d_user_step;
 		//! Arrows/lines on plot
-		QList<QwtPlotMarker*> d_lines;
+		QList<ArrowMarker*> d_lines;
 		//! Pointer to the currently selected line/image
-		QwtPlotMarker* d_selected_marker;
+		ArrowMarker* d_selected_arrow;
 		//! The markers selected for move/resize operations or NULL if none are selected.
 		QPointer<SelectionMoveResizer> d_markers_selector;
 		//! The current curve selection, or NULL if none is active.
@@ -811,5 +838,8 @@ signals:
 
 		double d_Douglas_Peuker_tolerance;
 		int d_speed_mode_points;
+		AxisTitlePolicy d_axis_title_policy;
+		bool d_synchronize_scales;
+		QStringList d_axis_titles;
 };
 #endif // GRAPH_H
