@@ -168,7 +168,9 @@ void LegendWidget::drawVector(PlotCurve *c, QPainter *p, int x, int y, int l)
 	if (d_plot->antialiasing())
 		p->setRenderHints(QPainter::Antialiasing);
 
-	p->setPen(QwtPainter::scaledPen(v->vectorPen()));
+	QPen pen = v->vectorPen();
+	pen.setCosmetic(false);
+	p->setPen(QwtPainter::scaledPen(pen));
 	p->drawLine(x, y, x + l, y);
 
 	p->translate(x + l, y);
@@ -191,18 +193,21 @@ void LegendWidget::drawVector(PlotCurve *c, QPainter *p, int x, int y, int l)
 
 void LegendWidget::drawSymbol(PlotCurve *c, int point, QPainter *p, int x, int y, int l)
 {
-    if (!c || c->rtti() == QwtPlotItem::Rtti_PlotSpectrogram)
-        return;
+	if (!c || c->rtti() == QwtPlotItem::Rtti_PlotSpectrogram)
+		return;
 
-    if (c->type() == Graph::VectXYXY || c->type() == Graph::VectXYAM){
-        drawVector(c, p, x, y, l);
-        return;
-    }
+	if (c->type() == Graph::VectXYXY || c->type() == Graph::VectXYAM){
+		drawVector(c, p, x, y, l);
+		return;
+	}
 
 	if (c->type() == Graph::Pie){
 		QwtPieCurve *pie = (QwtPieCurve *)c;
 		const QBrush br = QBrush(pie->color(point), pie->pattern());
-		QPen pen = QwtPainter::scaledPen(pie->pen());
+		QPen pen = pie->pen();
+		pen.setCosmetic(false);
+		pen = QwtPainter::scaledPen(pen);
+
 		p->save();
 		p->setPen (QPen(pen.color(), pen.widthF(), Qt::SolidLine, Qt::FlatCap, Qt::MiterJoin));
 		QRect lr = QRect(x, y - l/4, l, l/2);
@@ -212,27 +217,37 @@ void LegendWidget::drawSymbol(PlotCurve *c, int point, QPainter *p, int x, int y
 		return;
 	}
 
-    p->save();
-    if (c->style() != 0){
-        p->setPen (QwtPainter::scaledPen(c->pen()));
+	p->save();
+	if (c->style() != 0){
+		QPen pen = c->pen();
+		pen.setCosmetic(false);
+		p->setPen (QwtPainter::scaledPen(pen));
 		if (c->type() == Graph::VerticalBars || c->type() == Graph::HorizontalBars ||
 			c->type() == Graph::Histogram || c->type() == Graph::Box){
 			QRect lr = QRect(x, y - l/4, l, l/2);
-            p->setBrush(c->brush());
-            QwtPainter::drawRect(p, lr);
-        } else
-            QwtPainter::drawLine(p, x, y, x + l, y);
-    }
+			p->setBrush(c->brush());
+			QwtPainter::drawRect(p, lr);
+
+			p->restore();
+			return;
+		} else
+			QwtPainter::drawLine(p, x, y, x + l, y);
+	}
 
 	QwtSymbol symb = c->symbol();
-    int symb_size = symb.size().width();
-    if (symb_size > 15)
-        symb_size = 15;
-    else if (symb_size < 3)
-        symb_size = 3;
-    symb.setSize(symb_size);
-    symb.draw(p, x + l/2, y);
-    p->restore();
+	int symb_size = symb.size().width();
+	if (symb_size > 15)
+		symb_size = 15;
+	else if (symb_size < 3)
+		symb_size = 3;
+	symb.setSize(symb_size);
+
+	QPen pen = symb.pen();
+	pen.setCosmetic(false);
+	symb.setPen(pen);
+
+	symb.draw(p, x + l/2, y);
+	p->restore();
 }
 
 void LegendWidget::drawText(QPainter *p, const QRect& rect,
@@ -562,33 +577,20 @@ QString LegendWidget::parse(const QString& str)
 		QString spec = str.mid(pos + 2, pos2 - pos - 2);
 		QStringList lst = spec.split(",");
 		if (!lst.isEmpty()){
-		    int lcmd=0;
-			if (lst.count()>=2)
-			{
+			int lcmd = -1;
+			if (lst.count() >= 2){
 				if (lst[1].contains("@d",Qt::CaseInsensitive)) //look for d cmd (use curve name), default
-				{
-					  lcmd=0;
-				}
-				if (lst[1].contains("@c",Qt::CaseInsensitive)) //look for c cmd (use column name)
-				{
-					  lcmd=1;
-				}
-				if (lst[1].contains("@w",Qt::CaseInsensitive)) //look for w cmd (use table name)
-				{
-					  lcmd=2;
-				}
-				if (lst[1].contains("@wl",Qt::CaseInsensitive)) //look for wl cmd (use table label)
-				{
-					  lcmd=3;
-				}
-				if (lst[1].contains("@l",Qt::CaseInsensitive)) //look for l cmd (use column comment)
-				{
-					  lcmd=4;
-				}
-				if (lst[1].contains("@u",Qt::CaseInsensitive)) //look for c cmd (use column comment)
-				{
-					  lcmd=5;
-				}
+					lcmd=0;
+				else if (lst[1].contains("@c",Qt::CaseInsensitive)) //look for c cmd (use column name)
+					lcmd=1;
+				else if (lst[1].contains("@w",Qt::CaseInsensitive)) //look for w cmd (use table name)
+					lcmd=2;
+				else if (lst[1].contains("@wl",Qt::CaseInsensitive)) //look for wl cmd (use table label)
+					lcmd=3;
+				else if (lst[1].contains("@l",Qt::CaseInsensitive)) //look for l cmd (use column comment)
+					lcmd=4;
+				else if (lst[1].contains("@u",Qt::CaseInsensitive)) //look for c cmd (use column comment)
+					lcmd=5;
 			}
 
 			int point = 0;
@@ -601,8 +603,16 @@ QString LegendWidget::parse(const QString& str)
 					t = ((DataCurve *)c)->table();
 
 				if (t){
-					switch(lcmd)
+					switch(lcmd){
+						case -1: //use table comment if not empty else use curve title
 						{
+							QString comment = t->comment(t->colIndex(c->title().text()));
+							if (!comment.isEmpty())
+								s = s.replace(pos, pos2-pos+1, comment.replace("\n", " "));
+							else
+								s = s.replace(pos, pos2-pos+1, c->title().text());
+							break;
+						}
 						case 0: //use curve title
 						{
 							s = s.replace(pos, pos2-pos+1, c->title().text());
@@ -636,12 +646,12 @@ QString LegendWidget::parse(const QString& str)
 								{
 									int col = t->colIndex(c->title().text()); //use y column
 									int row = lst[2].toInt() - 1;
-									s = s.replace(pos, pos2-pos+1, t->text(row, col));
+									s = s.replace(pos, pos2-pos+1, t->text(((DataCurve *)c)->tableRow(row), col));
 									break;
 								}
 								case 4:  //4 arguments, display cell contents
 								{
-									int col = lst[0].toInt() - 1;
+									int col = lst[3].toInt() - 1;
 									int row = lst[2].toInt() - 1;
 									s = s.replace(pos, pos2-pos+1, t->text(row, col));
 									break;
@@ -651,14 +661,20 @@ QString LegendWidget::parse(const QString& str)
 						}
 					}
 				}
-
+			} else {
+				Table *t = d_plot->multiLayer()->applicationWindow()->table(lst[0]);
+				if (t && lst.size() >= 3){
+					int col = lst[1].toInt() - 1;
+					int row = lst[2].toInt() - 1;
+					s = s.replace(pos, pos2-pos+1, t->text(row, col));
+				}
 			}
 			aux = aux.right(aux.length() - pos2 - 1);
 		}
     }
 
 	if (plot()->isExportingTeX()){
-		if (!d_tex_output)
+		if (!d_tex_output && plot()->escapeTeXStrings())
 			s = Graph::escapeTeXSpecialCharacters(s);
 
 		s = Graph::texSuperscripts(s);
@@ -719,6 +735,7 @@ void LegendWidget::clone(LegendWidget* t)
 	setFont(t->font());
 	setText(t->text());
 	setOriginCoord(t->xValue(), t->yValue());
+	setAttachPolicy(t->attachPolicy());
 }
 
 QString LegendWidget::saveToString()
@@ -767,6 +784,8 @@ void LegendWidget::restore(Graph *g, const QStringList& lst)
 			y = s.remove("<y>").remove("</y>").toDouble();
 		else if (s.contains("<attachTo>"))
 			l->setAttachPolicy((FrameWidget::AttachPolicy)s.remove("<attachTo>").remove("</attachTo>").toInt());
+		else if (s.contains("<onTop>"))
+			l->setOnTop(s.remove("<onTop>").remove("</onTop>").toInt());
 		else if (s.contains("<Text>")){
 			QStringList txt;
 			while ( s != "</Text>" ){
@@ -818,16 +837,11 @@ void LegendWidget::setAngle(int angle)
 */
 QSize LegendWidget::textSize(QPainter *p, const QwtText& text)
 {
-	QSize size = text.textSize(text.font());
-	QwtMetricsMap map;
-	map.setMetrics(this, p->device());
-	if (!map.isIdentity()){
-		QString s = text.text();
-		if (s.contains("<sub>") || s.contains("<sup>")){
-			int width = size.width() + QFontMetrics(text.font(), p->device()).boundingRect(" ").width();
-			size =  QSize(width, size.height());
-		} else
-			size = QFontMetrics(text.font(), p->device()).boundingRect(s + " ").size();
+	QSize size = text.textSize();
+	QString s = text.text();
+	if (s.contains("<sub>") || s.contains("<sup>")){
+		int width = size.width() + QFontMetrics(text.font(), p->device()).boundingRect(" ").width();
+		size =  QSize(width, size.height());
 	}
 	return size;
 }
