@@ -27,17 +27,69 @@
  *                                                                         *
  ***************************************************************************/
 #include "PlotCurve.h"
+#include "Graph.h"
 #include "ScaleDraw.h"
-#include "../SymbolBox.h"
-#include "../PatternBox.h"
 #include "ScaleEngine.h"
+
+#include <SymbolBox.h>
+#include <PatternBox.h>
+
 #include <QDateTime>
-#include <QMessageBox>
 #include <QPainter>
 #include <qwt_symbol.h>
+#include <qwt_painter.h>
+#include <qwt_curve_fitter.h>
+
+PlotCurve::PlotCurve(const QString& name): QwtPlotCurve(name),
+d_type(0),
+d_plot_style(0),
+d_x_offset(0.0),
+d_y_offset(0.0),
+d_side_lines(false),
+d_skip_symbols(1)
+{
+    setPaintAttribute(PaintFiltered);
+    setPaintAttribute(ClipPolygons);
+	setCurveAttribute(QwtPlotCurve::Fitted, false);
+}
+
+QwtDoubleRect PlotCurve::boundingRect() const
+{
+	QwtDoubleRect r = QwtPlotCurve::boundingRect();
+
+	double percent = 0.01;
+
+	double dw = percent*fabs(r.right() - r.left());
+	double left = r.left() - dw;
+	if (left <= 0.0){
+		ScaleEngine *sc_engine = (ScaleEngine *)this->plot()->axisScaleEngine(xAxis());
+		if (sc_engine && (sc_engine->type() == ScaleTransformation::Log10 ||
+			sc_engine->type() == ScaleTransformation::Log2 ||
+			sc_engine->type() == ScaleTransformation::Ln))
+			left = r.left();
+	}
+
+	r.setLeft(left);
+	r.setRight(r.right() + dw);
+
+	double dh = percent*fabs(r.top() - r.bottom());
+	r.setBottom(r.bottom() + dh);
+
+	double top = r.top() - dh;
+	if (top <= 0.0){
+		ScaleEngine *sc_engine = (ScaleEngine *)this->plot()->axisScaleEngine(yAxis());
+		if (sc_engine && (sc_engine->type() == ScaleTransformation::Log10 ||
+			sc_engine->type() == ScaleTransformation::Log2 ||
+			sc_engine->type() == ScaleTransformation::Ln))
+			top = r.top();
+	}
+	r.setTop(top);
+
+	return r;
+}
 
 QString PlotCurve::saveCurveLayout()
-{			
+{
 	QString s = "<Style>" + QString::number(d_plot_style) + "</Style>\n";
 	if (d_plot_style == Graph::Spline)
 		s += "<LineStyle>5</LineStyle>\n";
@@ -45,7 +97,7 @@ QString PlotCurve::saveCurveLayout()
 		s += "<LineStyle>6</LineStyle>\n";
 	else
 		s += "<LineStyle>" + QString::number(this->style()) + "</LineStyle>\n";
-			
+
 	QPen pen = this->pen();
 	if (pen.style() != Qt::NoPen){
 		s += "<Pen>\n";
@@ -54,26 +106,27 @@ QString PlotCurve::saveCurveLayout()
 		s += "\t<Width>" + QString::number(pen.widthF()) + "</Width>\n";
 		s += "</Pen>\n";
 	}
-	
+
 	QBrush brush = this->brush();
 	if (brush.style() != Qt::NoBrush){
 		s += "<Brush>\n";
 		s += "\t<Color>" + brush.color().name() + "</Color>\n";
+		s += "\t<Alpha>" + QString::number(brush.color().alpha()) + "</Alpha>\n";
 		s += "\t<Style>" + QString::number(PatternBox::patternIndex(brush.style())) + "</Style>\n";
 		s += "</Brush>\n";
 	}
-	
+
 	const QwtSymbol symbol = this->symbol();
 	if (symbol.style() != QwtSymbol::NoSymbol){
 		s += "<Symbol>\n";
 		s += "\t<Style>" + QString::number(SymbolBox::symbolIndex(symbol.style())) + "</Style>\n";
 		s += "\t<Size>" + QString::number(symbol.size().width()) + "</Size>\n";
-	
+
 		s += "\t<SymbolPen>\n";
 		s += "\t\t<Color>" + symbol.pen().color().name() + "</Color>\n";
 		s += "\t\t<Width>" + QString::number(symbol.pen().widthF()) + "</Width>\n";
 		s += "\t</SymbolPen>\n";
-	
+
 		brush = this->brush();
 		if (brush.style() != Qt::NoBrush){
 			s += "\t<SymbolBrush>\n";
@@ -85,6 +138,7 @@ QString PlotCurve::saveCurveLayout()
 	}
 	s += "<xAxis>" + QString::number(xAxis()) + "</xAxis>\n";
 	s += "<yAxis>" + QString::number(yAxis()) + "</yAxis>\n";
+	s += "<CurveType>" + QString::number(curveType()) + "</CurveType>\n";
 	s += "<Visible>" + QString::number(isVisible()) + "</Visible>\n";
 	return s;
 }
@@ -92,10 +146,11 @@ QString PlotCurve::saveCurveLayout()
 void PlotCurve::restoreCurveLayout(const QStringList& lst)
 {
 	QStringList::const_iterator line = lst.begin();
-	for (line++; line != lst.end(); line++){
-        QString s = *line;
+	for (line = lst.begin(); line != lst.end(); line++){
+        QString s = (*line).stripWhiteSpace();
         if (s == "<Pen>"){
 			QPen pen;
+			pen.setCosmetic(true);
 			while(s != "</Pen>"){
 				s = (*(++line)).stripWhiteSpace();
 				if (s.contains("<Color>"))
@@ -103,7 +158,7 @@ void PlotCurve::restoreCurveLayout(const QStringList& lst)
 				else if (s.contains("<Style>"))
 					pen.setStyle(Graph::getPenStyle(s.remove("<Style>").remove("</Style>").toInt()));
 				else if (s.contains("<Width>"))
-					pen.setWidthF(s.remove("<Width>").remove("</Width>").toDouble());	
+					pen.setWidthF(s.remove("<Width>").remove("</Width>").toDouble());
 			}
 			setPen(pen);
 		} else if (s == "<Brush>"){
@@ -112,7 +167,11 @@ void PlotCurve::restoreCurveLayout(const QStringList& lst)
 				s = (*(++line)).stripWhiteSpace();
 				if (s.contains("<Color>"))
 					brush.setColor(QColor(s.remove("<Color>").remove("</Color>")));
-				else if (s.contains("<Style>"))
+				else if (s.contains("<Alpha>")){
+					QColor c = brush.color();
+					c.setAlpha(s.remove("<Alpha>").remove("</Alpha>").toInt());
+					brush.setColor(c);
+				} else if (s.contains("<Style>"))
 					brush.setStyle(PatternBox::brushStyle(s.remove("<Style>").remove("</Style>").toInt()));
 			}
 			setBrush(brush);
@@ -133,9 +192,10 @@ void PlotCurve::restoreCurveLayout(const QStringList& lst)
 						else if (s.contains("<Style>"))
 							pen.setStyle(Graph::getPenStyle(s.remove("<Style>").remove("</Style>").toInt()));
 						else if (s.contains("<Width>"))
-							pen.setWidthF(s.remove("<Width>").remove("</Width>").toDouble());	
+							pen.setWidthF(s.remove("<Width>").remove("</Width>").toDouble());
 					}
-				symbol.setPen(pen);
+					pen.setCosmetic(true);
+					symbol.setPen(pen);
 				} else if (s == "<SymbolBrush>"){
 					QBrush brush;
 					while(s != "</SymbolBrush>"){
@@ -148,16 +208,257 @@ void PlotCurve::restoreCurveLayout(const QStringList& lst)
 					symbol.setBrush(brush);
 				}
 				setSymbol(symbol);
-			}		
+			}
 		} else if (s.contains("<xAxis>"))
 			setXAxis(s.remove("<xAxis>").remove("</xAxis>").toInt());
 		else if (s.contains("<yAxis>"))
 			setYAxis(s.remove("<yAxis>").remove("</yAxis>").toInt());
+		else if (s.contains("<CurveType>"))
+			setCurveType((QwtPlotCurve::CurveType)s.remove("<CurveType>").remove("</CurveType>").toInt());
 		else if (s.contains("<Visible>"))
 			setVisible(s.remove("<Visible>").remove("</Visible>").toInt());
 	}
 }
 
+void PlotCurve::drawCurve(QPainter *p, int style, const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const
+{
+	if(d_side_lines)
+		drawSideLines(p, xMap, yMap, from, to);
+	QwtPlotCurve::drawCurve(p, style, xMap, yMap, from, to);
+}
+
+/*!
+  Draw sticks
+
+  \param painter Painter
+  \param xMap x map
+  \param yMap y map
+  \param from index of the first point to be painted
+  \param to index of the last point to be painted
+
+  \sa draw(), drawCurve(), drawDots()
+*/
+void PlotCurve::drawSticks(QPainter *painter,
+    const QwtScaleMap &xMap, const QwtScaleMap &yMap,
+    int from, int to) const
+{
+	if (d_skip_symbols < 2){
+		QwtPlotCurve::drawSticks(painter, xMap, yMap, from, to);
+		return;
+	}
+
+    int x0 = xMap.transform(baseline());
+    int y0 = yMap.transform(baseline());
+
+    for (int i = from; i <= to; i += d_skip_symbols){
+        const int xi = xMap.transform(x(i));
+        const int yi = yMap.transform(y(i));
+
+        if (curveType() == Xfy)
+            QwtPainter::drawLine(painter, x0, yi, xi, yi);
+        else
+            QwtPainter::drawLine(painter, xi, y0, xi, yi);
+    }
+}
+
+void PlotCurve::setSkipSymbolsCount(int count)
+{
+	if (count < 1 || count > dataSize())
+		return;
+
+	d_skip_symbols = count;
+}
+
+/*!
+  \brief Draw symbols
+  \param painter Painter
+  \param symbol Curve symbol
+  \param xMap x map
+  \param yMap y map
+  \param from index of the first point to be painted
+  \param to index of the last point to be painted
+
+  \sa setSymbol(), draw(), drawCurve()
+*/
+void PlotCurve::drawSymbols(QPainter *painter, const QwtSymbol &symbol,
+    const QwtScaleMap &xMap, const QwtScaleMap &yMap,
+    int from, int to) const
+{
+	if (d_skip_symbols < 2){
+		QwtPlotCurve::drawSymbols(painter, symbol, xMap, yMap, from, to);
+		return;
+	}
+
+    painter->setBrush(symbol.brush());
+    painter->setPen(QwtPainter::scaledPen(symbol.pen()));
+
+    const QwtMetricsMap &metricsMap = QwtPainter::metricsMap();
+
+    QRect rect;
+    rect.setSize(metricsMap.screenToLayout(symbol.size()));
+
+	for (int i = from; i <= to; i += d_skip_symbols){
+		const int xi = xMap.transform(x(i));
+		const int yi = yMap.transform(y(i));
+
+		rect.moveCenter(QPoint(xi, yi));
+		symbol.draw(painter, rect);
+	}
+}
+
+void PlotCurve::drawSideLines(QPainter *p, const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const
+{
+	if (!p || dataSize() <= 0)
+		return;
+
+	if (to < 0)
+		to = dataSize() - 1;
+
+	p->save();
+	QPen pen = p->pen();
+	pen.setCapStyle(Qt::FlatCap);
+	pen.setJoinStyle(Qt::MiterJoin);
+	p->setPen(pen);
+
+	double lw = 0.5*pen.widthF();
+	const double xl = xMap.xTransform(x(from));
+	const double xr = xMap.xTransform(x(to));
+	const double yl = yMap.xTransform(y(from)) - lw;
+	const double yr = yMap.xTransform(y(to)) - lw;
+	const double base = yMap.xTransform(baseline());
+
+	p->drawLine(QPointF(xl, yl), QPointF(xl, base));
+	p->drawLine(QPointF(xr, yr), QPointF(xr, base));
+
+	p->restore();
+
+	/*int size = to - from + 1;
+    if ( size <= 0 )
+        return;
+
+	size += 2;
+
+	p->save();
+	QPen pen = p->pen();
+	pen.setCapStyle(Qt::FlatCap);
+	pen.setJoinStyle(Qt::MiterJoin);
+	p->setPen(pen);
+
+	const double xl = xMap.xTransform(minXValue());
+	const double xr = xMap.xTransform(maxXValue());
+	const double base = yMap.xTransform(baseline());
+
+    QwtPolygonF polyline;
+    if ( testCurveAttribute(QwtPlotCurve::Fitted) && curveFitter () )
+    {
+        // Transform x and y values to window coordinates
+        // to avoid a distinction between linear and
+        // logarithmic scales.
+
+        QPolygonF points(size);
+        for (int i = from; i <= to; i++)
+        {
+            QwtDoublePoint &p = points[i];
+            p.setX( xMap.xTransform(x(i)) );
+            p.setY( yMap.xTransform(y(i)) );
+        }
+
+        points = curveFitter()->fitCurve(points);
+        size = points.size();
+
+        if ( size == 0 )
+            return;
+
+        // Round QwtDoublePoints to QPoints
+        // When Qwt support for Qt3 has been dropped (Qwt 6.x)
+        // we will use a doubles for painting and the following
+        // step will be obsolete.
+
+        polyline.resize(size);
+
+        const QwtDoublePoint *p = points.data();
+        QPointF *pl = polyline.data();
+        if (testPaintAttribute(QwtPlotCurve::PaintFiltered))
+        {
+
+            QPointF pp(p[0].x(), p[0].y());
+            pl[0] = pp;
+
+            int count = 1;
+            for (int i = 1; i < size; i++)
+            {
+                const QPointF pi(p[i].x(), p[i].y());
+                if ( pi != pp )
+                {
+                    pl[count++] = pi;
+                    pp = pi;
+                }
+            }
+            if ( count != size )
+                polyline.resize(count);
+        }
+        else
+        {
+            for ( int i = 0; i < size; i++ )
+            {
+                pl[i].setX( p[i].x() );
+                pl[i].setY( p[i].y() );
+            }
+        }
+    }
+    else
+    {
+        polyline.resize(size);
+
+        if ( testPaintAttribute(QwtPlotCurve::PaintFiltered) )
+        {
+        	QPointF pp(xl, base);
+            polyline[0] = pp;
+
+            int count = 1;
+            for (int i = from; i <= to; i++)
+            {
+                const QPointF pi(xMap.xTransform(x(i)), yMap.xTransform(y(i)));
+                if ( pi != pp )
+                {
+                    polyline[count] = pi;
+                    count++;
+
+                    pp = pi;
+                }
+            }
+
+            polyline[size - 1] = QPointF(xr, base);
+			count++;
+
+            if ( count != size )
+                polyline.resize(count);
+        }
+        else
+        {
+            polyline[0] = QPointF( xl, base );
+
+            for (int i = from; i <= to; i++)
+            {
+                const QPointF pi(xMap.xTransform(x(i)), yMap.xTransform(y(i)));
+                polyline[i - from + 1] = pi;
+            }
+
+            polyline[size - 1] = QPointF(xr, base);
+        }
+    }
+
+    if ( testPaintAttribute(QwtPlotCurve::ClipPolygons) )
+        polyline = QwtClipper::clipPolygonF(p->window(), polyline);
+
+    QwtPainter::drawPolyline(p, polyline);
+
+    if ( brush().style() != Qt::NoBrush )
+        fillCurve(p, xMap, yMap, polyline);
+
+	p->restore();*/
+}
+
 DataCurve::DataCurve(Table *t, const QString& xColName, const QString& name, int startRow, int endRow):
     PlotCurve(name),
 	d_table(t),
@@ -256,6 +557,33 @@ bool DataCurve::updateData(Table *t, const QString& colName)
 	return true;
 }
 
+void DataCurve::enableSpeedMode()
+{
+	Graph *g = (Graph *)plot();
+	if (!g)
+		return;
+
+	if (g->getDouglasPeukerTolerance() != 0.0 && dataSize() >= g->speedModeMaxPoints()){
+		setCurveAttribute(QwtPlotCurve::Fitted);
+
+		QwtWeedingCurveFitter *fitter = new QwtWeedingCurveFitter(g->getDouglasPeukerTolerance());
+		setCurveFitter(fitter);
+	}
+	//g->replot();
+}
+
+void DataCurve::drawCurve(QPainter *p, int style, const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const
+{
+	if (d_data_ranges.empty())
+		return PlotCurve::drawCurve(p, style, xMap, yMap, from, to);
+
+	if(d_side_lines)
+		drawSideLines(p, xMap, yMap, from, to);
+
+	for (unsigned int i = 0; i < d_data_ranges.size(); i++)
+		QwtPlotCurve::drawCurve(p, style, xMap, yMap, d_data_ranges[i].from, d_data_ranges[i].to);
+}
+
 void DataCurve::loadData()
 {
 	Graph *g = (Graph *)plot();
@@ -277,35 +605,50 @@ void DataCurve::loadData()
 
 	QStringList xLabels, yLabels;// store text labels
 
+	int xAxis = QwtPlot::xBottom;
+	if (d_type == Graph::HorizontalBars)
+		xAxis = QwtPlot::yLeft;
+
 	QTime time0;
 	QDateTime date0;
 	QString date_time_fmt = d_table->columnFormat(xcol);
 	if (xColType == Table::Time){
-		for (int i = d_start_row; i <= d_end_row; i++ ){
-			QString xval=d_table->text(i,xcol);
-			if (!xval.isEmpty()){
-				time0 = QTime::fromString (xval, date_time_fmt);
-				if (time0.isValid())
-					break;
+		ScaleDraw *sd = (ScaleDraw *)g->axisScaleDraw(xAxis);
+		if (sd && sd->scaleType() == ScaleDraw::Time)
+			time0 = sd->dateTimeOrigin().time();
+		else {
+			for (int i = d_start_row; i <= d_end_row; i++ ){
+				QString xval=d_table->text(i,xcol);
+				if (!xval.isEmpty()){
+					time0 = QTime::fromString (xval, date_time_fmt);
+					if (time0.isValid())
+						break;
+				}
 			}
 		}
 	} else if (xColType == Table::Date){
-		for (int i = d_start_row; i <= d_end_row; i++ ){
-			QString xval=d_table->text(i,xcol);
-			if (!xval.isEmpty()){
-				date0 = QDateTime::fromString (xval, date_time_fmt);
-				if (date0.isValid())
-					break;
+		ScaleDraw *sd = (ScaleDraw *)g->axisScaleDraw(xAxis);
+		if (sd && sd->scaleType() == ScaleDraw::Date)
+			date0 = sd->dateTimeOrigin();
+		else {
+			for (int i = d_start_row; i <= d_end_row; i++ ){
+				QString xval=d_table->text(i,xcol);
+				if (!xval.isEmpty()){
+					date0 = QDateTime::fromString (xval, date_time_fmt);
+					if (date0.isValid())
+						break;
+				}
 			}
 		}
 	}
 
-	int size = 0;
+	int size = 0, from = 0;
+	d_data_ranges.clear();
 	for (int i = d_start_row; i <= d_end_row; i++ ){
 		QString xval = d_table->text(i,xcol);
 		QString yval = d_table->text(i,ycol);
 		if (!xval.isEmpty() && !yval.isEmpty()){
-		    bool valid_data = true;
+			bool valid_data = true;
 			if (xColType == Table::Text){
 				xLabels << xval;
 				X[size] = (double)(size + 1);
@@ -326,13 +669,20 @@ void DataCurve::loadData()
 			} else
 				Y[size] = g->locale().toDouble(yval, &valid_data);
 
-            if (valid_data)
-                size++;
+			if (valid_data)
+				size++;
+		} else if (from < size){
+			DataRange range;
+			range.from = from;
+			range.to = size - 1;
+			d_data_ranges.push_back(range);
+
+			from = size;
 		}
 	}
 
-    X.resize(size);
-    Y.resize(size);
+	X.resize(size);
+	Y.resize(size);
 
 	if (!size){
 		remove();
@@ -349,31 +699,27 @@ void DataCurve::loadData()
 		}
 
 		if (xColType == Table::Text){
-			if (d_type == Graph::HorizontalBars)
-				g->setLabelsTextFormat(QwtPlot::yLeft, ScaleDraw::Text, d_x_column, xLabels);
-			else
-                g->setLabelsTextFormat(QwtPlot::xBottom, ScaleDraw::Text, d_x_column, xLabels);
+			g->setLabelsTextFormat(xAxis, ScaleDraw::Text, d_x_column, xLabels);
 		} else if (xColType == Table::Time || xColType == Table::Date){
-			int axis = QwtPlot::xBottom;
-			if (d_type == Graph::HorizontalBars)
-                axis = QwtPlot::yLeft;
-            ScaleDraw *old_sd = (ScaleDraw *)g->axisScaleDraw(axis);
-            ScaleDraw *sd = new ScaleDraw(g, old_sd);
-            if (xColType == Table::Date)
-                sd->setDateTimeOrigin(date0);
-            else
-                sd->setDateTimeOrigin(QDateTime(QDate::currentDate(), time0));
-            g->setAxisScaleDraw(axis, sd);
+			ScaleDraw *old_sd = (ScaleDraw *)g->axisScaleDraw(xAxis);
+			ScaleDraw *sd = new ScaleDraw(g, old_sd);
+			if (xColType == Table::Date)
+				sd->setDateTimeOrigin(date0);
+			else
+				sd->setDateTimeOrigin(QDateTime(QDate::currentDate(), time0));
+			g->setAxisScaleDraw(xAxis, sd);
 		}
 
 		if (yColType == Table::Text)
-            g->setLabelsTextFormat(QwtPlot::yLeft, ScaleDraw::Text, title().text(), yLabels);
+			g->setLabelsTextFormat(QwtPlot::yLeft, ScaleDraw::Text, title().text(), yLabels);
 	}
 
-    if (!d_labels_list.isEmpty()){
-        ((Graph*)plot())->updatePlot();
-        loadLabels();
-    }
+	enableSpeedMode();
+
+	if (!d_labels_list.isEmpty()){
+		((Graph*)plot())->updatePlot();
+		loadLabels();
+	}
 }
 
 void DataCurve::removeErrorBars(DataCurve *c)
@@ -409,6 +755,8 @@ void DataCurve::setVisible(bool on)
 	QwtPlotCurve::setVisible(on);
 	foreach(DataCurve *c, d_error_bars)
 		c->setVisible(on);
+	foreach(PlotMarker *m, d_labels_list)
+		m->setVisible(on);
 }
 
 int DataCurve::tableRow(int point)
@@ -416,6 +764,15 @@ int DataCurve::tableRow(int point)
     if (!d_table)
         return -1;
 
+	if (d_type == Graph::Pie){
+		double y_val = y(point);
+		int ycol = d_table->colIndex(title().text());
+		for (int i = d_start_row; i <= d_end_row; i++ ){
+			if (d_table->cell(i, ycol) == y_val)
+				return i;
+		}
+	}
+
 	int xcol = d_table->colIndex(d_x_column);
 	int ycol = d_table->colIndex(title().text());
 
@@ -447,6 +804,12 @@ int DataCurve::tableRow(int point)
                     return i;
             }
         }
+    } else if (xColType == Table::Text){
+		double y_val = y(point);
+        for (int i = d_start_row; i <= d_end_row; i++ ){
+			if (d_table->cell(i, ycol) == y_val)
+				return i;
+        }
     }
 
 	double x_val = x(point);
@@ -455,7 +818,8 @@ int DataCurve::tableRow(int point)
 		if (d_table->cell(i, xcol) == x_val && d_table->cell(i, ycol) == y_val)
 			return i;
 	}
-	return -1;
+
+	return point;
 }
 
 void DataCurve::setLabelsColumnName(const QString& name)
@@ -684,6 +1048,7 @@ void DataCurve::clone(DataCurve* c)
 	d_labels_align = c->labelsAlignment();
 	d_labels_x_offset = c->labelsXOffset();
 	d_labels_y_offset = c->labelsYOffset();
+	d_skip_symbols = c->skipSymbolsCount();
 
     if (!c->labelsColumnName().isEmpty()){
         plot()->replot();
@@ -696,18 +1061,18 @@ QString DataCurve::saveToString()
 	if (!validCurveType())
 		return QString();
 
-    if (d_labels_list.isEmpty() || type() == Graph::Function || type() == Graph::Box)
-        return QString();
+	QString s = QString::null;
+	if (d_skip_symbols > 1)
+		s += "<SkipPoints>" + QString::number(d_skip_symbols) + "</SkipPoints>\n";
 
-    QString s = "<CurveLabels>\n";
+    if (d_labels_list.isEmpty() || type() == Graph::Function || type() == Graph::Box)
+        return s;
+
+    s += "<CurveLabels>\n";
     s += "\t<column>" + d_labels_column + "</column>\n";
     s += "\t<color>" + d_labels_color.name() + "</color>\n";
     s += "\t<whiteOut>" + QString::number(d_white_out_labels) + "</whiteOut>\n";
-    s += "\t<font>" + d_labels_font.family() + "\t";
-    s += QString::number(d_labels_font.pointSize()) + "\t";
-    s += QString::number(d_labels_font.bold()) + "\t";
-    s += QString::number(d_labels_font.italic()) + "\t";
-    s += QString::number(d_labels_font.underline()) + "</font>\n";
+    s += "\t<Font>" + d_labels_font.toString() + "</Font>\n";
     s += "\t<angle>" + QString::number(d_labels_angle) + "</angle>\n";
     s += "\t<justify>" + QString::number(d_labels_align) + "</justify>\n";
     if (d_labels_x_offset != 0.0)
@@ -717,22 +1082,68 @@ QString DataCurve::saveToString()
     return s + "</CurveLabels>\n";
 }
 
+void DataCurve::restoreLabels(const QStringList& lst)
+{
+    QString labelsColumn = QString();
+    int xoffset = 0, yoffset = 0;
+  	QStringList::const_iterator line = lst.begin();
+  	QString s = *line;
+    if (s.contains("<column>"))
+        labelsColumn = s.remove("<column>").remove("</column>").trimmed();
+
+  	for (line++; line != lst.end(); line++){
+        s = *line;
+        if (s.contains("<color>"))
+            setLabelsColor(QColor(s.remove("<color>").remove("</color>").trimmed()));
+        else if (s.contains("<whiteOut>"))
+            setLabelsWhiteOut(s.remove("<whiteOut>").remove("</whiteOut>").toInt());
+        else if (s.contains("<font>")){//versions older than 0.9.7.8
+            QStringList fontList = s.remove("<font>").remove("</font>").trimmed().split("\t");
+            QFont font = QFont(fontList[0], fontList[1].toInt());
+            if (fontList.count() >= 3)
+                font.setBold(fontList[2].toInt());
+            if (fontList.count() >= 4)
+                font.setItalic(fontList[3].toInt());
+            if (fontList.count() >= 5)
+                font.setUnderline(fontList[4].toInt());
+            setLabelsFont(font);
+        } else if (s.contains("<Font>")){
+            s.remove("<Font>").remove("</Font>").trimmed();
+            QFont fnt;
+			fnt.fromString(s);
+            setLabelsFont(fnt);
+        }else if (s.contains("<angle>"))
+            setLabelsRotation(s.remove("<angle>").remove("</angle>").toDouble());
+        else if (s.contains("<justify>"))
+            setLabelsAlignment(s.remove("<justify>").remove("</justify>").toInt());
+        else if (s.contains("<xoffset>"))
+            xoffset = s.remove("<xoffset>").remove("</xoffset>").toInt();
+        else if (s.contains("<yoffset>"))
+            yoffset = s.remove("<yoffset>").remove("</yoffset>").toInt();
+    }
+    setLabelsOffset(xoffset, yoffset);
+    setLabelsColumnName(labelsColumn);
+}
+
 bool DataCurve::selectedLabels(const QPoint& pos)
 {
 	if (!validCurveType())
 		return false;
 
     QwtPlot *d_plot = plot();
-    if (!d_plot)
+    if (!d_plot || ((Graph *)d_plot)->hasActiveTool())
         return false;
 
-
     bool selected = false;
 	d_selected_label = NULL;
     foreach(PlotMarker *m, d_labels_list){
         int x = d_plot->transform(xAxis(), m->xValue());
         int y = d_plot->transform(yAxis(), m->yValue());
-        if (QRect(QPoint(x, y), m->label().textSize()).contains(pos)){
+
+        QMatrix wm;
+        wm.translate(x, y);
+		wm.rotate(-d_labels_angle);
+        if (wm.mapToPolygon(QRect(QPoint(0, 0), m->label().textSize())).containsPoint(pos, Qt::OddEvenFill)){
 			d_selected_label = m;
 			d_click_pos_x = d_plot->invTransform(xAxis(), pos.x());
 			d_click_pos_y = d_plot->invTransform(yAxis(), pos.y());
@@ -789,7 +1200,7 @@ bool DataCurve::validCurveType()
 {
 	int style = type();
 	if (style == Graph::Function || style == Graph::Box ||
-		style == Graph::Pie || style == Graph::ErrorBars ||
+		style == Graph::Pie || //style == Graph::ErrorBars ||
 		style == Graph::ColorMap ||  style == Graph::GrayScale ||
 		style == Graph::Contour || style == Graph::ImagePlot)
 		return false;
@@ -826,7 +1237,9 @@ void DataCurve::moveLabels(const QPoint& pos)
 
 PlotMarker::PlotMarker(int index, double angle):QwtPlotMarker(),
 	d_index(index),
-	d_angle(angle)
+	d_angle(angle),
+	d_label_x_offset(0.0),
+	d_label_y_offset(0.0)
 {}
 
 void PlotMarker::draw(QPainter *p, const QwtScaleMap &xMap, const QwtScaleMap &yMap, const QRect &) const
