@@ -31,7 +31,6 @@
 #include "PlotCurve.h"
 #include "MultiLayer.h"
 #include <QwtErrorPlotCurve.h>
-#include <cursors.h>
 #include <ApplicationWindow.h>
 
 #include <qwt_symbol.h>
@@ -88,7 +87,7 @@ RangeSelectorTool::RangeSelectorTool(Graph *graph, const QObject *status_target,
 
 	setTrackerMode(QwtPicker::AlwaysOn);
 	setSelectionFlags(QwtPicker::PointSelection | QwtPicker::ClickSelection);
-	d_graph->canvas()->setCursor(QCursor(QPixmap(vizor_xpm), -1, -1));
+	d_graph->canvas()->setCursor(QCursor(QPixmap(":/vizor.png"), -1, -1));
 	d_graph->canvas()->setFocus();
 	d_graph->replot();
 
@@ -641,7 +640,7 @@ void RangeSelectorTool::setEnabled(bool on)
 {
     d_enabled = on;
     if (on)
-        d_graph->canvas()->setCursor(QCursor(QPixmap(vizor_xpm), -1, -1));
+		d_graph->canvas()->setCursor(QCursor(QPixmap(":/vizor.png"), -1, -1));
 }
 
 void RangeSelectorTool::setVisible(bool on)
@@ -653,7 +652,7 @@ void RangeSelectorTool::setVisible(bool on)
 
     if (on){
 		setTrackerMode(QwtPicker::AlwaysOn);
-        d_graph->canvas()->setCursor(QCursor(QPixmap(vizor_xpm), -1, -1));
+		d_graph->canvas()->setCursor(QCursor(QPixmap(":/vizor.png"), -1, -1));
 		d_active_marker.attach(d_graph);
 		d_inactive_marker.attach(d_graph);
 	} else {
