@@ -38,6 +38,7 @@
 
 #include <qwt_painter.h>
 #include <qwt_text.h>
+#include <qwt_scale_widget.h>
 
 /*****************************************************************************
  *
@@ -58,7 +59,10 @@ ScaleDraw::ScaleDraw(Graph *plot, const QString& formula):
 	d_name_format(ShortName),
 	d_date_time_origin(QDateTime::currentDateTime()),
 	d_format_info("YYYY-MM-DDTHH:MM:SS"),
-	d_text_labels(QStringList())
+	d_text_labels(QStringList()),
+	d_show_ticks_policy(ShowAll),
+	d_prefix(""),
+	d_suffix("")
 {}
 
 ScaleDraw::ScaleDraw(Graph *plot, const QStringList& labels, const QString& format, ScaleType type):
@@ -74,7 +78,9 @@ ScaleDraw::ScaleDraw(Graph *plot, const QStringList& labels, const QString& form
 	d_name_format(ShortName),
 	d_date_time_origin(QDateTime::currentDateTime()),
 	d_format_info(format),
-	d_text_labels(labels)
+	d_text_labels(labels),
+	d_prefix(""),
+	d_suffix("")
 {}
 
 ScaleDraw::ScaleDraw(Graph *plot, ScaleDraw* sd):
@@ -92,12 +98,24 @@ ScaleDraw::ScaleDraw(Graph *plot, ScaleDraw* sd):
 	d_date_time_origin = sd->d_date_time_origin;
 	d_format_info = sd->d_format_info;
 	d_text_labels = sd->d_text_labels;
+	d_show_ticks_policy = sd->showTicksPolicy();
+	d_prefix = sd->prefix();
+	d_suffix = sd->suffix();
 
 	setLabelAlignment(sd->labelAlignment());
 	setLabelRotation(sd->labelRotation());
 }
 
 QwtText ScaleDraw::label(double value) const
+{
+	QString s = labelString(value);
+	if (s.isEmpty())
+		return QwtText();
+
+	return QwtText(d_prefix + s + d_suffix);
+}
+
+QString ScaleDraw::labelString(double value) const
 {
 	switch (d_type){
 		case Numeric:
@@ -105,11 +123,14 @@ QwtText ScaleDraw::label(double value) const
             QLocale locale = d_plot->locale();
 			if (d_plot->parent())
 				locale = d_plot->multiLayer()->locale();
-			if (d_numeric_format == Superscripts){
+			if ((d_numeric_format == Superscripts)||(d_numeric_format == SuperscriptsGER)){
 				QString txt = locale.toString(transformValue(value), 'e', d_prec);
 				QStringList list = txt.split("e", QString::SkipEmptyParts);
+				if (list.isEmpty())
+					return QString::null;
+
 				if (list[0].toDouble() == 0.0)
-					return QString("0");
+					return "0";
 
 				QString s = list[1];
 				int l = s.length();
@@ -125,79 +146,54 @@ QwtText ScaleDraw::label(double value) const
 					s.prepend(sign);
 
 				if (list[0] == "1")
-					return QwtText("10<sup>" + s + "</sup>");
-				else
-					return QwtText(list[0] + "x10<sup>" + s + "</sup>");
-
-			}
-			else if (d_numeric_format == Engineering){
-
+					return "10<sup>" + s + "</sup>";
+				else{
+					if (d_numeric_format == SuperscriptsGER)
+						return list[0] + "10<sup>" + s + "</sup>";
+					else
+						return list[0] + QString(QChar(0x00D7)) + "10<sup>" + s + "</sup>";
+				}
+			} else if (d_numeric_format == Engineering){
 				QString eng_suff;
-				bool ok;
-				double new_value = QString("%1").arg(value, 0, 'e', d_prec).toDouble(&ok);
+				double new_value = value;
 
-				if(fabs(new_value) >= 1e18)
-				{
+				if(fabs(new_value) >= 1e18){
 					eng_suff = 'E';
 					new_value /= 1e18;
-				}
-				else if(fabs(new_value) >= 1e15)
-				{
+				} else if(fabs(new_value) >= 1e15){
 					eng_suff = 'P';
 					new_value /= 1e15;
-				}
-				else if(fabs(new_value) >= 1e12)
-				{
+				} else if(fabs(new_value) >= 1e12){
 					eng_suff = 'T';
 					new_value /= 1e12;
-				}
-				else if(fabs(new_value) >= 1e9)
-				{
+				} else if(fabs(new_value) >= 1e9){
 					eng_suff = 'G';
 					new_value /= 1e9;
-				}
-				else if(fabs(new_value) >= 1e6)
-				{
+				} else if(fabs(new_value) >= 1e6){
 					eng_suff = 'M';
 					new_value /= 1e6;
-				}
-				else if(fabs(new_value) >= 1e3)
-				{
+				} else if(fabs(new_value) >= 1e3){
 					eng_suff = 'k';
 					new_value /= 1e3;
-				}
-				else if(fabs(new_value) >= 1)
-				{
+				} else if(fabs(new_value) >= 1){
 					eng_suff = "";
-					new_value /= 1;
-				}
-				else if(fabs(new_value) >= 1e-3)
-				{
+					new_value /= 1.0;
+				} else if(fabs(new_value) >= 1e-3){
 					eng_suff = 'm';
 					new_value /= 1e-3;
-				}
-				else if(fabs(new_value) >= 1e-6)
-				{
+				} else if(fabs(new_value) >= 1e-6){
 					eng_suff = '';
 					new_value /= 1e-6;
-				}
-				else if(fabs(new_value) >= 1e-9)
-				{
+				} else if(fabs(new_value) >= 1e-9){
 					eng_suff = 'n';
 					new_value /= 1e-9;
-				}
-				else if(fabs(new_value) >= 1e-12)
-				{
+				} else if(fabs(new_value) >= 1e-12){
 					eng_suff = 'p';
 					new_value /= 1e-12;
-				}
-				else if(fabs(new_value) >= 1e-15)
-				{
+				} else if(fabs(new_value) >= 1e-15){
 					eng_suff = 'f';
 					new_value /= 1e-15;
-				}
-				else
-				{
+				} else {
 					eng_suff = 'a';
 					new_value /= 1e-18;
 				}
@@ -205,12 +201,11 @@ QwtText ScaleDraw::label(double value) const
 				QString txt = locale.toString((new_value), 'f', d_prec);
 
 				if(txt.contains(QRegExp("^0[\\.,]?0*$")))
-					return QwtText("0");
-
-				return QwtText(txt + eng_suff);
+					return "0";
 
+				return txt + eng_suff;
 			} else
-				return QwtText(locale.toString(transformValue(value), d_fmt, d_prec));
+				return locale.toString(transformValue(value), d_fmt, d_prec);
 		break;
 		}
 
@@ -234,7 +229,7 @@ QwtText ScaleDraw::label(double value) const
 					day = (QDate::shortDayName (val)).left(1);
 				break;
 			}
-			return QwtText(day);
+			return day;
 		break;
 		}
 
@@ -258,24 +253,32 @@ QwtText ScaleDraw::label(double value) const
 					day = (QDate::shortMonthName (val)).left(1);
 				break;
 			}
-			return QwtText(day);
+			return day;
 		break;
 		}
 
 		case Time:
-			return QwtText(d_date_time_origin.time().addMSecs((int)value).toString(d_format_info));
+		{
+			QTime time = d_date_time_origin.time().addMSecs((int)value);
+			if (d_format_info == "M")
+				return QString::number(60*time.hour() + time.minute());
+			else if (d_format_info == "S")
+				return QString::number(3600*time.hour() + 60*time.minute() + time.second());
+
+			return time.toString(d_format_info);
 		break;
+		}
 
 		case Date:
-            return QwtText(d_date_time_origin.addSecs((int)value).toString(d_format_info));
+			return d_date_time_origin.addSecs((int)value).toString(d_format_info);
 		break;
 
 		case ColHeader:
 		case Text:
 		{
 			const QwtScaleDiv scDiv = scaleDiv();
-			if (!scDiv.contains (value))
-				return QwtText();
+			if (!scDiv.contains (value) || floor(value) < value)
+				return QString::null;
 
 			QwtValueList ticks = scDiv.ticks (QwtScaleDiv::MajorTick);
 
@@ -316,6 +319,9 @@ QwtText ScaleDraw::label(double value) const
 			    }
 			}
 
+			if (ticks.size() < 2)
+				return QString::null;
+
         	double step = ticks[1] - ticks[0];
         	int index = int(ticks[0] + step*ticks.indexOf(value) - 1);
             int offset = abs((int)floor(break_offset/step));
@@ -326,13 +332,13 @@ QwtText ScaleDraw::label(double value) const
             else
                 index -= offset;
 			if (index >= 0 && index < (int)d_text_labels.count())
-        		return QwtText(d_text_labels[index]);
+				return d_text_labels[index];
 			else
-				return QwtText();
+				return QString::null;
 		break;
 		}
 	}
-	return QwtText();
+	return QString::null;
 }
 
 void ScaleDraw::drawLabel(QPainter *painter, double value) const
@@ -349,6 +355,27 @@ void ScaleDraw::drawLabel(QPainter *painter, double value) const
         	return;
 	}
 
+	QwtValueList majTicks = scaleDiv().ticks(QwtScaleDiv::MajorTick);
+	if (majTicks.contains(value)){
+		switch (d_show_ticks_policy){
+			case ShowAll:
+			break;
+
+			case HideBegin:
+				if (majTicks.first() == value)
+					return;
+			break;
+			case HideEnd:
+				if (majTicks.last() == value)
+					return;
+			break;
+			case HideBeginEnd:
+				if (majTicks.first() == value || majTicks.last() == value)
+					return;
+			break;
+		}
+	}
+
 	QwtText lbl = tickLabel(painter->font(), value);
     if ( lbl.isEmpty() )
         return;
@@ -388,13 +415,13 @@ double ScaleDraw::transformValue(double value) const
 		double lbl=0.0;
 		try{
 			MyParser parser;
-			if (d_formula.contains("x"))
+			if (d_formula.contains("x", Qt::CaseInsensitive))
 				parser.DefineVar("x", &value);
-			else if (d_formula.contains("y"))
+			else if (d_formula.contains("y", Qt::CaseInsensitive))
 				parser.DefineVar("y", &value);
 
-			parser.SetExpr(d_formula.ascii());
-			lbl=parser.Eval();
+			parser.SetExpr(d_formula.lower().ascii());
+			lbl = parser.Eval();
         }
         catch(mu::ParserError &){
 			return 0;
@@ -428,6 +455,8 @@ void ScaleDraw::setNumericFormat(NumericFormat format)
 		case Engineering:
 			d_fmt = 'f';
 		break;
+		case SuperscriptsGER:
+			d_fmt = 'f';
 	}
 }
 
@@ -472,26 +501,33 @@ int ScaleDraw::axis() const
 void ScaleDraw::drawTick(QPainter *p, double value, int len) const
 {
 	ScaleEngine *sc_engine = (ScaleEngine *)d_plot->axisScaleEngine(axis());
-    if (sc_engine->hasBreak()){
-		int align = alignment();
-		bool invertedScale = sc_engine->testAttribute(QwtScaleEngine::Inverted);
-		if (align == BottomScale || align == LeftScale){
-			if (invertedScale && sc_engine->axisBreakRight() == value)
-            	return;
-			if (!invertedScale && sc_engine->axisBreakLeft() == value)
-				return;
-		} else {
-			if (!invertedScale && sc_engine->axisBreakRight() == value)
-				return;
-			if (invertedScale && sc_engine->axisBreakLeft() == value)
-            	return;
-		}
-    }
+	if (sc_engine->hasBreak() && (sc_engine->axisBreakLeft() == value || sc_engine->axisBreakRight() == value))
+		return;
 
 	QwtScaleDiv scDiv = scaleDiv();
-    QwtValueList majTicks = scDiv.ticks(QwtScaleDiv::MajorTick);
-    if (majTicks.contains(value) && (d_majTicks == In || d_majTicks == None))
-        return;
+	QwtValueList majTicks = scDiv.ticks(QwtScaleDiv::MajorTick);
+	if (majTicks.contains(value)){
+		if (d_majTicks == In || d_majTicks == None)
+			return;
+
+		switch (d_show_ticks_policy){
+			case ShowAll:
+			break;
+
+			case HideBegin:
+				if (majTicks.first() == value)
+					return;
+			break;
+			case HideEnd:
+				if (majTicks.last() == value)
+					return;
+			break;
+			case HideBeginEnd:
+				if (majTicks.first() == value || majTicks.last() == value)
+					return;
+			break;
+		}
+	}
 
     QwtValueList medTicks = scDiv.ticks(QwtScaleDiv::MediumTick);
     if (medTicks.contains(value) && (d_minTicks == In || d_minTicks == None))
@@ -519,52 +555,51 @@ void ScaleDraw::drawBreak(QPainter *painter) const
     painter->save();
     painter->setRenderHint(QPainter::Antialiasing);
 
+	QPen pen = painter->pen();
+	pen.setColor(d_plot->axisWidget(axis())->palette().color(QPalette::Active, QColorGroup::Foreground));
+	painter->setPen(pen);
+
 	int len = majTickLength();
 
     QwtScaleMap scaleMap = map();
     const QwtMetricsMap metricsMap = QwtPainter::metricsMap();
     QPoint pos = this->pos();
 
-    if ( !metricsMap.isIdentity() ){
-        QwtPainter::resetMetricsMap();
-        pos = metricsMap.layoutToDevice(pos);
+	if (!d_plot->isPrinting()){
+		QwtPainter::resetMetricsMap();
+		pos = metricsMap.layoutToDevice(pos);
 
-        if ( orientation() == Qt::Vertical ){
-            scaleMap.setPaintInterval(
-                metricsMap.layoutToDeviceY((int)scaleMap.p1()),
-                metricsMap.layoutToDeviceY((int)scaleMap.p2()));
-            len = metricsMap.layoutToDeviceX(len);
-        } else {
-            scaleMap.setPaintInterval(
-                metricsMap.layoutToDeviceX((int)scaleMap.p1()),
-                metricsMap.layoutToDeviceX((int)scaleMap.p2()));
-            len = metricsMap.layoutToDeviceY(len);
-        }
-    }
+		if ( orientation() == Qt::Vertical ){
+			scaleMap.setPaintInterval(
+				metricsMap.layoutToDeviceY((int)scaleMap.p1()),
+				metricsMap.layoutToDeviceY((int)scaleMap.p2()));
+			len = metricsMap.layoutToDeviceX(len);
+		} else {
+			scaleMap.setPaintInterval(
+				metricsMap.layoutToDeviceX((int)scaleMap.p1()),
+				metricsMap.layoutToDeviceX((int)scaleMap.p2()));
+			len = metricsMap.layoutToDeviceY(len);
+		}
+	}
 
-    int lval = scaleMap.transform(sc_engine->axisBreakLeft());
+	int lval = scaleMap.transform(sc_engine->axisBreakLeft());
 	int rval = scaleMap.transform(sc_engine->axisBreakRight());
 	switch(alignment()){
-        case LeftScale:
-            QwtPainter::drawLine(painter, pos.x(), lval, pos.x() - len, lval + len);
-			QwtPainter::drawLine(painter, pos.x(), rval, pos.x() - len, rval + len);
-        break;
-        case RightScale:
-            QwtPainter::drawLine(painter, pos.x(), lval, pos.x() + len, lval - len);
-			QwtPainter::drawLine(painter, pos.x(), rval, pos.x() + len, rval - len);
-        break;
-        case BottomScale:
-            QwtPainter::drawLine(painter, lval, pos.y(), lval - len, pos.y() + len);
-			QwtPainter::drawLine(painter, rval, pos.y(), rval - len, pos.y() + len);
-        break;
-        case TopScale:
-            QwtPainter::drawLine(painter, lval, pos.y(), lval + len, pos.y() - len);
-			QwtPainter::drawLine(painter, rval, pos.y(), rval + len, pos.y() - len);
-        break;
-    }
+		case LeftScale:
+		case RightScale:
+			QwtPainter::drawLine(painter, pos.x() + len, lval - len, pos.x() - len, lval + len);
+			QwtPainter::drawLine(painter, pos.x() + len, rval - len, pos.x() - len, rval + len);
+		break;
+		case TopScale:
+		case BottomScale:
+			QwtPainter::drawLine(painter, lval + len, pos.y() - len, lval - len, pos.y() + len);
+			QwtPainter::drawLine(painter, rval + len, pos.y() - len, rval - len, pos.y() + len);
+		break;
+	}
 
-    QwtPainter::setMetricsMap(metricsMap); // restore metrics map
-    painter->restore();
+	if (!d_plot->isPrinting())
+		QwtPainter::setMetricsMap(metricsMap); // restore metrics map
+	painter->restore();
 }
 
 void ScaleDraw::drawBackbone(QPainter *painter) const
@@ -594,24 +629,7 @@ void ScaleDraw::drawBackbone(QPainter *painter) const
     }
 
     QwtScaleMap scaleMap = map();
-    const QwtMetricsMap metricsMap = QwtPainter::metricsMap();
     QPoint pos = this->pos();
-
-    if ( !metricsMap.isIdentity() ){
-        QwtPainter::resetMetricsMap();
-        pos = metricsMap.layoutToDevice(pos);
-
-        if ( orientation() == Qt::Vertical ){
-            scaleMap.setPaintInterval(
-                metricsMap.layoutToDeviceY((int)scaleMap.p1()),
-                metricsMap.layoutToDeviceY((int)scaleMap.p2()));
-        } else {
-            scaleMap.setPaintInterval(
-                metricsMap.layoutToDeviceX((int)scaleMap.p1()),
-                metricsMap.layoutToDeviceX((int)scaleMap.p2()));
-        }
-    }
-
 	const int start = scaleMap.transform(sc_engine->axisBreakLeft());
 	const int end = scaleMap.transform(sc_engine->axisBreakRight());
     int lb = start, rb = end;
@@ -620,33 +638,49 @@ void ScaleDraw::drawBackbone(QPainter *painter) const
 		rb = start;
 	}
 
-	const int bw = painter->pen().width();
-    const int bw2 = bw / 2;
-    const int len = length() - 1;
-    int aux;
-	switch(alignment())
-    {
-        case LeftScale:
-            aux = pos.x() - bw2;
-            QwtPainter::drawLine(painter, aux, pos.y(), aux, rb);
-            QwtPainter::drawLine(painter, aux, lb + bw, aux, pos.y() + len);
-            break;
-        case RightScale:
-            aux = pos.x() + bw2;
-            QwtPainter::drawLine(painter, aux, pos.y(), aux, rb - bw - 1);
-            QwtPainter::drawLine(painter, aux, lb - bw2, aux, pos.y() + len);
-            break;
-        case TopScale:
-            aux = pos.y() - bw2;
-            QwtPainter::drawLine(painter, pos.x(), aux, lb - bw2, aux);
-            QwtPainter::drawLine(painter, rb + bw, aux, pos.x() + len, aux);
-            break;
-        case BottomScale:
-            aux = pos.y() + bw2;
-            QwtPainter::drawLine(painter, pos.x(), aux, lb - bw, aux);
-            QwtPainter::drawLine(painter, rb, aux, pos.x() + len, aux);
-            break;
-    }
+	if (d_plot->isPrinting()){
+		const int len = length();
+		switch(alignment()){
+			case LeftScale:
+			case RightScale:
+				QwtPainter::drawLine(painter, pos.x(), pos.y(), pos.x(), rb - 1);
+				QwtPainter::drawLine(painter, pos.x(), lb + 1, pos.x(), pos.y() + len);
+			break;
+			case TopScale:
+			case BottomScale:
+				QwtPainter::drawLine(painter, pos.x(), pos.y(), lb - 1, pos.y());
+				QwtPainter::drawLine(painter, rb + 1, pos.y(), pos.x() + len, pos.y());
+			break;
+		}
+	} else {
+		const int bw = painter->pen().width();
+		const int bw2 = bw / 2;
+		const int len = length() - 1;
+		int aux;
+		switch(alignment())
+		{
+			case LeftScale:
+				aux = pos.x() - bw2;
+				QwtPainter::drawLine(painter, aux, pos.y(), aux, rb);
+				QwtPainter::drawLine(painter, aux, lb + bw, aux, pos.y() + len);
+				break;
+			case RightScale:
+				aux = pos.x() + bw2;
+				QwtPainter::drawLine(painter, aux, pos.y(), aux, rb - bw - 1);
+				QwtPainter::drawLine(painter, aux, lb - bw2, aux, pos.y() + len);
+				break;
+			case TopScale:
+				aux = pos.y() - bw2;
+				QwtPainter::drawLine(painter, pos.x(), aux, lb - bw2, aux);
+				QwtPainter::drawLine(painter, rb + bw, aux, pos.x() + len, aux);
+				break;
+			case BottomScale:
+				aux = pos.y() + bw2;
+				QwtPainter::drawLine(painter, pos.x(), aux, lb - bw, aux);
+				QwtPainter::drawLine(painter, rb, aux, pos.x() + len, aux);
+				break;
+		}
+	}
 }
 
 void ScaleDraw::setDayFormat(NameFormat format)
