@@ -31,12 +31,17 @@
 
 #include "PlotToolInterface.h"
 #include <QObject>
+#include <QPointer>
+#include <QLabel>
+#include <DoubleSpinBox.h>
+
 #include <qwt_double_rect.h>
 #include <qwt_plot_marker.h>
 #include <qwt_plot_picker.h>
 
 class ApplicationWindow;
 class Table;
+class Matrix;
 class DataCurve;
 
 /*!Plot tool for selecting arbitrary points.
@@ -51,9 +56,12 @@ class ScreenPickerTool : public QwtPlotPicker, public PlotToolInterface
 		enum MoveRestriction { NoRestriction, Vertical, Horizontal };
 		ScreenPickerTool(Graph *graph, const QObject *status_target=NULL, const char *status_slot="");
 		virtual ~ScreenPickerTool();
-		void append(const QwtDoublePoint &pos);
+		virtual void append(const QwtDoublePoint &pos);
 		void setMoveRestriction(ScreenPickerTool::MoveRestriction r){d_move_restriction = r;};
 
+		double xValue(){return d_selection_marker.xValue();};
+		double yValue(){return d_selection_marker.yValue();};
+
 	signals:
 		/*! Emitted whenever a new message should be presented to the user.
 		 *
@@ -85,4 +93,37 @@ class DrawPointTool : public ScreenPickerTool
 		ApplicationWindow *d_app;
 };
 
+/*!Plot tool for image analysis.
+ *
+ */
+class ImageProfilesTool : public ScreenPickerTool
+{
+	Q_OBJECT
+	public:
+		ImageProfilesTool(ApplicationWindow *app, Graph *graph, Matrix *m, Table *horTable, Table *verTable,
+						const QObject *status_target=NULL, const char *status_slot="");
+
+		ImageProfilesTool* clone(Graph *g);
+
+		virtual ~ImageProfilesTool();
+		virtual void append(const QwtDoublePoint &pos);
+		virtual int rtti() const { return Rtti_ImageProfilesTool;};
+
+
+		QPointer<Matrix> matrix(){return d_matrix;};
+		QPointer<Table> horizontalTable(){return d_hor_table;};
+		QPointer<Table> verticalTable(){return d_ver_table;};
+
+	private slots:
+		void modifiedMatrix(Matrix *);
+		void updateCursorPosition();
+
+	protected:
+		ApplicationWindow *d_app;
+		QPointer<Matrix> d_matrix;
+		QPointer<Table> d_hor_table, d_ver_table;
+		DoubleSpinBox *horSpinBox, *vertSpinBox;
+		QLabel *zLabel;
+};
+
 #endif // ifndef SCREEN_PICKER_TOOL_H
