@@ -50,29 +50,42 @@ SelectionMoveResizer::SelectionMoveResizer(ArrowMarker *target)
 SelectionMoveResizer::SelectionMoveResizer(QWidget *target)
 	: QWidget(target->parentWidget())
 {
+	QwtPlotCanvas *canvas = qobject_cast<QwtPlotCanvas *>(target);
+	if (canvas)
+		setParent(canvas->plot()->parentWidget());
+
 	init();
 	add(target);
 }
 
 void SelectionMoveResizer::init()
 {
-	d_bounding_rect = QRect(0,0,-1,-1);
+	d_bounding_rect = QRect(0, 0, -1, -1);
 	d_op = None;
-	d_op_start = QPoint(0,0);
-	d_op_dp = QPoint(0,0);
+	d_op_start = QPoint(0, 0);
+	d_op_dp = QPoint(0, 0);
 
     setAttribute(Qt::WA_DeleteOnClose);
 	setGeometry(0, 0, parentWidget()->width(), parentWidget()->height());
 	setMouseTracking(true);
 	parentWidget()->installEventFilter(this);
 	show();
+	setFocusPolicy (Qt::StrongFocus);
 	setFocus();
 }
 
 SelectionMoveResizer::~SelectionMoveResizer()
 {
-    if (parentWidget())
-        parentWidget()->removeEventFilter(this);
+	foreach(QWidget *w, d_widgets){
+		QwtPlotCanvas *canvas = qobject_cast<QwtPlotCanvas *>(w);
+		if (canvas){
+			((Graph*)canvas->parent())->raiseEnrichements();
+			break;
+		}
+	}
+
+	if (parentWidget())
+		parentWidget()->removeEventFilter(this);
 }
 
 void SelectionMoveResizer::add(ArrowMarker *target)
@@ -91,16 +104,22 @@ void SelectionMoveResizer::add(ArrowMarker *target)
 
 void SelectionMoveResizer::add(QWidget *target)
 {
-	if (target->parentWidget() != parent())
-		return;
+	//if (target->parentWidget() != parent())
+		//return;
+
 	d_widgets << target;
 	target->installEventFilter(this);
 	connect(target, SIGNAL(destroyed(QObject*)), this, SLOT(removeWidget(QObject*)));
 
+	QRect r = target->frameGeometry();
+	QwtPlotCanvas *canvas = qobject_cast<QwtPlotCanvas *>(target);
+	if (canvas)
+		r = boundingRectOf(canvas);
+
 	if (d_bounding_rect.isValid())
-		d_bounding_rect |= target->frameGeometry();
+		d_bounding_rect |= r;
 	else
-		d_bounding_rect = target->frameGeometry();
+		d_bounding_rect = r;
 
 	update();
 }
@@ -110,6 +129,14 @@ QRect SelectionMoveResizer::boundingRectOf(QwtPlotMarker *target) const
 	return ((ArrowMarker *)target)->rect();
 }
 
+QRect SelectionMoveResizer::boundingRectOf(QwtPlotCanvas *canvas) const
+{
+	if (!canvas)
+		return QRect();
+
+	return canvas->frameGeometry().translated(canvas->plot()->pos());
+}
+
 int SelectionMoveResizer::removeAll(ArrowMarker *target)
 {
 	int result = d_line_markers.removeAll(target);
@@ -123,6 +150,11 @@ int SelectionMoveResizer::removeAll(ArrowMarker *target)
 int SelectionMoveResizer::removeAll(QWidget *target)
 {
 	int result = d_widgets.removeAll(target);
+
+	FrameWidget *fw = qobject_cast<FrameWidget *>(target);
+	if (fw && !fw->isOnTop())
+		fw->lower();
+
 	if (d_line_markers.isEmpty() && d_widgets.isEmpty())
 		delete this;
 	else
@@ -130,21 +162,39 @@ int SelectionMoveResizer::removeAll(QWidget *target)
 	return result;
 }
 
+void SelectionMoveResizer::raiseTargets(bool on)
+{
+	foreach(QWidget *w, d_widgets){
+		FrameWidget *fw = qobject_cast<FrameWidget *>(w);
+		if (fw)
+			fw->setOnTop(on);
+	}
+
+	if (on)
+		this->raise();
+}
+
 void SelectionMoveResizer::recalcBoundingRect()
 {
 	d_bounding_rect = QRect(0, 0, -1, -1);
 
-	foreach(ArrowMarker *i, d_line_markers) {
+	foreach(ArrowMarker *i, d_line_markers){
 		if(d_bounding_rect.isValid())
 			d_bounding_rect |= boundingRectOf(i);
 		else
 			d_bounding_rect = boundingRectOf(i);
 	}
-	foreach(QWidget *i, d_widgets) {
+
+	foreach(QWidget *i, d_widgets){
+		QRect r = i->frameGeometry();
+		QwtPlotCanvas *canvas = qobject_cast<QwtPlotCanvas *>(i);
+		if (canvas)
+			r = boundingRectOf(canvas);
+
 		if(d_bounding_rect.isValid())
-			d_bounding_rect |= i->frameGeometry();
+			d_bounding_rect |= r;
 		else
-			d_bounding_rect = i->frameGeometry();
+			d_bounding_rect = r;
 	}
 
 	update();
@@ -223,6 +273,7 @@ QRect SelectionMoveResizer::operateOn(const QRect in)
 	boundary_out = boundary_out.normalized();
 	if (in == d_bounding_rect)
 		return boundary_out;
+
 	double scale_x = ((double)boundary_out.width())/d_bounding_rect.width();
 	double scale_y = ((double)boundary_out.height())/d_bounding_rect.height();
 	int offset_x = qRound(boundary_out.left()-d_bounding_rect.left()*scale_x);
@@ -233,7 +284,7 @@ QRect SelectionMoveResizer::operateOn(const QRect in)
 
 void SelectionMoveResizer::operateOnTargets()
 {
-	foreach(ArrowMarker *i, d_line_markers) {
+	foreach(ArrowMarker *i, d_line_markers){
 		QPoint p1 = i->startPoint();
 		QPoint p2 = i->endPoint();
 		QRect new_rect = operateOn(i->rect());
@@ -246,6 +297,12 @@ void SelectionMoveResizer::operateOnTargets()
 	}
 
 	foreach(QWidget *i, d_widgets){
+		QwtPlotCanvas *canvas = qobject_cast<QwtPlotCanvas *>(i);
+		if (canvas){
+			((Graph *)canvas->plot())->setCanvasGeometry(operateOn(d_bounding_rect));
+			continue;
+		}
+
 		LegendWidget *l = qobject_cast<LegendWidget *>(i);
 		if (!l){
 			QRect r = operateOn(i->geometry());
@@ -309,12 +366,12 @@ void SelectionMoveResizer::mousePressEvent(QMouseEvent *me)
 		return;
 	}
 
-	if (me->button() == Qt::LeftButton){
+	if (me->button() == Qt::LeftButton && !d_widgets.isEmpty()){
 		FrameWidget *fw = qobject_cast<FrameWidget *>(d_widgets[0]);
 		if (fw && fw->plot()){
-			QList <FrameWidget *> lst = fw->plot()->enrichmentsList();
+			QList <FrameWidget *> lst = fw->plot()->increasingAreaEnrichmentsList();
 			foreach(FrameWidget *f, lst){
-				if(!d_widgets.contains(f) && f->geometry().contains(me->pos()))
+				if(!d_widgets.contains(f) && f->geometry().contains(me->pos()) && !fw->geometry().contains(me->pos()))
 					return fw->plot()->select(f, me->modifiers() & Qt::ShiftModifier);
 			}
 		}
@@ -413,6 +470,11 @@ void SelectionMoveResizer::keyPressEvent(QKeyEvent *ke)
 			ke->accept();
 			break;
 		case Qt::Key_Escape:
+			foreach(QWidget *w, d_widgets){
+				FrameWidget *l = qobject_cast<FrameWidget *>(w);
+				if (l && !l->isOnTop())
+					l->lower();
+			}
 			delete this;
 			ke->accept();
 			return;
