@@ -161,6 +161,7 @@ void TexWidget::restore(Graph *g, const QStringList& lst)
 	QPixmap pix;
 	QStringList::const_iterator line;
 	QString formula;
+	bool onTop = true;
 	for (line = lst.begin(); line != lst.end(); line++){
         QString s = *line;
         if (s.contains("<Frame>"))
@@ -181,6 +182,8 @@ void TexWidget::restore(Graph *g, const QStringList& lst)
 			bottom = s.remove("<bottom>").remove("</bottom>").toDouble();
 		else if (s.contains("<attachTo>"))
 			attachTo = (FrameWidget::AttachPolicy)s.remove("<attachTo>").remove("</attachTo>").toInt();
+		else if (s.contains("<onTop>"))
+			onTop = s.remove("<onTop>").remove("</onTop>").toInt();
 		else if (s.contains("<tex>"))
 			formula = s.remove("<tex>").remove("</tex>");
 		else if (s.contains("<xpm>")){
@@ -202,4 +205,5 @@ void TexWidget::restore(Graph *g, const QStringList& lst)
 	t->setFramePen(pen);
 	t->setCoordinates(x, y, right, bottom);
 	t->setAttachPolicy(attachTo);
+	t->setOnTop(onTop);
 }
