@@ -2,7 +2,8 @@
     File                 : AxesDialog.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
+	Copyright            : (C) 2004 - 2010 by Ion Vasilief,
+						   (C) 2006 - June 2007 Tilman Hoener zu Siederdissen
     Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
     Description          : General plot options dialog
 
@@ -38,10 +39,8 @@
 #include <ScaleEngine.h>
 #include <FunctionCurve.h>
 #include <DoubleSpinBox.h>
-#include <ColorBox.h>
 #include <ColorButton.h>
 #include <PenStyleBox.h>
-#include <axes_icons.h>
 
 #include <QCheckBox>
 #include <QComboBox>
@@ -105,8 +104,7 @@ AxesDialog::AxesDialog( QWidget* parent, Qt::WFlags fl )
     connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
     connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
     connect( buttonApply, SIGNAL( clicked() ), this, SLOT(updatePlot() ) );
-    connect( generalDialog, SIGNAL( currentChanged ( QWidget * ) ),
-            this, SLOT(pageChanged ( QWidget * ) ) );
+	connect( generalDialog, SIGNAL( currentChanged ( QWidget * ) ), this, SLOT(pageChanged( QWidget *)));
 }
 
 void AxesDialog::initScalesPage()
@@ -150,13 +148,13 @@ void AxesDialog::initScalesPage()
 	boxScaleTypeLabel = new QLabel(tr( "Type" ));
 	boxScaleType = new QComboBox();
 	boxScaleType->setIconSize(QSize(27, 16));
-	boxScaleType->addItem(QIcon(QPixmap(liniar_scale_xpm)), tr( "Linear" ));
-	boxScaleType->addItem(QIcon(QPixmap(log10_xpm)), tr( "Log10" ));
-	boxScaleType->addItem(QIcon(QPixmap(ln_scale_xpm)), tr( "ln" ));
-	boxScaleType->addItem(QIcon(QPixmap(log2_scale_xpm)), tr( "Log2" ));
-	boxScaleType->addItem(QIcon(QPixmap(reciprocal_scale_xpm)), tr( "Reciprocal" ));
-	boxScaleType->addItem(QIcon(QPixmap(probability_scale_xpm)), tr( "Probability" ));
-	boxScaleType->addItem(QIcon(QPixmap(logit_scale_xpm)), tr( "Logit" ));
+	boxScaleType->addItem(QIcon(":/liniar_scale.png"), tr( "Linear" ));
+	boxScaleType->addItem(QIcon(":/log10.png"), tr( "Log10" ));
+	boxScaleType->addItem(QIcon(":/ln_scale.png"), tr( "ln" ));
+	boxScaleType->addItem(QIcon(":/log2_scale.png"), tr( "Log2" ));
+	boxScaleType->addItem(QIcon(":/reciprocal_scale.png"), tr( "Reciprocal" ));
+	boxScaleType->addItem(QIcon(":/probability_scale.png"), tr( "Probability" ));
+	boxScaleType->addItem(QIcon(":/logit_scale.png"), tr( "Logit" ));
 	middleLayout->addWidget( boxScaleTypeLabel, 2, 0);
 	middleLayout->addWidget( boxScaleType, 2, 1);
 
@@ -276,10 +274,10 @@ void AxesDialog::initScalesPage()
 	vl->addLayout(hl);
 	vl->addWidget(boxAxesBreaks);
 
-	QPixmap image0( ( const char** ) bottom_scl_xpm );
-	QPixmap image1( ( const char** ) left_scl_xpm );
-	QPixmap image2( ( const char** ) top_scl_xpm );
-	QPixmap image3( ( const char** ) right_scl_xpm );
+	QPixmap image0(":/bottom_scl.png");
+	QPixmap image1(":/left_scl.png");
+	QPixmap image2(":/top_scl.png");
+	QPixmap image3(":/right_scl.png");
 
 	axesList = new QListWidget();
 	axesList->addItem( new QListWidgetItem(image0, tr( "Bottom" )));
@@ -334,10 +332,10 @@ void AxesDialog::initGridPage()
 
     rightLayout->addWidget( new QLabel(tr( "Line Color" )), 1, 0 );
 
-    boxColorMajor = new ColorBox(0);
+	boxColorMajor = new ColorButton();
     rightLayout->addWidget( boxColorMajor, 1, 1);
 
-    boxColorMinor = new ColorBox(0);
+	boxColorMinor = new ColorButton();
     boxColorMinor->setDisabled(true);
     rightLayout->addWidget( boxColorMinor, 1, 2);
 
@@ -400,8 +398,8 @@ void AxesDialog::initGridPage()
     rightLayout->setRowStretch(7, 1);
     rightLayout->setColumnStretch(4, 1);
 
-    QPixmap image2( ( const char** ) image2_data );
-    QPixmap image3( ( const char** ) image3_data );
+	QPixmap image2(":/vertical_grid.png");
+	QPixmap image3(":/horizontal_grid.png");
 
     axesGridList = new QListWidget();
     axesGridList->addItem( new QListWidgetItem(image3, tr( "Horizontal" )) );
@@ -433,8 +431,8 @@ void AxesDialog::initGridPage()
 	connect(boxMajorGrid,SIGNAL(toggled(bool)), this, SLOT(majorGridEnabled(bool)));
 	connect(boxMinorGrid,SIGNAL(toggled(bool)), this, SLOT(minorGridEnabled(bool)));
 	connect(boxAntialiseGrid,SIGNAL(toggled(bool)), this, SLOT(updateGrid()));
-	connect(boxColorMajor,SIGNAL(activated(int)),this, SLOT(updateGrid()));
-	connect(boxColorMinor,SIGNAL(activated(int)),this, SLOT(updateGrid()));
+	connect(boxColorMajor, SIGNAL(colorChanged(const QColor &)),this, SLOT(updateGrid()));
+	connect(boxColorMinor, SIGNAL(colorChanged(const QColor &)),this, SLOT(updateGrid()));
 	connect(boxTypeMajor,SIGNAL(activated(int)),this, SLOT(updateGrid()));
 	connect(boxTypeMinor,SIGNAL(activated(int)),this, SLOT(updateGrid()));
 	connect(boxWidthMajor,SIGNAL(valueChanged(double)),this, SLOT(updateGrid()));
@@ -446,10 +444,10 @@ void AxesDialog::initGridPage()
 void AxesDialog::initAxesPage()
 {
 	//axes page
-	QPixmap bottom_axis_pic( ( const char** ) bottom_axis_xpm );
-	QPixmap left_axis_pic( ( const char** ) left_axis_xpm );
-	QPixmap top_axis_pic( ( const char** ) top_axis_xpm );
-	QPixmap right_axis_pic( ( const char** ) right_axis_xpm );
+	QPixmap bottom_axis_pic(":/bottom_axis.png");
+	QPixmap left_axis_pic(":/left_axis.png");
+	QPixmap top_axis_pic(":/top_axis.png");
+	QPixmap right_axis_pic(":/right_axis.png");
 
 	axesPage = new QWidget();
 
@@ -498,6 +496,7 @@ void AxesDialog::initAxesPage()
 	hl->addWidget(new QLabel(tr("Distance to axis")));
 	boxLabelsDistance = new QSpinBox();
 	boxLabelsDistance->setRange(0, 1000);
+	boxLabelsDistance->setSuffix(" " + tr("pixels"));
 	connect(boxLabelsDistance, SIGNAL(valueChanged(int)), this, SLOT(updatePlot()));
 	hl->addWidget(boxLabelsDistance);
 
@@ -551,34 +550,56 @@ void AxesDialog::initAxesPage()
 
 	leftBoxLayout->addWidget( new QLabel(tr( "Minor Ticks" )), 4, 0 );
 
-	boxMinorTicksType= new QComboBox();
+	boxMinorTicksType = new QComboBox();
 	boxMinorTicksType->addItem(tr( "None" ) );
 	boxMinorTicksType->addItem(tr( "Out" ) );
 	boxMinorTicksType->addItem(tr( "In & Out" ) );
 	boxMinorTicksType->addItem(tr( "In" ) );
 	leftBoxLayout->addWidget( boxMinorTicksType, 4, 1);
 
-	leftBoxLayout->addWidget( new QLabel(tr("Stand-off")), 5, 0);
+	leftBoxLayout->addWidget( new QLabel(tr( "Hide Ticks" )), 5, 0 );
+	showTicksPolicyBox = new QComboBox();
+	showTicksPolicyBox->addItem(tr("None"));
+	showTicksPolicyBox->addItem(tr("At Axis Begin"));
+	showTicksPolicyBox->addItem(tr("At Axis End"));
+	showTicksPolicyBox->addItem(tr("At Axis Begin & End"));
+	leftBoxLayout->addWidget(showTicksPolicyBox, 5, 1);
+
+	leftBoxLayout->addWidget( new QLabel(tr("Labels space")), 6, 0);
+	boxTickLabelDistance = new QSpinBox();
+	boxTickLabelDistance->setRange( 0, 10000);
+	boxTickLabelDistance->setSuffix(" " + tr("pixels"));
+	leftBoxLayout->addWidget(boxTickLabelDistance, 6, 1);
+
+	leftBoxLayout->addWidget( new QLabel(tr("Stand-off")), 7, 0);
 	boxBaseline = new QSpinBox();
 	boxBaseline->setRange( 0, 1000 );
-	leftBoxLayout->addWidget(boxBaseline, 5, 1);
+	boxBaseline->setSuffix(" " + tr("pixels"));
+	leftBoxLayout->addWidget(boxBaseline, 7, 1);
 
 	QLabel *l = new QLabel(tr("Apply &to"));
-	leftBoxLayout->addWidget(l, 6, 0);
+	leftBoxLayout->addWidget(l, 8, 0);
 
 	axisFormatApplyToBox = new QComboBox();
 	axisFormatApplyToBox->insertItem(tr("Axis"));
 	axisFormatApplyToBox->insertItem(tr("Layer"));
     axisFormatApplyToBox->insertItem(tr("Window"));
     axisFormatApplyToBox->insertItem(tr("All Windows"));
-	leftBoxLayout->addWidget(axisFormatApplyToBox, 6, 1);
+	leftBoxLayout->addWidget(axisFormatApplyToBox, 8, 1);
 	l->setBuddy(axisFormatApplyToBox);
 
 	boxShowLabels = new QGroupBox(tr("Show Labels"));
 	boxShowLabels->setCheckable(true);
 	boxShowLabels->setChecked(true);
 
-	bottomLayout->addWidget( boxShowLabels );
+	boxAxisBackbone = new QCheckBox(tr("Show &backbone"));
+
+	QVBoxLayout *vb = new QVBoxLayout;
+	vb->addWidget( boxShowLabels );
+	vb->addWidget( boxAxisBackbone );
+
+	bottomLayout->addLayout( vb );
+
 	QGridLayout *rightBoxLayout = new QGridLayout( boxShowLabels );
 
 	label1 = new QLabel(tr("Column"));
@@ -622,15 +643,22 @@ void AxesDialog::initAxesPage()
 	boxAxisNumColor = new ColorButton();
 	rightBoxLayout->addWidget( boxAxisNumColor, 5, 1 );
 
+	rightBoxLayout->addWidget(new QLabel(tr( "Prefix" )), 6, 0);
+	boxPrefix = new QLineEdit();
+	rightBoxLayout->addWidget( boxPrefix, 6, 1 );
+	rightBoxLayout->addWidget(new QLabel(tr( "Suffix" )), 7, 0);
+	boxSuffix = new QLineEdit();
+	rightBoxLayout->addWidget( boxSuffix, 7, 1 );
+
 	boxShowFormula = new QCheckBox(tr( "For&mula" ));
-	rightBoxLayout->addWidget( boxShowFormula, 6, 0 );
+	rightBoxLayout->addWidget( boxShowFormula, 8, 0 );
 
 	boxFormula = new QTextEdit();
 	boxFormula->setTextFormat(Qt::PlainText);
 	boxFormula->setMaximumHeight(3*metrics.height());
 	boxFormula->hide();
-	rightBoxLayout->addWidget( boxFormula, 6, 1 );
-	rightBoxLayout->setRowStretch(7, 1);
+	rightBoxLayout->addWidget( boxFormula, 8, 1 );
+	rightBoxLayout->setRowStretch(9, 1);
 
 	QVBoxLayout * rightLayout = new QVBoxLayout();
 	rightLayout->addLayout( topLayout );
@@ -652,6 +680,7 @@ void AxesDialog::initAxesPage()
 	connect(axesTitlesList,SIGNAL(currentRowChanged(int)), this, SLOT(setAxisType(int)));
 	connect(axesTitlesList,SIGNAL(currentRowChanged(int)), this, SLOT(setBaselineDist(int)));
 	connect(axesTitlesList,SIGNAL(currentRowChanged(int)), this, SLOT(updateLabelsFormat(int)));
+	connect(axesTitlesList,SIGNAL(currentRowChanged(int)), this, SLOT(updateCurrentAxis()));
 
 	connect(boxShowLabels,SIGNAL(clicked(bool)), this, SLOT(updateTickLabelsList(bool)));
 
@@ -659,7 +688,10 @@ void AxesDialog::initAxesPage()
     connect(boxAxisNumColor, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
 	connect(boxMajorTicksType, SIGNAL(activated(int)), this, SLOT(updatePlot()));
 	connect(boxMinorTicksType, SIGNAL(activated(int)), this, SLOT(updatePlot()));
+	connect(showTicksPolicyBox, SIGNAL(activated(int)), this, SLOT(updatePlot()));
 	connect(boxBaseline, SIGNAL(valueChanged(int)), this, SLOT(updatePlot()));
+	connect(boxAxisBackbone, SIGNAL(clicked(bool)), this, SLOT(updatePlot()));
+	connect(boxTickLabelDistance, SIGNAL(valueChanged(int)), this, SLOT(updatePlot()));
 
 	connect(boxShowFormula, SIGNAL(clicked()), this, SLOT(showFormulaBox()));
 	connect(boxShowAxis, SIGNAL(clicked()), this, SLOT(showAxis()));
@@ -787,6 +819,7 @@ void AxesDialog::showAxisFormatOptions(int format)
 			boxFormat->insertItem(tr( "Scientific: 1e4" ) );
 			boxFormat->insertItem(tr( "Scientific: 1x10^4" ) );
 			boxFormat->insertItem(tr( "Engineering: 10k" ) );
+			boxFormat->insertItem(tr( "Scientific: 110^4" ) );
 			boxFormat->setCurrentIndex(d_graph->axisLabelFormat(axis));
 
 			label3->show();
@@ -843,19 +876,23 @@ void AxesDialog::showAxisFormatOptions(int format)
 				originDateTimeBox->setDateTime (scaleDraw->dateTimeOrigin());
 			}
 
-            boxFormat->insertItem("h");
-            boxFormat->insertItem("h ap");
-            boxFormat->insertItem("h AP");
-            boxFormat->insertItem("h:mm");
-            boxFormat->insertItem("h:mm ap");
-            boxFormat->insertItem("hh:mm");
-            boxFormat->insertItem("h:mm:ss");
-            boxFormat->insertItem("h:mm:ss.zzz");
-            boxFormat->insertItem("mm:ss");
-            boxFormat->insertItem("mm:ss.zzz");
-            boxFormat->insertItem("hmm");
-            boxFormat->insertItem("hmmss");
-            boxFormat->insertItem("hhmmss");
+			boxFormat->insertItem("h");
+			boxFormat->insertItem("h ap");
+			boxFormat->insertItem("h AP");
+			boxFormat->insertItem("M");
+			boxFormat->insertItem("S");
+			boxFormat->insertItem("h:mm");
+			boxFormat->insertItem("h:mm ap");
+			boxFormat->insertItem("hh:mm");
+			boxFormat->insertItem("h:mm:ss");
+			boxFormat->insertItem("h:mm:ss.zzz");
+			boxFormat->insertItem("m");
+			boxFormat->insertItem("mm");
+			boxFormat->insertItem("mm:ss");
+			boxFormat->insertItem("mm:ss.zzz");
+			boxFormat->insertItem("hmm");
+			boxFormat->insertItem("hmmss");
+			boxFormat->insertItem("hhmmss");
         }
         break;
 
@@ -908,6 +945,7 @@ void AxesDialog::showAxis()
 	axisFormatBox->setEnabled(ok);
 	boxShowLabels->setEnabled(ok);
 	labelBox->setEnabled(ok);
+	boxAxisBackbone->setEnabled(ok);
 
     int axis = -1;
     int a = axesTitlesList->currentRow();
@@ -952,7 +990,8 @@ void AxesDialog::showAxis()
 
     showAxis(axis, boxAxisType->currentIndex(), boxColName->currentText(),ok, boxMajorTicksType->currentIndex(),
             boxMinorTicksType->currentIndex(), boxShowLabels->isChecked(), boxAxisColor->color(), boxFormat->currentIndex(),
-            boxPrecision->value(), boxAngle->value(), boxBaseline->value(), formula, boxAxisNumColor->color());
+			boxPrecision->value(), boxAngle->value(), boxBaseline->value(), formula, boxAxisNumColor->color(),
+			boxTickLabelDistance->value(), boxAxisBackbone->isChecked(), showTicksPolicyBox->currentIndex());
 }
 
 void AxesDialog::updateShowBox(int axis)
@@ -1108,18 +1147,14 @@ void AxesDialog::applyChangesToGrid(Grid *grid)
 		grid->enableX(boxMajorGrid->isChecked());
 		grid->enableXMin(boxMinorGrid->isChecked());
 
-		grid->setMajPenX(QPen(ColorBox::color(boxColorMajor->currentIndex()), boxWidthMajor->value(),
-					 		boxTypeMajor->style()));
-		grid->setMinPenX(QPen(ColorBox::color(boxColorMinor->currentIndex()), boxWidthMinor->value(),
-					 		boxTypeMinor->style()));
+		grid->setMajPenX(QPen(boxColorMajor->color(), boxWidthMajor->value(), boxTypeMajor->style()));
+		grid->setMinPenX(QPen(boxColorMinor->color(), boxWidthMinor->value(), boxTypeMinor->style()));
 	} else {
 		grid->enableY(boxMajorGrid->isChecked());
 		grid->enableYMin(boxMinorGrid->isChecked());
 
-		grid->setMajPenY(QPen(ColorBox::color(boxColorMajor->currentIndex()), boxWidthMajor->value(),
-					 		boxTypeMajor->style()));
-		grid->setMinPenY(QPen(ColorBox::color(boxColorMinor->currentIndex()), boxWidthMinor->value(),
-					 		boxTypeMinor->style()));
+		grid->setMajPenY(QPen(boxColorMajor->color(), boxWidthMajor->value(), boxTypeMajor->style()));
+		grid->setMinPenY(QPen(boxColorMinor->color(), boxWidthMinor->value(), boxTypeMinor->style()));
 	}
 
 	grid->enableZeroLineX(boxXLine->isChecked());
@@ -1241,10 +1276,13 @@ void AxesDialog::updateAxisColor(int)
 	boxAxisNumColor->blockSignals(false);
 }
 
-bool AxesDialog::updatePlot()
+bool AxesDialog::updatePlot(QWidget *page)
 {
-	if (generalDialog->currentWidget()==(QWidget*)scalesPage)
-	{
+	QWidget *currentWidget = generalDialog->currentWidget();
+	if (page)
+		currentWidget = page;
+
+	if (currentWidget == scalesPage){
         int a = mapToQwtAxis(axesList->currentRow());
         ScaleDraw::ScaleType type = d_graph->axisType(a);
 
@@ -1307,11 +1345,9 @@ bool AxesDialog::updatePlot()
                           boxMinorTicksBeforeBreak->currentText().toInt(), boxMinorTicksAfterBreak->currentText().toInt(),
                           boxLog10AfterBreak->isChecked(), boxBreakWidth->value(), boxBreakDecoration->isChecked());
 		d_graph->notifyChanges();
-	}
-	else if (generalDialog->currentWidget() == gridPage)
+	} else if (currentWidget == gridPage)
 		updateGrid();
-	else if (generalDialog->currentWidget() == (QWidget*)axesPage)
-	{
+	else if (currentWidget == axesPage){
 		int axis = mapToQwtAxisId();
 		int format = boxAxisType->currentIndex();
 
@@ -1358,7 +1394,7 @@ bool AxesDialog::updatePlot()
 			d_graph->setAxisTitle(axis, boxTitle->text());
 
 		d_graph->setAxisTitleDistance(axis, boxLabelsDistance->value());
-
+		
 		if (axis == QwtPlot::xBottom)
 			xBottomLabelsRotation = boxAngle->value();
 		else if (axis == QwtPlot::xTop)
@@ -1369,7 +1405,8 @@ bool AxesDialog::updatePlot()
 			formula = QString();
 		showAxis(axis, format, formatInfo, boxShowAxis->isChecked(), boxMajorTicksType->currentIndex(), boxMinorTicksType->currentIndex(),
 				boxShowLabels->isChecked(), boxAxisColor->color(), boxFormat->currentIndex(),
-				boxPrecision->value(), boxAngle->value(), baseline, formula, boxAxisNumColor->color());
+				boxPrecision->value(), boxAngle->value(), baseline, formula, boxAxisNumColor->color(),
+				boxTickLabelDistance->value(), boxAxisBackbone->isChecked(), showTicksPolicyBox->currentIndex());
 
 		if (axis == QwtPlot::yRight){
 			QwtScaleWidget *scale = d_graph->axisWidget(axis);
@@ -1378,9 +1415,8 @@ bool AxesDialog::updatePlot()
 				scale->repaint();
 			}
 		}
-
-        applyAxisFormat();
-	} else if (generalDialog->currentWidget() == (QWidget*)frame)
+		applyAxisFormat();
+	} else if (currentWidget == frame)
 		applyCanvasFormat();
 
 	return true;
@@ -1634,7 +1670,28 @@ void AxesDialog::setAxisType(int)
 
 void AxesDialog::setBaselineDist(int)
 {
-    boxBaseline->setValue(axesBaseline[mapToQwtAxisId()]);
+	boxBaseline->setValue(axesBaseline[mapToQwtAxisId()]);
+}
+
+void AxesDialog::updateCurrentAxis()
+{
+	int axis = mapToQwtAxisId();
+	ScaleDraw *sd = (ScaleDraw *)d_graph->axisScaleDraw (axis);
+	if (sd){
+		boxAxisBackbone->setChecked(sd->hasComponent(QwtAbstractScaleDraw::Backbone));
+
+		boxTickLabelDistance->blockSignals(true);
+		boxTickLabelDistance->setValue(sd->spacing());
+		boxTickLabelDistance->blockSignals(false);
+
+		showTicksPolicyBox->blockSignals(true);
+		showTicksPolicyBox->setCurrentIndex(sd->showTicksPolicy());
+		showTicksPolicyBox->blockSignals(false);
+
+		boxPrefix->setText(sd->prefix());
+		boxSuffix->setText(sd->suffix());
+	}
+	boxShowLabels->repaint();
 }
 
 void AxesDialog::setTicksType(int)
@@ -1677,7 +1734,8 @@ void AxesDialog::updateTickLabelsList(bool on)
 
     showAxis(axis, type, formatInfo, boxShowAxis->isChecked(), boxMajorTicksType->currentIndex(), boxMinorTicksType->currentIndex(),
                   boxShowLabels->isChecked(), boxAxisColor->color(), boxFormat->currentIndex(), boxPrecision->value(),
-	              boxAngle->value(), boxBaseline->value(), formula, boxAxisNumColor->color());
+				  boxAngle->value(), boxBaseline->value(), formula, boxAxisNumColor->color(),
+				  boxTickLabelDistance->value(), boxAxisBackbone->isChecked(), showTicksPolicyBox->currentIndex());
 }
 
 void AxesDialog::setCurrentScale(int axisPos)
@@ -1745,7 +1803,8 @@ void AxesDialog::setLabelsNumericFormat(int)
 
     showAxis(axis, type, formatInfo, boxShowAxis->isChecked(), boxMajorTicksType->currentIndex(),
 			boxMinorTicksType->currentIndex(), boxShowLabels->isChecked(), boxAxisColor->color(),
-			format, prec, boxAngle->value(), boxBaseline->value(), formula, boxAxisNumColor->color());
+			format, prec, boxAngle->value(), boxBaseline->value(), formula, boxAxisNumColor->color(),
+			boxTickLabelDistance->value(), boxAxisBackbone->isChecked(), showTicksPolicyBox->currentIndex());
 }
 
 void AxesDialog::showAxisFormula(int axis)
@@ -1813,14 +1872,19 @@ void AxesDialog::customAxisLabelFont()
 
 void AxesDialog::pageChanged ( QWidget *page )
 {
-  	if (lastPage == scalesPage && page == axesPage){
-  		axesTitlesList->setCurrentRow(axesList->currentRow());
-  	    lastPage = page;
-  	}else if (lastPage == axesPage && page == scalesPage){
-  		axesList->setCurrentRow(axesTitlesList->currentRow());
-  		updateScale();
-  	    lastPage = page;
-  	}
+	updatePlot(lastPage);
+
+	if (lastPage == scalesPage && page == axesPage){
+		axesTitlesList->setCurrentRow(axesList->currentRow());
+	} else if (lastPage == axesPage && page == scalesPage){
+		axesList->setCurrentRow(axesTitlesList->currentRow());
+		updateScale();
+	}
+
+	if (page == axesPage)
+		updateCurrentAxis();
+
+	lastPage = page;
 }
 
 int AxesDialog::exec()
@@ -1868,7 +1932,8 @@ void AxesDialog::updateMinorTicksList(int scaleType)
 
 void AxesDialog::showAxis(int axis, int type, const QString& labelsColName, bool axisOn,
 		int majTicksType, int minTicksType, bool labelsOn, const QColor& c, int format,
-		int prec, int rotation, int baselineDist, const QString& formula, const QColor& labelsColor)
+		int prec, int rotation, int baselineDist, const QString& formula, const QColor& labelsColor,
+		int spacing, bool backbone, int showTicks)
 {
 	ApplicationWindow *app = (ApplicationWindow *)this->parent();
 	if (!app)
@@ -1881,7 +1946,8 @@ void AxesDialog::showAxis(int axis, int type, const QString& labelsColName, bool
 	if (!d_graph)
 		return;
 	d_graph->showAxis(axis, type, labelsColName, w, axisOn, majTicksType, minTicksType, labelsOn,
-			c, format, prec, rotation, baselineDist, formula, labelsColor);
+			c, format, prec, rotation, baselineDist, formula, labelsColor, spacing, backbone,
+			(ScaleDraw::ShowTicksPolicy)showTicks, boxPrefix->text(), boxSuffix->text());
 }
 
 void AxesDialog::applyCanvasFormatTo(Graph *g)
@@ -1961,12 +2027,12 @@ void AxesDialog::setFrameDefaultValues()
 
 void AxesDialog::applyAxisFormatToLayer(Graph *g)
 {
-    if (!g)
-        return;
+	if (!g)
+		return;
 
-    for (int i=0; i<QwtPlot::axisCnt; i++){
-        if (!g->axisEnabled(i))
-            continue;
+	for (int i = 0; i<QwtPlot::axisCnt; i++){
+		if (!g->axisEnabled(i))
+			continue;
 
 		QwtScaleWidget *axis = g->axisWidget(i);
 		if (!axis)
@@ -1983,10 +2049,16 @@ void AxesDialog::applyAxisFormatToLayer(Graph *g)
 		g->setAxisTicksLength(i, boxMajorTicksType->currentIndex(), boxMinorTicksType->currentIndex(),
 							boxMinorTicksLength->value(), boxMajorTicksLength->value());
 		g->setAxisFont(i, d_graph->axisFont(mapToQwtAxisId()));
+
+		ScaleDraw *sd = (ScaleDraw *)g->axisScaleDraw(i);
+		sd->setSpacing(boxTickLabelDistance->value());
+		sd->enableComponent (QwtAbstractScaleDraw::Backbone, boxAxisBackbone->isChecked());
+		sd->setShowTicksPolicy((ScaleDraw::ShowTicksPolicy)showTicksPolicyBox->currentIndex());
+
 		axis->repaint();
 	}
 	g->updateLayout();
-    g->replot();
+	g->replot();
 }
 
 void AxesDialog::applyAxisFormat()
@@ -2028,7 +2100,13 @@ void AxesDialog::applyAxisFormat()
 	app->modifiedProject();
 }
 
-void AxesDialog::setDisplayDateTimeFormat( const QString & format)
+void AxesDialog::setDisplayDateTimeFormat(const QString & format)
 {
+	ScaleDraw *scaleDraw = (ScaleDraw *) d_graph->axisScaleDraw(mapToQwtAxisId());
+	if (scaleDraw && scaleDraw->scaleType() == ScaleDraw::Time && (format == "M" || format == "S")){
+		originDateTimeBox->setDisplayFormat("hh:mm:ss");
+		return;
+	}
+
 	originDateTimeBox->setDisplayFormat(format);
 }
