@@ -48,37 +48,6 @@
 #include <DoubleSpinBox.h>
 #include <PatternBox.h>
 #include <PenStyleBox.h>
-#include <pixmaps.h>
-
-static const char* choose_folder_xpm[]={
-    "16 16 11 1",
-    "# c #000000",
-    "g c #c0c0c0",
-    "e c #303030",
-    "a c #ffa858",
-    "b c #808080",
-    "d c #a0a0a4",
-    "f c #585858",
-    "c c #ffdca8",
-    "h c #dcdcdc",
-    "i c #ffffff",
-    ". c None",
-    "....###.........",
-    "....#ab##.......",
-    "....#acab####...",
-    "###.#acccccca#..",
-    "#ddefaaaccccca#.",
-    "#bdddbaaaacccab#",
-    ".eddddbbaaaacab#",
-    ".#bddggdbbaaaab#",
-    "..edgdggggbbaab#",
-    "..#bgggghghdaab#",
-    "...ebhggghicfab#",
-    "....#edhhiiidab#",
-    "......#egiiicfb#",
-    "........#egiibb#",
-    "..........#egib#",
-    "............#ee#"};
 
 EnrichmentDialog::EnrichmentDialog(WidgetType wt, Graph *g, QWidget *parent)
     : QDialog(parent), d_plot(g), d_widget(NULL), d_widget_type(wt)
@@ -272,7 +241,7 @@ void EnrichmentDialog::initImagePage()
 
 	QPushButton *browseBtn = new QPushButton();
 	connect(browseBtn, SIGNAL(clicked()), this, SLOT(chooseImageFile()));
-	browseBtn->setIcon(QIcon(QPixmap(choose_folder_xpm)));
+	browseBtn->setIcon(QIcon(":/folder_open.png"));
 	gl->addWidget(browseBtn, 0, 2);
 
 	boxSaveImagesInternally = new QCheckBox(tr("&Save internally"));
@@ -433,8 +402,12 @@ void EnrichmentDialog::initGeometryPage()
 	attachToBox->insertItem(tr("Layer Scales"));
 	//attachToBox->setCurrentIndex((int)lm->attachPolicy());
 
-	bl1->addWidget(new QLabel(tr( "Attach to" )), 0, 0);
+	if (d_widget_type != MDIWindow)
+		bl1->addWidget(new QLabel(tr( "Attach to" )), 0, 0);
+
 	bl1->addWidget(attachToBox, 0, 1);
+	if (d_widget_type == MDIWindow)
+		attachToBox->hide();
 
 	unitBox = new QComboBox();
 	unitBox->insertItem(tr("inch"));
@@ -487,6 +460,7 @@ void EnrichmentDialog::initGeometryPage()
     gl2->addWidget(heightBox, 1, 1);
 
 	keepAspectBox = new QCheckBox(tr("&Keep aspect ratio"));
+	keepAspectBox->setChecked(app->d_keep_aspect_ration);
 	gl2->addWidget(keepAspectBox, 2, 1);
 
 	bestSizeButton = new QPushButton(tr("&Best size"));
@@ -673,6 +647,10 @@ void EnrichmentDialog::apply()
 		FrameWidget *fw = qobject_cast<FrameWidget *>(d_widget);
         if (fw)
             fw->setAttachPolicy((FrameWidget::AttachPolicy)attachToBox->currentIndex());
+
+		ApplicationWindow *app = (ApplicationWindow *)this->parent();
+		if (app)
+			app->d_keep_aspect_ration = keepAspectBox->isChecked();
 	} else if (patternPage && tabWidget->currentPage() == patternPage)
 		patternApplyTo();
 	else if (textPage && tabWidget->currentPage() == textPage){
@@ -788,14 +766,17 @@ void EnrichmentDialog::chooseImageFile(const QString& fn)
 	if (!app)
 		return;
 
+	ImageWidget *i = qobject_cast<ImageWidget *>(d_widget);
+	if (!i)
+		return;
+
 	QString path = fn;
 	if (path.isEmpty())
-		path = ApplicationWindow::getFileName(this, tr("QtiPlot - Import image from file"), app->imagesDirPath,
+		path = ApplicationWindow::getFileName(this, tr("QtiPlot - Import image from file"), i->fileName(),
 					ApplicationWindow::imageFilter(), 0, false);
 
 	if (!path.isEmpty()){
-		ImageWidget *i = qobject_cast<ImageWidget *>(d_widget);
-		if (i && i->load(path)){
+		if (i->load(path)){
 			imagePathBox->setText(path);
 			QFileInfo fi(path);
 			app->imagesDirPath = fi.dirPath(true);
@@ -832,14 +813,14 @@ void EnrichmentDialog::setCoordinates(int unit)
 		double left = xBox->value();
 		double top = yBox->value();
 		FrameWidget *fw = qobject_cast<FrameWidget *>(d_widget);
-        if (fw)
-            fw->setCoordinates(left, top, left + widthBox->value(), top - heightBox->value());
+		if (fw)
+			fw->setCoordinates(left, top, left + widthBox->value(), top - heightBox->value());
 	} else
 		FrameWidget::setRect(d_widget, xBox->value(), yBox->value(),
-        widthBox->value(), heightBox->value(), (FrameWidget::Unit)unit);
+		widthBox->value(), heightBox->value(), (FrameWidget::Unit)unit);
 
-    if (d_plot)
-        d_plot->multiLayer()->notifyChanges();
+	if (d_plot)
+		d_plot->multiLayer()->notifyChanges();
 
 	ApplicationWindow *app = (ApplicationWindow *)parentWidget();
 	if (app)
