@@ -32,6 +32,8 @@
 #include <MultiLayer.h>
 #include <FunctionCurve.h>
 #include <DoubleSpinBox.h>
+#include <ScriptEdit.h>
+#include <NonLinearFit.h>
 
 #include <QTextEdit>
 #include <QLineEdit>
@@ -46,22 +48,24 @@
 #include <QMessageBox>
 #include <QTableWidget>
 #include <QHeaderView>
+#include <QCompleter>
+#include <QStringListModel>
+#include <QInputDialog>
+#include <QApplication>
 
-FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
-: QDialog( parent, fl )
+FunctionDialog::FunctionDialog(ApplicationWindow* parent, bool standAlone, Qt::WFlags fl )
+: QDialog( parent, fl ), d_app(parent), d_active_editor(0), d_stand_alone(standAlone)
 {
-	ApplicationWindow *app = (ApplicationWindow *)parent;
 	QLocale locale = QLocale();
 	int prec = 6;
-	if (app){
-		locale = app->locale();
-		prec = app->d_decimal_digits;
+	if (d_app){
+		locale = d_app->locale();
+		prec = d_app->d_decimal_digits;
 	}
 
     setObjectName( "FunctionDialog" );
-	setWindowTitle( tr( "QtiPlot - Add function curve" ) );
-	setSizeGripEnabled(true);
-	setAttribute(Qt::WA_DeleteOnClose);
+	QString recentTip = tr("Click here to select a recently typed expression");
+	QString recentBtnText = tr("Rece&nt");
 
 	QHBoxLayout *hbox1 = new QHBoxLayout();
 	hbox1->addWidget(new QLabel(tr( "Curve type " )));
@@ -75,14 +79,25 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 	optionStack->setFrameShape( QFrame::StyledPanel );
 	optionStack->setFrameShadow( QStackedWidget::Plain );
 
+	QVBoxLayout *vl = new QVBoxLayout();
+	vl->addWidget(new QLabel(tr( "f(x)= " )));
+
+	buttonFunctionLog = new QPushButton(recentBtnText);
+	buttonFunctionLog->setToolTip(recentTip);
+	connect(buttonFunctionLog, SIGNAL(clicked()), this, SLOT(showFunctionLog()));
+	vl->addWidget(buttonFunctionLog);
+	vl->addStretch();
+
 	QGridLayout *gl1 = new QGridLayout();
-    gl1->addWidget(new QLabel(tr( "f(x)= " )), 0, 0);
-	boxFunction = new QTextEdit();
-	boxFunction->setMinimumWidth(350);
+	gl1->addLayout(vl, 0, 0);
+
+	boxFunction = new ScriptEdit(d_app->scriptingEnv());
+	boxFunction->enableShortcuts();
+	connect(boxFunction, SIGNAL(textChanged()), this, SLOT(guessConstants()));
 	gl1->addWidget(boxFunction, 0, 1);
+
 	gl1->addWidget(new QLabel(tr( "From x= " )), 1, 0);
 	boxFrom = new DoubleSpinBox();
-	boxFrom->setValue(0);
 	boxFrom->setLocale(locale);
 	boxFrom->setDecimals(prec);
 	gl1->addWidget(boxFrom, 1, 1);
@@ -115,8 +130,6 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 
 	QHBoxLayout *hb = new QHBoxLayout(functionPage);
 	hb->addLayout(gl1);
-	hb->addWidget(boxConstants);
-
 	optionStack->addWidget( functionPage );
 
 	QGridLayout *gl2 = new QGridLayout();
@@ -124,25 +137,59 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 	boxParameter = new QLineEdit();
 	boxParameter->setText("m");
 	gl2->addWidget(boxParameter, 0, 1);
-	gl2->addWidget(new QLabel(tr( "From" )), 1, 0);
-	boxParFrom = new QLineEdit();
-	boxParFrom->setText("0");
-	gl2->addWidget(boxParFrom, 1, 1);
-	gl2->addWidget(new QLabel(tr( "To" )), 2, 0);
-	boxParTo = new QLineEdit();
-	boxParTo->setText("1");
-	gl2->addWidget(boxParTo, 2, 1);
-	gl2->addWidget(new QLabel(tr( "x = " )), 3, 0);
-	boxXFunction = new QComboBox( );
-	boxXFunction->setEditable ( true );
-	gl2->addWidget(boxXFunction, 3, 1);
-	gl2->addWidget(new QLabel(tr( "y = " )), 4, 0);
-	boxYFunction = new QComboBox( );
-	boxYFunction->setEditable ( true );
-	gl2->addWidget(boxYFunction, 4, 1);
+
+	int maxH = 80;
+
+	boxXFunction = new ScriptEdit(d_app->scriptingEnv());
+	boxXFunction->setMaximumHeight(maxH);
+	boxXFunction->enableShortcuts();
+	connect(boxXFunction, SIGNAL(activated(ScriptEdit *)), this, SLOT(setActiveEditor(ScriptEdit *)));
+	connect(boxXFunction, SIGNAL(textChanged()), this, SLOT(guessConstants()));
+	gl2->addWidget(boxXFunction, 1, 1);
+
+	buttonXParLog = new QPushButton(recentBtnText);
+	buttonXParLog->setToolTip(recentTip);
+	connect(buttonXParLog, SIGNAL(clicked()), this, SLOT(showXParLog()));
+
+	QVBoxLayout *vlxpar = new QVBoxLayout();
+	vlxpar->addWidget(new QLabel(tr( "x = " )));
+	vlxpar->addWidget(buttonXParLog);
+	vlxpar->addStretch();
+	gl2->addLayout(vlxpar, 1, 0);
+
+	boxYFunction = new ScriptEdit(d_app->scriptingEnv());
+	boxYFunction->setMaximumHeight(maxH);
+	boxYFunction->enableShortcuts();
+	connect(boxYFunction, SIGNAL(activated(ScriptEdit *)), this, SLOT(setActiveEditor(ScriptEdit *)));
+	connect(boxYFunction, SIGNAL(textChanged()), this, SLOT(guessConstants()));
+	gl2->addWidget(boxYFunction, 2, 1);
+
+	buttonYParLog = new QPushButton(recentBtnText);
+	buttonYParLog->setToolTip(recentTip);
+	connect(buttonYParLog, SIGNAL(clicked()), this, SLOT(showYParLog()));
+
+	QVBoxLayout *vlypar = new QVBoxLayout();
+	vlypar->addWidget(new QLabel(tr( "y = " )));
+	vlypar->addWidget(buttonYParLog);
+	vlypar->addStretch();
+	gl2->addLayout(vlypar, 2, 0);
+
+	gl2->addWidget(new QLabel(tr( "From" )), 3, 0);
+	boxParFrom = new DoubleSpinBox();
+	boxParFrom->setLocale(locale);
+	boxParFrom->setDecimals(prec);
+	gl2->addWidget(boxParFrom, 3, 1);
+
+	gl2->addWidget(new QLabel(tr( "To" )), 4, 0);
+	boxParTo = new DoubleSpinBox();
+	boxParTo->setValue(1);
+	boxParTo->setLocale(locale);
+	boxParTo->setDecimals(prec);
+	gl2->addWidget(boxParTo, 4, 1);
+
 	gl2->addWidget(new QLabel(tr( "Points" )), 5, 0);
 	boxParPoints = new QSpinBox();
-	boxParPoints->setRange(2, 1000000);
+	boxParPoints->setRange(2, INT_MAX);
 	boxParPoints->setSingleStep(100);
 	boxParPoints->setValue(100);
 	gl2->addWidget(boxParPoints, 5, 1);
@@ -157,29 +204,61 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 	boxPolarParameter = new QLineEdit();
 	boxPolarParameter->setText ("t");
 	gl3->addWidget(boxPolarParameter, 0, 1);
-	gl3->addWidget(new QLabel(tr( "From" )), 2, 0);
-	boxPolarFrom = new QLineEdit();
-	boxPolarFrom->setText("0");
-	gl3->addWidget(boxPolarFrom, 2, 1);
-	gl3->addWidget(new QLabel(tr( "To" )), 3, 0);
-	boxPolarTo = new QLineEdit();
-	boxPolarTo->setText("pi");
-	gl3->addWidget(boxPolarTo, 3, 1);
-	gl3->addWidget(new QLabel(tr( "R =" )), 4, 0);
-	boxPolarRadius = new QComboBox();
-	boxPolarRadius->setEditable ( true );
-	gl3->addWidget(boxPolarRadius, 4, 1);
-	gl3->addWidget(new QLabel(tr( "Theta =" )), 5, 0);
-	boxPolarTheta = new QComboBox();
-	boxPolarTheta->setEditable ( true );
-	gl3->addWidget(boxPolarTheta, 5, 1);
-	gl3->addWidget(new QLabel(tr( "Points" )), 6, 0);
+
+	boxPolarRadius = new ScriptEdit(d_app->scriptingEnv());
+	boxPolarRadius->setMaximumHeight(maxH);
+	boxPolarRadius->enableShortcuts();
+	connect(boxPolarRadius, SIGNAL(activated(ScriptEdit *)), this, SLOT(setActiveEditor(ScriptEdit *)));
+	connect(boxPolarRadius, SIGNAL(textChanged()), this, SLOT(guessConstants()));
+	gl3->addWidget(boxPolarRadius, 1, 1);
+
+	buttonPolarRadiusLog = new QPushButton(recentBtnText);
+	buttonPolarRadiusLog->setToolTip(recentTip);
+	connect(buttonPolarRadiusLog, SIGNAL(clicked()), this, SLOT(showPolarRadiusLog()));
+
+	QVBoxLayout *vlrpar = new QVBoxLayout();
+	vlrpar->addWidget(new QLabel(tr( "R =" )));
+	vlrpar->addWidget(buttonPolarRadiusLog);
+	vlrpar->addStretch();
+	gl3->addLayout(vlrpar, 1, 0);
+
+	boxPolarTheta = new ScriptEdit(d_app->scriptingEnv());
+	boxPolarTheta->setMaximumHeight(maxH);
+	boxPolarTheta->enableShortcuts();
+	connect(boxPolarTheta, SIGNAL(activated(ScriptEdit *)), this, SLOT(setActiveEditor(ScriptEdit *)));
+	connect(boxPolarTheta, SIGNAL(textChanged()), this, SLOT(guessConstants()));
+	gl3->addWidget(boxPolarTheta, 2, 1);
+
+	buttonPolarRThetaLog = new QPushButton(recentBtnText);
+	buttonPolarRThetaLog->setToolTip(recentTip);
+	connect(buttonPolarRThetaLog, SIGNAL(clicked()), this, SLOT(showPolarThetaLog()));
+
+	QVBoxLayout *vltpar = new QVBoxLayout();
+	vltpar->addWidget(new QLabel(tr( "Theta =" )));
+	vltpar->addWidget(buttonPolarRThetaLog);
+	vltpar->addStretch();
+	gl3->addLayout(vltpar, 2, 0);
+
+	gl3->addWidget(new QLabel(tr( "From" )), 3, 0);
+	boxPolarFrom = new DoubleSpinBox();
+	boxPolarFrom->setLocale(locale);
+	boxPolarFrom->setDecimals(prec);
+	gl3->addWidget(boxPolarFrom, 3, 1);
+
+	gl3->addWidget(new QLabel(tr( "To" )), 4, 0);
+	boxPolarTo = new DoubleSpinBox();
+	boxPolarTo->setValue(M_PI);
+	boxPolarTo->setLocale(locale);
+	boxPolarTo->setDecimals(prec);
+	gl3->addWidget(boxPolarTo, 4, 1);
+
+	gl3->addWidget(new QLabel(tr( "Points" )), 5, 0);
 	boxPolarPoints = new QSpinBox();
-	boxPolarPoints->setRange(2, 1000000);
+	boxPolarPoints->setRange(2, INT_MAX);
 	boxPolarPoints->setSingleStep(100);
 	boxPolarPoints->setValue(100);
-	gl3->addWidget(boxPolarPoints, 6, 1);
-	gl3->setRowStretch(7, 1);
+	gl3->addWidget(boxPolarPoints, 5, 1);
+	gl3->setRowStretch(6, 1);
 
 	polarPage = new QWidget();
 	polarPage->setLayout(gl3);
@@ -187,7 +266,7 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 
 	boxFunctionExplain = new QTextEdit;
 	boxFunctionExplain->setReadOnly(true);
-	boxFunctionExplain->setMaximumHeight(50);
+	boxFunctionExplain->setMaximumHeight(80);
 	QPalette palette = boxFunctionExplain->palette();
 	palette.setColor(QPalette::Active, QPalette::Base, Qt::lightGray);
 	boxFunctionExplain->setPalette(palette);
@@ -205,36 +284,52 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 	vbox->addWidget(boxMathFunctions);
 	vbox->addWidget(addFunctionBtn);
 
+	buttonClear = new QPushButton(tr( "Clea&r Function" ));
+	connect( buttonClear, SIGNAL( clicked() ), this, SLOT(clearList()));
+	vbox->addWidget(buttonClear);
+	vbox->addStretch();
+
 	QHBoxLayout *hbox3 = new QHBoxLayout();
 	hbox3->addWidget(boxFunctionExplain);
 	hbox3->addLayout(vbox);
 
-	buttonClear = new QPushButton(tr( "Clea&r Function" ));
-	buttonClear->setAutoDefault(false);
-	buttonOk = new QPushButton(tr( "&Ok" ));
-	buttonOk->setDefault(true);
-	buttonCancel = new QPushButton(tr( "&Close" ));
-	buttonCancel->setAutoDefault(false);
+	QVBoxLayout *vbox2 = new QVBoxLayout();
+	vbox2->addLayout(hbox3);
+	vbox2->addStretch();
 
-	QHBoxLayout *hbox2 = new QHBoxLayout();
-	hbox2->addWidget(buttonClear);
-	hbox2->addWidget(buttonOk);
-	hbox2->addWidget(buttonCancel);
-	hbox2->addStretch();
+	hb = new QHBoxLayout();
+	hb->addWidget(optionStack);
+	hb->addWidget(boxConstants);
 
-	QVBoxLayout *vbox1 = new QVBoxLayout();
-    vbox1->addLayout(hbox1);
-	vbox1->addWidget(optionStack);
-	vbox1->addLayout(hbox3);
-	vbox1->addLayout(hbox2);
+	QVBoxLayout *vbox1 = new QVBoxLayout(this);
+	vbox1->addLayout(hbox1);
+	vbox1->addLayout(hb);
+	vbox1->addLayout(vbox2);
+	vbox1->addStretch();
+
+	if (standAlone){
+		buttonOk = new QPushButton(tr( "&Ok" ));
+		buttonOk->setDefault(true);
+		connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
+
+		buttonCancel = new QPushButton(tr( "&Close" ));
+		buttonCancel->setAutoDefault(false);
+		connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
+
+		QHBoxLayout *hbox2 = new QHBoxLayout();
+		hbox2->addWidget(buttonOk);
+		hbox2->addWidget(buttonCancel);
+		hbox2->addStretch();
+
+		vbox1->addLayout(hbox2);
+
+		setSizeGripEnabled(true);
+		setWindowTitle( tr( "QtiPlot - Add function curve" ) );
+		setAttribute(Qt::WA_DeleteOnClose);
+	}
 
-	setLayout(vbox1);
 	setFocusProxy (boxFunction);
-
 	connect( boxType, SIGNAL( activated(int) ), this, SLOT( raiseWidget(int) ) );
-	connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
-	connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
-	connect( buttonClear, SIGNAL( clicked() ), this, SLOT(clearList() ) );
 
 	curveID = -1;
 	graph = 0;
@@ -242,12 +337,17 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 
 void FunctionDialog::raiseWidget(int index)
 {
-	if (index)
-		buttonClear->setText( tr( "Clear list" ) );
-	else
-		buttonClear->setText( tr( "Clear Function" ) );
-
 	optionStack->setCurrentIndex(index);
+	guessConstants();
+}
+
+void FunctionDialog::setCurveToModify(FunctionCurve *c)
+{
+	Graph *g = (Graph *)c->plot();
+	if (!g)
+		return;
+
+	setCurveToModify(g, g->curveIndex(c));
 }
 
 void FunctionDialog::setCurveToModify(Graph *g, int curve)
@@ -269,7 +369,6 @@ void FunctionDialog::setCurveToModify(Graph *g, int curve)
 		boxConstants->clearContents();
 		boxConstants->setRowCount(constants.size());
 		boxConstants->show();
-		ApplicationWindow *app = (ApplicationWindow *)parent();
 		QMapIterator<QString, double> i(constants);
 		int row = 0;
  		while (i.hasNext()) {
@@ -277,46 +376,57 @@ void FunctionDialog::setCurveToModify(Graph *g, int curve)
 			boxConstants->setItem(row, 0, new QTableWidgetItem(i.key()));
 
 			DoubleSpinBox *sb = new DoubleSpinBox();
-			sb->setLocale(app->locale());
-			sb->setDecimals(app->fit_output_precision);
+			sb->setLocale(QLocale());
 			sb->setValue(i.value());
         	boxConstants->setCellWidget(row, 1, sb);
-			connect(sb, SIGNAL(valueChanged(double)), this, SLOT(acceptFunction()));
+			connect(sb, SIGNAL(valueChanged(double)), this, SLOT(accept()));
 			row++;
  		}
-	}
+	} else
+		boxConstants->hide();
 
 	if (c->functionType() == FunctionCurve::Normal){
+		boxFunction->blockSignals(true);
 		boxFunction->setText(formulas[0]);
-		boxFrom->setValue(c->startRange());
-		boxTo->setValue(c->endRange());
+		boxFunction->blockSignals(false);
+
+		boxFrom->setValue(QMIN(c->startRange(), c->endRange()));
+		boxTo->setValue(QMAX(c->startRange(), c->endRange()));
 		boxPoints->setValue(c->dataSize());
 	} else if (c->functionType() == FunctionCurve::Polar) {
-		optionStack->setCurrentIndex(2);
-		boxType->setCurrentItem(2);
+		boxPolarRadius->blockSignals(true);
+		boxPolarRadius->setText(formulas[0]);
+		boxPolarRadius->blockSignals(false);
+
+		boxPolarTheta->blockSignals(true);
+		boxPolarTheta->setText(formulas[1]);
+		boxPolarTheta->blockSignals(false);
 
-		boxPolarRadius->setCurrentText(formulas[0]);
-		boxPolarTheta->setCurrentText(formulas[1]);
 		boxPolarParameter->setText(c->variable());
-		boxPolarFrom->setText(QString::number(c->startRange(), 'g', 15));
-		boxPolarTo->setText(QString::number(c->endRange(), 'g', 15));
+		boxPolarFrom->setValue(c->startRange());
+		boxPolarTo->setValue(c->endRange());
 		boxPolarPoints->setValue(c->dataSize());
 	} else if (c->functionType() == FunctionCurve::Parametric) {
-		boxType->setCurrentItem(1);
-		optionStack->setCurrentIndex(1);
+		boxXFunction->blockSignals(true);
+		boxXFunction->setText(formulas[0]);
+		boxXFunction->blockSignals(false);
+
+		boxYFunction->blockSignals(true);
+		boxYFunction->setText(formulas[1]);
+		boxYFunction->blockSignals(false);
 
-		boxXFunction->setCurrentText(formulas[0]);
-		boxYFunction->setCurrentText(formulas[1]);
 		boxParameter->setText(c->variable());
-		boxParFrom->setText(QString::number(c->startRange(), 'g', 15));
-		boxParTo->setText(QString::number(c->endRange(), 'g', 15));
+		boxParFrom->setValue(c->startRange());
+		boxParTo->setValue(c->endRange());
 		boxParPoints->setValue(c->dataSize());
 	}
+	boxType->setCurrentIndex(c->functionType());
+	optionStack->setCurrentIndex(c->functionType());
 }
 
 void FunctionDialog::clearList()
 {
-	int type=boxType->currentItem();
+	int type = boxType->currentItem();
 	switch (type)
 	{
 		case 0:
@@ -326,13 +436,11 @@ void FunctionDialog::clearList()
 		case 1:
 			boxXFunction->clear();
 			boxYFunction->clear();
-			emit clearParamFunctionsList();
 			break;
 
 		case 2:
 			boxPolarTheta->clear();
 			boxPolarRadius->clear();
-			emit clearPolarFunctionsList();
 			break;
 	}
 }
@@ -378,57 +486,29 @@ void FunctionDialog::acceptFunction()
 	int type = boxType->currentItem();
 	QStringList formulas;
 	formulas += formula;
-	if (!error){
-		ApplicationWindow *app = (ApplicationWindow *)this->parent();
-		app->updateFunctionLists(type,formulas);
+	if (!error && d_app){
+		d_app->updateFunctionLists(type, formulas);
 		if (!graph){
-			MultiLayer *plot = app->newFunctionPlot(formulas, start, end, boxPoints->value(), "x", type);
-			if (plot)
+			MultiLayer *plot = d_app->newFunctionPlot(formulas, start, end, boxPoints->value(), "x", type);
+			if (plot){
 				graph = plot->activeLayer();
+				setConstants((FunctionCurve *)graph->curve(graph->curveCount() - 1), constants);
+			}
 		} else {
 			if (curveID >= 0)
 				graph->modifyFunctionCurve(curveID, type, formulas, "x", start, end, boxPoints->value(), constants);
 			else
-				graph->addFunction(formulas, start, end, boxPoints->value(), "x", type);
+				setConstants(graph->addFunction(formulas, start, end, boxPoints->value(), "x", type), constants);
 		}
 	}
-
 }
+
 void FunctionDialog::acceptParametric()
 {
-	QString from=boxParFrom->text().lower();
-	QString to=boxParTo->text().lower();
-	QString points=boxParPoints->text().lower();
+	double start = boxParFrom->value();
+	double end = boxParTo->value();
 
-	double start = 0.0, end = 0.0;
-	try
-	{
-		MyParser parser;
-		parser.SetExpr(from.ascii());
-		start = parser.Eval();
-	}
-	catch(mu::ParserError &e)
-	{
-		QMessageBox::critical(this, tr("QtiPlot - Start limit error"), QString::fromStdString(e.GetMsg()));
-		boxParFrom->setFocus();
-		return;
-	}
-
-	try
-	{
-		MyParser parser;
-		parser.SetExpr(to.ascii());
-		end=parser.Eval();
-	}
-	catch(mu::ParserError &e)
-	{
-		QMessageBox::critical(this, tr("QtiPlot - End limit error"), QString::fromStdString(e.GetMsg()));
-		boxParTo->setFocus();
-		return;
-	}
-
-	if (start>=end)
-	{
+	if (start >= end){
 		QMessageBox::critical(this, tr("QtiPlot - Input error"),
 				tr("Please enter parameter limits that satisfy: from < end!"));
 		boxParTo->setFocus();
@@ -436,100 +516,83 @@ void FunctionDialog::acceptParametric()
 	}
 
 	double parameter;
-	QString xformula=boxXFunction->currentText();
-	QString yformula=boxYFunction->currentText();
-	bool error=false;
+	QString xformula = boxXFunction->text().simplified();
+	QString yformula = boxYFunction->text().simplified();
+	bool error = false;
 
-	try
-	{
+	QMap<QString, double> constants;
+
+	try {
 		MyParser parser;
 		parser.DefineVar((boxParameter->text()).ascii(), &parameter);
+		for (int i = 0; i < boxConstants->rowCount(); i++){
+			double val = ((DoubleSpinBox*)boxConstants->cellWidget(i, 1))->value();
+			QString constName = boxConstants->item(i, 0)->text();
+			if (!constName.isEmpty()){
+				constants.insert(constName, val);
+				parser.DefineConst(constName.ascii(), val);
+			}
+		}
 		parser.SetExpr(xformula.ascii());
 
-		parameter=start;
+		parameter = start;
 		parser.Eval();
-		parameter=end;
+		parameter = end;
 		parser.Eval();
-	}
-	catch(mu::ParserError &e)
-	{
+	} catch(mu::ParserError &e) {
 		QMessageBox::critical(this, tr("QtiPlot - Input function error"), QString::fromStdString(e.GetMsg()));
 		boxXFunction->setFocus();
-		error=true;
+		error = true;
 	}
-	try
-	{
+
+	try {
 		MyParser parser;
 		parser.DefineVar((boxParameter->text()).ascii(), &parameter);
+
+		for (int i = 0; i < boxConstants->rowCount(); i++){
+			double val = ((DoubleSpinBox*)boxConstants->cellWidget(i, 1))->value();
+			QString constName = boxConstants->item(i, 0)->text();
+			if (!constName.isEmpty())
+				parser.DefineConst(constName.ascii(), val);
+		}
 		parser.SetExpr(yformula.ascii());
 
-		parameter=start;
+		parameter = start;
 		parser.Eval();
-		parameter=end;
+		parameter = end;
 		parser.Eval();
-	}
-	catch(mu::ParserError &e)
-	{
+	} catch(mu::ParserError &e) {
 		QMessageBox::critical(this, tr("QtiPlot - Input function error"), QString::fromStdString(e.GetMsg()));
 		boxYFunction->setFocus();
-		error=true;
+		error = true;
 	}
 	// Collecting all the information
 	int type = boxType->currentItem();
 	QStringList formulas;
 	formulas += xformula;
 	formulas += yformula;
-	if (!error){
-		ApplicationWindow *app = (ApplicationWindow *)this->parent();
-		app->updateFunctionLists(type,formulas);
+	if (!error && d_app){
+		d_app->updateFunctionLists(type, formulas);
 		if (!graph){
-			MultiLayer *plot = app->newFunctionPlot(formulas, start, end, boxParPoints->value(), boxParameter->text(), type);
-			if (plot)
+			MultiLayer *plot = d_app->newFunctionPlot(formulas, start, end, boxParPoints->value(), boxParameter->text(), type);
+			if (plot){
 				graph = plot->activeLayer();
+				setConstants((FunctionCurve *)graph->curve(graph->curveCount() - 1), constants);
+			}
 		} else {
 			if (curveID >= 0)
-				graph->modifyFunctionCurve(curveID, type, formulas, boxParameter->text(), start, end, boxParPoints->value(), QMap<QString, double>());
+				graph->modifyFunctionCurve(curveID, type, formulas, boxParameter->text(), start, end, boxParPoints->value(), constants);
 			else
-				graph->addFunction(formulas, start, end, boxParPoints->value(), boxParameter->text(), type);
+				setConstants(graph->addFunction(formulas, start, end, boxParPoints->value(), boxParameter->text(), type), constants);
 		}
 	}
 }
 
 void FunctionDialog::acceptPolar()
 {
-	QString from=boxPolarFrom->text().lower();
-	QString to=boxPolarTo->text().lower();
-	QString points=boxPolarPoints->text().lower();
-
-	double start = 0, end = 0;
-	try
-	{
-		MyParser parser;
-		parser.SetExpr(from.ascii());
-		start=parser.Eval();
-	}
-	catch(mu::ParserError &e)
-	{
-		QMessageBox::critical(this, tr("QtiPlot - Start limit error"), QString::fromStdString(e.GetMsg()));
-		boxPolarFrom->setFocus();
-		return;
-	}
-
-	try
-	{
-		MyParser parser;
-		parser.SetExpr(to.ascii());
-		end=parser.Eval();
-	}
-	catch(mu::ParserError &e)
-	{
-		QMessageBox::critical(this, tr("QtiPlot - End limit error"), QString::fromStdString(e.GetMsg()));
-		boxPolarTo->setFocus();
-		return;
-	}
-
-	if (start>=end)
-	{
+	double start = boxPolarFrom->value();
+	double end = boxPolarTo->value();
+	if (start >= end){
 		QMessageBox::critical(this, tr("QtiPlot - Input error"),
 				tr("Please enter parameter limits that satisfy: from < end!"));
 		boxPolarTo->setFocus();
@@ -537,62 +600,74 @@ void FunctionDialog::acceptPolar()
 	}
 
 	double parameter;
-	QString rformula=boxPolarRadius->currentText();
-	QString tformula=boxPolarTheta->currentText();
-	bool error=false;
+	QString rformula = boxPolarRadius->text().simplified();
+	QString tformula = boxPolarTheta->text().simplified();
+	bool error = false;
 
-	try
-	{
+	QMap<QString, double> constants;
+
+	try {
 		MyParser parser;
 		parser.DefineVar((boxPolarParameter->text()).ascii(), &parameter);
+		for (int i = 0; i < boxConstants->rowCount(); i++){
+			double val = ((DoubleSpinBox*)boxConstants->cellWidget(i, 1))->value();
+			QString constName = boxConstants->item(i, 0)->text();
+			if (!constName.isEmpty()){
+				constants.insert(constName, val);
+				parser.DefineConst(constName.ascii(), val);
+			}
+		}
 		parser.SetExpr(rformula.ascii());
 
-		parameter=start;
+		parameter = start;
 		parser.Eval();
-		parameter=end;
+		parameter = end;
 		parser.Eval();
-	}
-	catch(mu::ParserError &e)
-	{
+	} catch(mu::ParserError &e) {
 		QMessageBox::critical(this, tr("QtiPlot - Input function error"), QString::fromStdString(e.GetMsg()));
 		boxPolarRadius->setFocus();
-		error=true;
+		error = true;
 	}
-	try
-	{
+
+	try {
 		MyParser parser;
 		parser.DefineVar((boxPolarParameter->text()).ascii(), &parameter);
+		for (int i = 0; i < boxConstants->rowCount(); i++){
+			double val = ((DoubleSpinBox*)boxConstants->cellWidget(i, 1))->value();
+			QString constName = boxConstants->item(i, 0)->text();
+			if (!constName.isEmpty())
+				parser.DefineConst(constName.ascii(), val);
+		}
 		parser.SetExpr(tformula.ascii());
 
-		parameter=start;
+		parameter = start;
 		parser.Eval();
-		parameter=end;
+		parameter = end;
 		parser.Eval();
-	}
-	catch(mu::ParserError &e)
-	{
+	} catch(mu::ParserError &e) {
 		QMessageBox::critical(this, tr("QtiPlot - Input function error"), QString::fromStdString(e.GetMsg()));
 		boxPolarTheta->setFocus();
-		error=true;
+		error = true;
 	}
 	// Collecting all the information
 	int type = boxType->currentItem();
 	QStringList formulas;
-	formulas+=rformula;
-	formulas+=tformula;
-	if (!error){
-		ApplicationWindow *app = (ApplicationWindow *)this->parent();
-		app->updateFunctionLists(type,formulas);
+	formulas += rformula;
+	formulas += tformula;
+	if (!error && d_app){
+		d_app->updateFunctionLists(type, formulas);
 
 		if (!graph){
-			MultiLayer *plot = app->newFunctionPlot(formulas, start, end, boxPolarPoints->value(), boxPolarParameter->text(), type);
-			if (plot)
+			MultiLayer *plot = d_app->newFunctionPlot(formulas, start, end, boxPolarPoints->value(), boxPolarParameter->text(), type);
+			if (plot){
 				graph = plot->activeLayer();
+				setConstants((FunctionCurve *)graph->curve(graph->curveCount() - 1), constants);
+			}
 		} else {
 			if (curveID >= 0)
-				graph->modifyFunctionCurve(curveID, type, formulas, boxPolarParameter->text(), start, end, boxPolarPoints->value(), QMap<QString, double>());
+				graph->modifyFunctionCurve(curveID, type, formulas, boxPolarParameter->text(), start, end, boxPolarPoints->value(), constants);
 			else
-				graph->addFunction(formulas, start, end, boxPolarPoints->value(), boxPolarParameter->text(), type);
+				setConstants(graph->addFunction(formulas, start, end, boxPolarPoints->value(), boxPolarParameter->text(), type), constants);
 		}
 	}
 }
@@ -615,41 +690,110 @@ void FunctionDialog::accept()
 	}
 }
 
-void FunctionDialog::insertParamFunctionsList(const QStringList& xList, const QStringList& yList)
+void FunctionDialog::showFunctionLog()
 {
-	boxXFunction->insertItems (0, xList);
-	boxYFunction->insertItems (0, yList);
+	if (!d_app)
+		return;
+
+	if (d_app->d_recent_functions.isEmpty()){
+		QMessageBox::information(this, tr("QtiPlot"), tr("Sorry, there are no recent expressions available!"));
+		return;
+	}
+
+	bool ok;
+	QString s = QInputDialog::getItem(this, tr("QtiPlot") + " - " + tr("Recent Functions"), tr("Please, choose a function:"), d_app->d_recent_functions, 0, false, &ok);
+	if (ok && !s.isEmpty())
+		boxFunction->setText(s);
 }
 
-void FunctionDialog::insertPolarFunctionsList(const QStringList& rList, const QStringList& thetaList)
+void FunctionDialog::showXParLog()
 {
-	boxPolarRadius->insertItems (0, rList);
-	boxPolarTheta->insertItems (0, thetaList);
+	if (!d_app)
+		return;
+
+	if (d_app->xFunctions.isEmpty()){
+		QMessageBox::information(this, tr("QtiPlot"), tr("Sorry, there are no recent expressions available!"));
+		return;
+	}
+
+	bool ok;
+	QString s = QInputDialog::getItem(this, tr("QtiPlot") + " - " + tr("Recent Functions"), tr("Please, choose a function:"), d_app->xFunctions, 0, false, &ok);
+	if (ok && !s.isEmpty())
+		boxXFunction->setText(s);
+}
+
+
+void FunctionDialog::showYParLog()
+{
+	if (!d_app)
+		return;
+
+	if (d_app->yFunctions.isEmpty()){
+		QMessageBox::information(this, tr("QtiPlot"), tr("Sorry, there are no recent expressions available!"));
+		return;
+	}
+
+	bool ok;
+	QString s = QInputDialog::getItem(this, tr("QtiPlot") + " - " + tr("Recent Functions"), tr("Please, choose a function:"), d_app->yFunctions, 0, false, &ok);
+	if (ok && !s.isEmpty())
+		boxYFunction->setText(s);
+}
+
+void FunctionDialog::showPolarRadiusLog()
+{
+	if (!d_app)
+		return;
+
+	if (d_app->rFunctions.isEmpty()){
+		QMessageBox::information(this, tr("QtiPlot"), tr("Sorry, there are no recent expressions available!"));
+		return;
+	}
+
+	bool ok;
+	QString s = QInputDialog::getItem(this, tr("QtiPlot") + " - " + tr("Recent Functions"), tr("Please, choose a function:"), d_app->rFunctions, 0, false, &ok);
+	if (ok && !s.isEmpty())
+		boxPolarRadius->setText(s);
+}
+
+
+void FunctionDialog::showPolarThetaLog()
+{
+	if (!d_app)
+		return;
+
+	if (d_app->thetaFunctions.isEmpty()){
+		QMessageBox::information(this, tr("QtiPlot"), tr("Sorry, there are no recent expressions available!"));
+		return;
+	}
+
+	bool ok;
+	QString s = QInputDialog::getItem(this, tr("QtiPlot") + " - " + tr("Recent Functions"), tr("Please, choose a function:"), d_app->thetaFunctions, 0, false, &ok);
+	if (ok && !s.isEmpty())
+		boxPolarTheta->setText(s);
 }
 
 void FunctionDialog::insertFunction()
 {
-	QString fname = boxMathFunctions->currentText();
+	QString fname = boxMathFunctions->currentText().remove("(").remove(")").remove(",").remove(";");
 	if (optionStack->currentWidget () == functionPage){
-		QTextCursor cursor = boxFunction->textCursor();
-		QString markedText = cursor.selectedText();
-		if(markedText.isEmpty()){
-			cursor.insertText(fname);
-			cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::MoveAnchor,1);
-		} else
-			cursor.insertText(fname.remove(")").remove(",") + markedText + ")");
-
-		boxFunction->setTextCursor(cursor);
+		boxFunction->insertFunction(fname);
+		boxFunction->setFocus();
 	} else if (optionStack->currentWidget () == parametricPage){
-		if (boxYFunction->lineEdit()->hasFocus())
-			boxYFunction->lineEdit()->insert(fname);
-		else
-			boxXFunction->lineEdit()->insert(fname);
+		if (d_active_editor == boxYFunction){
+			boxYFunction->insertFunction(fname);
+			boxYFunction->setFocus();
+		} else if (d_active_editor == boxXFunction){
+			boxXFunction->insertFunction(fname);
+			boxXFunction->setFocus();
+		}
 	} else if (optionStack->currentWidget () == polarPage){
-		if (boxPolarRadius->lineEdit()->hasFocus())
-			boxPolarRadius->lineEdit()->insert(fname);
-		else
-			boxPolarTheta->lineEdit()->insert(fname);
+		if (d_active_editor == boxPolarRadius){
+			boxPolarRadius->insertFunction(fname);
+			boxPolarRadius->setFocus();
+		} else if (d_active_editor == boxPolarTheta){
+			boxPolarTheta->insertFunction(fname);
+			boxPolarTheta->setFocus();
+		}
 	}
 }
 
@@ -657,3 +801,82 @@ void FunctionDialog::updateFunctionExplain(int index)
 {
 	boxFunctionExplain->setText(MyParser::explainFunction(index));
 }
+
+void FunctionDialog::guessConstants()
+{
+	QString text;
+	QString var = "x";
+	switch (boxType->currentIndex()){
+		case 0:
+			text = boxFunction->text().remove(QRegExp("\\s")).remove(".");
+			break;
+
+		case 1:
+			text = boxXFunction->text().remove(QRegExp("\\s")).remove(".");
+			text += "+" + boxYFunction->text().remove(QRegExp("\\s")).remove(".");
+			var = boxParameter->text();
+			break;
+
+		case 2:
+			text = boxPolarRadius->text().remove(QRegExp("\\s")).remove(".");
+			text += "+" + boxPolarTheta->text().remove(QRegExp("\\s")).remove(".");
+			var = boxPolarParameter->text();
+			break;
+	}
+
+	bool error = false;
+	string errMsg;
+	QStringList lst = NonLinearFit::guessParameters(text, &error, &errMsg, var);
+	if (error)
+		return;
+
+	QStringList constants;
+	QList<double> values;
+	for (int i = 0; i < boxConstants->rowCount(); i++){
+		constants << boxConstants->item(i, 0)->text();
+		values << ((DoubleSpinBox*)boxConstants->cellWidget(i, 1))->value();
+	}
+
+	if (lst == constants)
+		return;
+
+	boxConstants->setRowCount(lst.size());
+
+	int row = 0;
+	foreach(QString s, lst){
+		boxConstants->setItem(row, 0, new QTableWidgetItem(s));
+
+		DoubleSpinBox *sb = new DoubleSpinBox();
+		sb->setLocale(QLocale());
+		boxConstants->setCellWidget(row, 1, sb);
+		if (constants.contains(s)){
+			int index = constants.indexOf(s);
+			if (index >= 0 && index < values.size())
+				sb->setValue(values[index]);
+		}
+		if (!d_stand_alone)
+			connect(sb, SIGNAL(valueChanged(double)), this, SLOT(accept()));
+		row++;
+	}
+
+	if (!lst.size())
+		boxConstants->hide();
+	else if (!boxConstants->isVisible())
+		boxConstants->show();
+}
+
+void FunctionDialog::setConstants(FunctionCurve *c, const QMap<QString, double>& constants)
+{
+	if (!c || constants.isEmpty())
+		return;
+
+	c->setConstants(constants);
+	c->loadData();
+	if (graph){
+		graph->replot();
+		graph->updateMarkersBoundingRect();
+		graph->updateSecondaryAxis(QwtPlot::xTop);
+		graph->updateSecondaryAxis(QwtPlot::yRight);
+		graph->replot();
+	}
+}
