@@ -55,36 +55,6 @@
 
 #include <qwt3d_color.h>
 
-static const char* choose_folder_xpm[]={
-    "16 16 11 1",
-    "# c #000000",
-    "g c #c0c0c0",
-    "e c #303030",
-    "a c #ffa858",
-    "b c #808080",
-    "d c #a0a0a4",
-    "f c #585858",
-    "c c #ffdca8",
-    "h c #dcdcdc",
-    "i c #ffffff",
-    ". c None",
-    "....###.........",
-    "....#ab##.......",
-    "....#acab####...",
-    "###.#acccccca#..",
-    "#ddefaaaccccca#.",
-    "#bdddbaaaacccab#",
-    ".eddddbbaaaacab#",
-    ".#bddggdbbaaaab#",
-    "..edgdggggbbaab#",
-    "..#bgggghghdaab#",
-    "...ebhggghicfab#",
-    "....#edhhiiidab#",
-    "......#egiiicfb#",
-    "........#egiibb#",
-    "..........#egib#",
-    "............#ee#"};
-
 Plot3DDialog::Plot3DDialog( QWidget* parent,  Qt::WFlags fl )
     : QDialog( parent, fl )
 {
@@ -114,6 +84,7 @@ Plot3DDialog::Plot3DDialog( QWidget* parent,  Qt::WFlags fl )
 	initTitlePage();
 	initColorsPage();
 	initGeneralPage();
+	initPrintPage();
 
 	QVBoxLayout* vl = new QVBoxLayout(this);
 	vl->addWidget(generalDialog);
@@ -125,6 +96,21 @@ Plot3DDialog::Plot3DDialog( QWidget* parent,  Qt::WFlags fl )
 	connect( btnTable, SIGNAL( clicked() ), this, SLOT(worksheet() ) );
 }
 
+void Plot3DDialog::initPrintPage()
+{
+	printPage = new QWidget();
+	QVBoxLayout *printLayout = new QVBoxLayout( printPage );
+
+	boxScaleOnPrint = new QCheckBox(tr("Scale to paper si&ze"));
+	printLayout->addWidget( boxScaleOnPrint );
+
+	boxPrintCropmarks = new QCheckBox(tr("Print Crop&marks"));
+	printLayout->addWidget( boxPrintCropmarks );
+	printLayout->addStretch();
+
+	generalDialog->addTab(printPage, tr( "&Print" ) );
+}
+
 void Plot3DDialog::initScalesPage()
 {
 	axesList = new QListWidget();
@@ -136,49 +122,69 @@ void Plot3DDialog::initScalesPage()
 
 	ApplicationWindow *app = (ApplicationWindow *)parent();
 
-    QGridLayout *gl1 = new QGridLayout();
-    gl1->addWidget(new QLabel(tr("From")), 0, 0);
+	QGridLayout *gl1 = new QGridLayout();
+	gl1->addWidget(new QLabel(tr("From")), 0, 0);
 	boxFrom = new DoubleSpinBox();
 	boxFrom->setLocale(app->locale());
-    boxFrom->setDecimals(app->d_decimal_digits);
-    gl1->addWidget(boxFrom, 0, 1);
+	boxFrom->setDecimals(app->d_decimal_digits);
+	gl1->addWidget(boxFrom, 0, 1);
 
-    gl1->addWidget(new QLabel(tr("To")), 1, 0);
+	gl1->addWidget(new QLabel(tr("To")), 1, 0);
 	boxTo = new DoubleSpinBox();
 	boxTo->setLocale(app->locale());
-    boxTo->setDecimals(app->d_decimal_digits);
-    gl1->addWidget(boxTo, 1, 1);
+	boxTo->setDecimals(app->d_decimal_digits);
+	gl1->addWidget(boxTo, 1, 1);
 
-    gl1->addWidget(new QLabel(tr("Type")), 2, 0);
+	gl1->addWidget(new QLabel(tr("Type")), 2, 0);
 	boxType=new QComboBox();
 	boxType->addItem(tr("linear"));
 	boxType->addItem(tr("logarithmic"));
 	boxType->setMaximumWidth(150);
-    gl1->addWidget(boxType, 2, 1);
-    gl1->setRowStretch(3, 1);
+	gl1->addWidget(boxType, 2, 1);
+	gl1->setRowStretch(3, 1);
 
-    QGroupBox *gb1 = new QGroupBox();
-    gb1->setLayout(gl1);
+	QGroupBox *gb1 = new QGroupBox();
+	gb1->setLayout(gl1);
 
-    QGridLayout *gl2 = new QGridLayout();
-    gl2->addWidget(new QLabel(tr("Major Ticks")), 0, 0);
+	QGridLayout *gl2 = new QGridLayout();
+	gl2->addWidget(new QLabel(tr("Major Ticks")), 0, 0);
 	boxMajors = new QSpinBox();
-    gl2->addWidget(boxMajors, 0, 1);
-    gl2->addWidget(new QLabel(tr("Minor Ticks")), 1, 0);
+	gl2->addWidget(boxMajors, 0, 1);
+	gl2->addWidget(new QLabel(tr("Minor Ticks")), 1, 0);
 	boxMinors = new QSpinBox();
-    gl2->addWidget(boxMinors, 1, 1);
-    gl2->setRowStretch(2, 1);
+	gl2->addWidget(boxMinors, 1, 1);
+	gl2->setRowStretch(2, 1);
 
-    TicksGroupBox = new QGroupBox();
-    TicksGroupBox->setLayout(gl2);
+	TicksGroupBox = new QGroupBox();
+	TicksGroupBox->setLayout(gl2);
+
+	QGroupBox *tickLabelsGroupBox = new QGroupBox(tr("Tick Labels"));
+	QGridLayout *gl3 = new QGridLayout(tickLabelsGroupBox);
+	gl3->addWidget(new QLabel(tr("Format")), 0, 0);
+	boxTickLabelsFormat = new QComboBox();
+	boxTickLabelsFormat->insertItem(tr("Automatic"));
+	boxTickLabelsFormat->insertItem(tr("Decimal: 10000.0"));
+	boxTickLabelsFormat->insertItem(tr("Scientific: 1e4"));
+	boxTickLabelsFormat->insertItem(tr("Engineering: 10k"));
+
+	gl3->addWidget(boxTickLabelsFormat, 0, 1);
+	gl3->addWidget(new QLabel(tr("Precision")), 1, 0);
+	boxPrecision = new QSpinBox();
+	boxPrecision->setMaximum(14);
+	gl3->addWidget(boxPrecision, 1, 1);
+	gl3->setRowStretch(2, 1);
+
+	QVBoxLayout* vb = new QVBoxLayout();
+	vb->addWidget(TicksGroupBox);
+	vb->addWidget(tickLabelsGroupBox);
 
 	QHBoxLayout* hb = new QHBoxLayout();
 	hb->addWidget(axesList);
 	hb->addWidget(gb1, 1);
-    hb->addWidget(TicksGroupBox, 1);
+	hb->addLayout(vb, 1);
 
-    scale = new QWidget();
-    scale->setLayout(hb);
+	scale = new QWidget();
+	scale->setLayout(hb);
 	generalDialog->insertTab(scale, tr( "&Scale" ) );
 }
 
@@ -293,7 +299,7 @@ void Plot3DDialog::initColorsPage()
 	QGridLayout* layout = new QGridLayout(colorMapFileGroupBox);
 
 	btnColorMap = new QPushButton();
-	btnColorMap->setIcon(QIcon(QPixmap(choose_folder_xpm)));
+	btnColorMap->setIcon(QIcon(":/folder_open.png"));
 	layout->addWidget(btnColorMap, 0, 0);
 
 	colorMapPreviewLabel = new QLabel(tr("None"));
@@ -379,7 +385,7 @@ void Plot3DDialog::initGeneralPage()
 	boxMeshLineWidth = new QDoubleSpinBox();
 	boxMeshLineWidth->setDecimals(1);
 	boxMeshLineWidth->setSingleStep(0.1);
-    boxMeshLineWidth->setRange(1, 100);
+	boxMeshLineWidth->setRange(0, 100);
     gl1->addWidget(boxMeshLineWidth, 1, 1);
 
     gl1->addWidget(new QLabel( tr( "Resolution" )), 2, 0);
@@ -697,6 +703,7 @@ void Plot3DDialog::setPlot(Graph3D *g)
 	zAxisFont = g->zAxisLabelFont();
 
 	viewScaleLimits(0);
+	viewAxisOptions(0);
 
 	boxDistance->setValue(g->labelsDistance());
 	numbersFont = g->numbersFont();
@@ -757,6 +764,9 @@ void Plot3DDialog::setPlot(Graph3D *g)
 		else if (g->matrix())
 			btnTable->setText(tr("&Matrix"));
 
+	boxPrintCropmarks->setChecked(g->printCropmarksEnabled());
+	boxScaleOnPrint->setChecked(g->scaleOnPrint());
+
 	initConnections();
 }
 
@@ -767,6 +777,8 @@ void Plot3DDialog::initConnections()
     connect(boxTo, SIGNAL(valueChanged(double)), this, SLOT(updatePlot()));
 	connect(boxMajors, SIGNAL(valueChanged(int)), this, SLOT(updatePlot()));
 	connect(boxMinors, SIGNAL(valueChanged(int)), this, SLOT(updatePlot()));
+	connect(boxPrecision, SIGNAL(valueChanged(int)), this, SLOT(updatePlot()));
+	connect(boxTickLabelsFormat, SIGNAL(activated(int)), this, SLOT(updatePlot()));
 	connect(axesList, SIGNAL(currentRowChanged(int)), this, SLOT(viewScaleLimits(int)));
 
 	// axes page connections
@@ -822,6 +834,7 @@ void Plot3DDialog::showBarsTab()
 	boxBarsRad->setLocale(app->locale());
     boxBarsRad->setDecimals(app->d_decimal_digits);
 	boxBarsRad->setValue(d_plot->barsRadius());
+	boxBarsRad->setMinimum(0.0);
 	hb->addWidget(boxBarsRad, 0, 1);
 
 	boxBarLines = new QCheckBox(tr("Draw lines"));
@@ -946,6 +959,13 @@ void Plot3DDialog::viewAxisOptions(int axis)
 	boxMinorLength->blockSignals(false);
 
 	axesList->setCurrentRow(axis);
+
+	boxTickLabelsFormat->blockSignals(true);
+	boxTickLabelsFormat->setCurrentIndex(d_plot->axisNumericFormat(axis));
+	boxTickLabelsFormat->blockSignals(false);
+	boxPrecision->blockSignals(true);
+	boxPrecision->setValue(d_plot->axisNumericPrecision(axis));
+	boxPrecision->blockSignals(false);
 }
 
 void Plot3DDialog::viewScaleLimits(int axis)
@@ -1061,9 +1081,13 @@ bool Plot3DDialog::updatePlot()
 		d_plot->setNumbersFont(numbersFont);
 		d_plot->setZoom(zoom*boxZoom->value()*0.01);
 		d_plot->setScale(xScale*boxXScale->value()*0.01, yScale*boxYScale->value()*0.01, zScale*boxZScale->value()*0.01);
-	} else if (generalDialog->currentPage()==(QWidget*)scale){
-		d_plot->setScale(axesList->currentRow(), boxFrom->value(), boxTo->value(), boxMajors->value(),
+	} else if (generalDialog->currentPage() == (QWidget*)scale){
+		double start = qMin(boxFrom->value(), boxTo->value());
+		double end = qMax(boxFrom->value(), boxTo->value());
+		d_plot->setScale(axesList->currentRow(), start, end, boxMajors->value(),
 						 boxMinors->value(), (Qwt3D::SCALETYPE)boxType->currentIndex());
+
+		d_plot->setAxisNumericFormat(axesList->currentRow(), boxTickLabelsFormat->currentIndex(), boxPrecision->value());
 	} else if (generalDialog->currentPage() == axes){
 		int axis = axesList2->currentRow();
 		labels[axis] = boxLabel->text();
@@ -1086,6 +1110,9 @@ bool Plot3DDialog::updatePlot()
 			d_plot->setZAxisTickLength(boxMajorLength->text().toDouble(), boxMinorLength->text().toDouble());
 		break;
 		}
+	} else if (generalDialog->currentPage() == printPage){
+		d_plot->printCropmarks(boxPrintCropmarks->isChecked());
+		d_plot->setScaleOnPrint(boxScaleOnPrint->isChecked());
 	}
 
     d_plot->update();
