@@ -32,9 +32,9 @@
 #include "Graph3D.h"
 #include <QCheckBox>
 
+class QGroupBox;
 class QComboBox;
 class QLabel;
-class QLineEdit;
 class QTextEdit;
 class QListWidget;
 class QPushButton;
@@ -47,6 +47,8 @@ class QStackedWidget;
 class QDoubleSpinBox;
 class ColorButton;
 class TextFormatButtons;
+class DoubleSpinBox;
+class ColorMapEditor;
 
 using namespace Qwt3D;
 
@@ -73,14 +75,8 @@ private slots:
 	void pickAxisLabelFont();
 	void pickNumbersFont();
 
-	QStringList scaleOptions(int axis, double start, double end,
-							const QString& majors, const QString& minors);
 	void viewScaleLimits(int axis);
 	void disableMeshOptions();
-	void showBarsTab(double rad);
-	void showPointsTab(double rad, bool smooth);
-	void showConesTab(double rad, int quality);
-	void showCrossHairTab(double rad, double linewidth, bool smooth, bool boxed);
 
 	void worksheet();
 
@@ -88,28 +84,43 @@ private slots:
 	void changeZoom(int);
 	void changeTransparency(int val);
     void pickDataColorMap();
+	void updateColorMapFileGroupBox(bool);
+	void updateLinearColorMapGroupBox(bool);
+	void enableMajorGrids(bool on);
+	void enableMinorGrids(bool on);
 
 private:
+	void initConnections();
     void initScalesPage();
 	void initAxesPage();
 	void initTitlePage();
+	void initGridPage();
 	void initColorsPage();
 	void initGeneralPage();
+	void initPrintPage();
+	void setColorMapPreview(const QString& fileName);
+
+	void showBarsTab();
+    void showPointsTab(double rad, bool smooth);
+	void showConesTab(double rad, int quality);
+	void showCrossHairTab(double rad, double linewidth, bool smooth, bool boxed);
 
     Graph3D *d_plot;
+	QGroupBox *linearColorMapGroupBox, *colorMapFileGroupBox;
+	QLabel *colorMapPreviewLabel;
 	QFont titleFont, xAxisFont,yAxisFont,zAxisFont, numbersFont;
-	QStringList labels, scales, tickLengths;
+	QStringList labels, tickLengths;
     QDoubleSpinBox *boxMeshLineWidth;
     QPushButton* buttonApply;
     QPushButton* buttonOk;
     QPushButton* buttonCancel;
 	QPushButton *btnTitleFont, *btnLabelFont;
     QPushButton *btnNumbersFont, *btnTable, *btnColorMap;
-	ColorButton *btnBackground, *btnMesh, *btnAxes, *btnTitleColor, *btnLabels;
-	ColorButton *btnFromColor, *btnToColor, *btnNumbers, *btnGrid;
+	ColorButton *btnBackground, *btnMesh, *btnAxes, *btnTitleColor, *btnLabels, *btnNumbers, *btnGrid, *btnGridMinor;
+	ColorMapEditor *d_color_map_editor;
     QTabWidget* generalDialog;
-	QWidget *scale, *colors, *general, *axes, *title, *bars, *points;
-	QLineEdit *boxFrom, *boxTo;
+	QWidget *scale, *colors, *general, *axes, *title, *bars, *points, *gridPage, *printPage;
+	DoubleSpinBox *boxFrom, *boxTo;
 	QTextEdit *boxTitle, *boxLabel;
 	QSpinBox *boxMajors, *boxMinors;
 	QGroupBox *TicksGroupBox, *AxesColorGroupBox;
@@ -117,13 +128,18 @@ private:
 	QCheckBox *boxLegend, *boxSmooth, *boxBoxed, *boxCrossSmooth, *boxOrthogonal;
 	QListWidget *axesList, *axesList2;
 	QComboBox *boxType, *boxPointStyle;
-	QLineEdit *boxMajorLength, *boxMinorLength, *boxConesRad;
-	QSpinBox *boxZoom, *boxXScale, *boxYScale, *boxZScale, *boxQuality;
-	QLineEdit *boxSize, *boxBarsRad, *boxCrossRad, *boxCrossLinewidth;
+	DoubleSpinBox *boxMajorLength, *boxMinorLength, *boxConesRad;
+	QSpinBox *boxZoom, *boxXScale, *boxYScale, *boxZScale, *boxQuality, *boxPrecision;
+	DoubleSpinBox *boxSize, *boxBarsRad, *boxCrossRad, *boxCrossLinewidth;
 	QStackedWidget *optionStack;
 	QWidget *dotsPage, *conesPage, *crossPage;
+	QCheckBox *boxBarLines, *boxFilledBars, *boxScaleOnPrint, *boxPrintCropmarks;
 	TextFormatButtons *titleFormatButtons, *axisTitleFormatButtons;
     double zoom, xScale, yScale, zScale;
+	QString d_color_map_file;
+	DoubleSpinBox *boxMajorGridWidth, *boxMinorGridWidth;
+	QComboBox *boxMajorGridStyle, *boxMinorGridStyle, *boxTickLabelsFormat;
+	QCheckBox *boxMajorGrids, *boxMinorGrids;
 };
 
 #endif
