@@ -2,7 +2,8 @@
     File                 : Table.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Knut Franke
+	Copyright            : (C) 2004 - 2010 by Ion Vasilief
+						   (C) 2006 - june 2007 byKnut Franke
     Email (use @ for *)  : ion_vasilief*yahoo.fr, knut.franke*gmx.de
     Description          : Table worksheet class
 
@@ -31,7 +32,6 @@
 #include <ImportASCIIDialog.h>
 #include <muParserScript.h>
 #include <ApplicationWindow.h>
-#include <pixmaps.h>
 
 #include <QMessageBox>
 #include <QDateTime>
@@ -60,6 +60,11 @@
 #include <gsl/gsl_sort_vector.h>
 #include <gsl/gsl_heapsort.h>
 
+#ifdef XLS_IMPORT
+    #include <ExcelFormat.h>
+    using namespace ExcelFormat;
+#endif
+
 Table::Table(ScriptingEnv *env, int r, int c, const QString& label, ApplicationWindow* parent, const QString& name, Qt::WFlags f)
 : MdiSubWindow(label,parent,name,f), scripted(env)
 {
@@ -79,10 +84,8 @@ void Table::init(int rows, int cols)
 	d_table->setColumnMovingEnabled(true);
 	d_table->setCurrentCell(-1, -1);
 
-	connect(d_table->verticalHeader(), SIGNAL(indexChange(int, int, int)),
-			this, SLOT(notifyChanges()));
-    connect(d_table->horizontalHeader(), SIGNAL(indexChange(int, int, int)),
-			this, SLOT(moveColumn(int, int, int)));
+	connect(d_table->verticalHeader(), SIGNAL(indexChange(int, int, int)), this, SLOT(notifyChanges()));
+	connect(d_table->horizontalHeader(), SIGNAL(indexChange(int, int, int)), this, SLOT(moveColumn(int, int, int)));
 
 	setFocusPolicy(Qt::StrongFocus);
 	setFocus();
@@ -330,31 +333,38 @@ void Table::cellEdited(int row, int col)
 int Table::colX(int col)
 {
 	int i, xcol = -1;
-	for(i=col-1; i>=0; i--)
-	{
+	for(i = col - 1; i >= 0; i--){
 		if (col_plot_type[i] == X)
 			return i;
 	}
-	for(i=col+1; i<(int)d_table->numCols(); i++)
-	{
+	for(i = col + 1; i < (int)d_table->numCols(); i++){
 		if (col_plot_type[i] == X)
 			return i;
 	}
 	return xcol;
 }
 
-int Table::colY(int col)
+int Table::colY(int col, int xCol)
 {
-	int i, yCol = -1;
-	for(i=col-1; i>=0; i--)
-	{
-		if (col_plot_type[i] == Y)
-			return i;
-	}
-	for(i=col+1; i<(int)d_table->numCols(); i++)
-	{
-		if (col_plot_type[i] == Y)
-			return i;
+	int yCol = -1;
+	if (xCol >= 0){
+		for(int i = col - 1; i >= 0; i--){
+			if (col_plot_type[i] == Y && colX(i) == xCol)
+				return i;
+		}
+		for(int i = col + 1; i < (int)d_table->numCols(); i++){
+			if (col_plot_type[i] == Y && colX(i) == xCol)
+				return i;
+		}
+	} else {
+		for(int i = col - 1; i >= 0; i--){
+			if (col_plot_type[i] == Y)
+				return i;
+		}
+		for(int i = col + 1; i < (int)d_table->numCols(); i++){
+			if (col_plot_type[i] == Y)
+				return i;
+		}
 	}
 	return yCol;
 }
@@ -501,7 +511,7 @@ bool Table::calculate()
 {
 	Q3TableSelection sel = getSelection();
 	bool success = true;
-	for (int col=sel.leftCol(); col<=sel.rightCol(); col++)
+	for (int col = sel.leftCol(); col <= sel.rightCol(); col++)
 		if (!calculate(col, sel.topRow(), sel.bottomRow()))
 			success = false;
 	return success;
@@ -644,6 +654,73 @@ bool Table::calculate(int col, int startRow, int endRow, bool forceMuParser, boo
 	return true;
 }
 
+Table* Table::extractData(const QString& name, const QString& condition, int startRow, int endRow)
+{
+	if (startRow < 0)
+		startRow = 0;
+
+	if (endRow < 0 || endRow >= numRows())
+		endRow = numRows();
+
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	muParserScript *mup = new muParserScript(scriptEnv, condition, this,  QString("<%1>").arg(this->objectName()));
+	double *r = mup->defineVariable("i");
+	mup->defineVariable("sr", startRow + 1.0);
+	mup->defineVariable("er", endRow + 1.0);
+
+	if (!mup->compile()){
+		QApplication::restoreOverrideCursor();
+		return 0;
+	}
+
+	ApplicationWindow *app = applicationWindow();
+	if (!app)
+		return 0;
+
+	int cols = d_table->numCols();
+	Table *dest = app->table(name);
+	if (dest){
+		dest->setNumCols(cols);
+		dest->setNumRows(numRows());
+	} else
+		dest = app->newTable(numRows(), cols, name);
+
+	if (!dest)
+		return 0;
+
+	int aux = 0;
+	if (mup->codeLines() == 1){
+		for (int i = startRow; i <= endRow; i++){
+			*r = i + 1.0;
+			if (mup->evalSingleLine()){
+				for (int j = 0; j < cols; j++)
+					dest->setText(aux, j, this->text(i, j));
+
+				aux++;
+			}
+		}
+	} else {
+		QVariant ret;
+		for (int i = startRow; i <= endRow; i++){
+			*r = i + 1.0;
+			ret = mup->eval();
+			if (ret.type() == QVariant::Double && ret.toDouble()){
+				for (int j = 0; j < cols; j++)
+					dest->setText(aux, j, this->text(i, j));
+
+				aux++;
+			}
+		}
+	}
+
+	if (aux)
+		dest->setNumRows(aux);
+	dest->copy(this, false);
+	QApplication::restoreOverrideCursor();
+	return dest;
+}
+
 void Table::updateValues(Table* t, const QString& columnName)
 {
     if (!t || t != this)
@@ -891,7 +968,7 @@ QString Table::colName(int col)
 	return QString(this->objectName())+"_"+col_label[col];
 }
 
-void Table::setColName(int col, const QString& text, bool enumerateRight)
+void Table::setColName(int col, const QString& text, bool enumerateRight, bool warn)
 {
     if (text.isEmpty() || col<0 || col >= d_table->numCols())
 		return;
@@ -911,10 +988,12 @@ void Table::setColName(int col, const QString& text, bool enumerateRight)
 		if (enumerateRight)
             newLabel += QString::number(n);
 
-        if (col_label.contains(newLabel) > 0){
-            QMessageBox::critical(0,tr("QtiPlot - Error"),
-            tr("There is already a column called : <b>"+newLabel+"</b> in table <b>"+caption+"</b>!<p>Please choose another name!"));
-            return;
+		if (col_label.contains(newLabel) > 0){
+			if (warn){
+				QMessageBox::critical(0, tr("QtiPlot - Error"),
+				tr("There is already a column called : <b>"+newLabel+"</b> in table <b>"+caption+"</b>!<p>Please choose another name!"));
+			}
+			return;
         }
 		n++;
 	}
@@ -1380,7 +1459,7 @@ void Table::copySelection()
 	}
 
 	// Copy text into the clipboard
-	QApplication::clipboard()->setText(text);
+	QApplication::clipboard()->setText(text.trimmed());
 }
 
 // Paste text from the clipboard
@@ -1468,19 +1547,39 @@ void Table::pasteSelection()
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
+	QStringList colLabels;
+	for (int i = 0; i < left & i < col_label.size(); i++)
+		colLabels << col_label[i];
+	for (int i = left + cols; i < col_label.size(); i++)
+		colLabels << col_label[i];
+
 	if (pasteComments || pasteHeader){
 		for (int j = left; j < left + cols; j++){
-			if (d_table->isColumnReadOnly(j) || firstLine.size() <= j)
+			if (d_table->isColumnReadOnly(j))
+				continue;
+
+			int colIndex = j - left;
+			if (colIndex < 0 || colIndex >= firstLine.count())
 				continue;
 
 			if (pasteComments)
-				comments[j] = firstLine[j];
-			else if (pasteHeader)
-				col_label[j] = firstLine[j].replace("-","_").remove(QRegExp("\\W")).replace("_","-");
+				comments[j] = firstLine[colIndex];
+			else if (pasteHeader){
+				QString colName = firstLine[colIndex].replace("-", "_").remove(QRegExp("\\W")).replace("_", "-");
+				while(colLabels.contains(colName))
+					colName += "2";
+				if (!colLabels.contains(colName)){
+					col_label[j] = colName;
+					colLabels << colName;
+				}
+			}
 		}
-		if (pasteComments)
-			showComments();
-		else if (pasteHeader)
+		if (pasteComments){
+			if (d_show_comments)
+				setHeaderColType();
+			else
+				showComments();
+		} else if (pasteHeader)
 			setHeaderColType();
 	}
 
@@ -1564,6 +1663,7 @@ void Table::removeCol(const QStringList& list)
 
 			d_table->removeColumn(id);
 			emit removedCol(name);
+			emit removedCol(id);
 		}
 	}
 	emit modifiedWindow(this);
@@ -1723,6 +1823,8 @@ void Table::sortColumns(const QStringList&s, int type, int order, const QString&
 		else
 			gsl_sort_index(p, data_double.data(), 1, non_empty_cells);
 
+		blockSignals(true);
+
 		for(int i = 0; i < cols; i++){// Since we have the permutation index, sort all the columns
             int col = colIndex(s[i]);
             if (d_table->isColumnReadOnly(col))
@@ -1750,10 +1852,19 @@ void Table::sortColumns(const QStringList&s, int type, int order, const QString&
                     for (int j=0; j<non_empty_cells; j++)
                         d_table->setText(valid_cell[j], col, locale().toString(data_double[p[non_empty_cells-j-1]], f, prec));
             }
-            emit modifiedData(this, colName(col));
         }
         delete[] p;
     }
+
+	blockSignals(false);
+
+	for(int i = 0; i < cols; i++){// notify changes
+		int col = colIndex(s[i]);
+		if (d_table->isColumnReadOnly(col))
+			continue;
+
+		emit modifiedData(this, colName(col));
+	}
 	emit modifiedWindow(this);
 }
 
@@ -1793,6 +1904,8 @@ void Table::sortColumn(int col, int order)
         gsl_sort(r.data(), 1, non_empty_cells);
     }
 
+	blockSignals(true);
+
     if (columnType(col) == Table::Text){
         if (!order){
             for (int i=0; i<non_empty_cells; i++)
@@ -1813,18 +1926,21 @@ void Table::sortColumn(int col, int order)
                 d_table->setText(valid_cell[i], col, locale().toString(r[non_empty_cells-i-1], f, prec));
         }
     }
+
+	blockSignals(false);
+
 	emit modifiedData(this, colName(col));
 	emit modifiedWindow(this);
 }
 
 void Table::sortColAsc()
 {
-sortColumn(d_table->currentColumn ());
+	sortColumn(d_table->currentColumn ());
 }
 
 void Table::sortColDesc()
 {
-sortColumn(d_table->currentColumn(), 1);
+	sortColumn(d_table->currentColumn(), 1);
 }
 
 int Table::numRows()
@@ -1850,6 +1966,9 @@ bool Table::isEmptyRow(int row)
 
 bool Table::isEmptyColumn(int col)
 {
+	if (col < 0 || col >= d_table->numCols())
+		return true;
+
 	for (int i=0; i<d_table->numRows(); i++)
 	{
 		QString text=d_table->text(i,col);
@@ -2006,28 +2125,28 @@ bool Table::setDateFormat(const QString& format, int col, bool updateCells)
 		return true;
 
 	bool first_time = false;
-    if (updateCells){
-	for (int i=0; i<d_table->numRows(); i++){
-		QString s = d_table->text(i,col);
-		if (!s.isEmpty()){
-		    QDateTime d = QDateTime::fromString (s, format);
-			if (colTypes[col] != Date && d.isValid()){
-			//This might be the first time the user assigns a date format.
-            //If Qt understands the format we break the loop, assign it to the column and return true!
-				first_time = true;
-                break;
-			}
+	if (updateCells){
+		for (int i=0; i<d_table->numRows(); i++){
+			QString s = d_table->text(i,col);
+			if (!s.isEmpty()){
+				QDateTime d = QDateTime::fromString (s, format);
+				if (colTypes[col] != Date && d.isValid()){
+				//This might be the first time the user assigns a date format.
+				//If Qt understands the format we break the loop, assign it to the column and return true!
+					first_time = true;
+					break;
+				}
 
-		    if (d_saved_cells){
-                d = QDateTime(QDate::fromJulianDay((int)d_saved_cells[col][i] + 1));
-                double msecs = (d_saved_cells[col][i] - floor(d_saved_cells[col][i]))*864e5;
-                d.setTime(d.time().addMSecs(qRound(msecs)));
-				if (d.isValid())
-					d_table->setText(i, col, d.toString(format));
-		    }
+				if (d_saved_cells){
+					d = QDateTime(QDate::fromJulianDay((int)d_saved_cells[col][i] + 1));
+					double msecs = (d_saved_cells[col][i] - floor(d_saved_cells[col][i]))*864e5;
+					d.setTime(d.time().addMSecs(qRound(msecs)));
+					if (d.isValid())
+						d_table->setText(i, col, d.toString(format));
+				}
+			}
 		}
 	}
-    }
 	colTypes[col] = Date;
 	col_format[col] = format;
 	QTime ref = QTime(0, 0);
@@ -2366,7 +2485,8 @@ bool Table::noYColumn()
 
 void Table::importASCII(const QString &fname, const QString &sep, int ignoredLines, bool renameCols,
     bool stripSpaces, bool simplifySpaces, bool importComments, const QString& commentString,
-    bool readOnly, ImportMode importAs, int endLine, int maxRows)
+	bool readOnly, ImportMode importAs, const QLocale& importLocale, int endLine, int maxRows,
+	const QList<int>& newColTypes, const QStringList& colFormats)
 {
 	int rows;
 	QString name = MdiSubWindow::parseAsciiFile(fname, commentString, endLine, ignoredLines, maxRows, rows);
@@ -2379,6 +2499,9 @@ void Table::importASCII(const QString &fname, const QString &sep, int ignoredLin
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
+	QLocale locale = this->locale();
+	bool updateDecimalSeparators = (importLocale != locale) ? true : false;
+
 	QTextStream t(&f);
 	QString s = t.readLine();//read first line
 	if (simplifySpaces)
@@ -2392,7 +2515,7 @@ void Table::importASCII(const QString &fname, const QString &sep, int ignoredLin
 	bool allNumbers = true;
 	for (int i=0; i<cols; i++)
 	{//verify if the strings in the line used to rename the columns are not all numbers
-		locale().toDouble(line[i], &allNumbers);
+		locale.toDouble(line[i], &allNumbers);
 		if (!allNumbers)
 			break;
 	}
@@ -2401,7 +2524,7 @@ void Table::importASCII(const QString &fname, const QString &sep, int ignoredLin
 	if (importComments)
 		rows--;
 
-	QStringList oldHeader;
+	QStringList oldHeader = col_label;
 	int startRow = 0, startCol = 0;
 	int c = d_table->numCols();
 	int r = d_table->numRows();
@@ -2410,7 +2533,6 @@ void Table::importASCII(const QString &fname, const QString &sep, int ignoredLin
 			if (d_table->numRows() != rows)
 				d_table->setNumRows(rows);
 
-			oldHeader = col_label;
 			if (c != cols){
 				if (c < cols)
 					addColumns(cols - c);
@@ -2442,6 +2564,30 @@ void Table::importASCII(const QString &fname, const QString &sep, int ignoredLin
 		break;
 	}
 
+	if (!newColTypes.isEmpty()){
+		int ncols = newColTypes.size();
+		int fSize = colFormats.size();
+		switch(importAs){
+			case NewColumns:
+				for (int i = c; i < c + cols && i < ncols && i < colTypes.size(); i++){
+					int type = newColTypes[i];
+					colTypes[i] = type;
+					if (type != Numeric && type != Text && i < fSize)
+						col_format[i] = colFormats[i];
+				}
+			break;
+			default:
+				for (int i = 0; i < cols && i < ncols; i++){
+					int type = newColTypes[i];
+					colTypes[i] = type;
+					if (type != Numeric && type != Text && i < fSize){
+						col_format[i] = colFormats[i];
+					}
+				}
+			break;
+		}
+	}
+
 	if (renameCols && !allNumbers){//use first line to set the table header
 		for (int i = 0; i<cols; i++){
 			int aux = i + startCol;
@@ -2478,8 +2624,17 @@ void Table::importASCII(const QString &fname, const QString &sep, int ignoredLin
 
 	if ((!renameCols || allNumbers) && !importComments && rows > 0){
 		//put values in the first line of the table
-		for (int i = 0; i<cols; i++)
-			d_table->setText(startRow, startCol + i, line[i]);
+		for (int i = 0; i < cols; i++){
+			bool ok;
+			double val = importLocale.toDouble(line[i], &ok);
+			if (colTypes[startCol + i] == Table::Numeric && (ok || updateDecimalSeparators)){
+				char format;
+				int prec;
+				columnNumericFormat(startCol + i, &format, &prec);
+				d_table->setText(startRow, startCol + i, locale.toString(val, format, prec));
+			} else
+				d_table->setText(startRow, startCol + i, line[i]);
+		}
 		startRow++;
 	}
 
@@ -2516,14 +2671,23 @@ void Table::importASCII(const QString &fname, const QString &sep, int ignoredLin
 			addColumns(lc - cols);
 			cols = lc;
 		}
-		for (int j=0; j<cols && j<lc; j++)
-			d_table->setText(row, startCol + j, line[j]);
+
+		for (int j = 0; j<cols && j<lc; j++){
+			bool ok;
+			double val = importLocale.toDouble(line[j], &ok);
+			if (colTypes[startCol + j] == Table::Numeric && (ok || updateDecimalSeparators)){
+				char format;
+				int prec;
+				columnNumericFormat(startCol + j, &format, &prec);
+				d_table->setText(row, startCol + j, locale.toString(val, format, prec));
+			} else
+				d_table->setText(row, startCol + j, line[j]);
+		}
 
 		l++;
 		row++;
 		if (l%100 == 0)
 			progress.setValue(l/100);
-		qApp->processEvents();
 	}
 
 	d_table->blockSignals(false);
@@ -2537,15 +2701,17 @@ void Table::importASCII(const QString &fname, const QString &sep, int ignoredLin
 	if (importAs == Overwrite || importAs == NewRows){
 		if (cols > c)
 			cols = c;
-		for (int i=0; i<cols; i++){
+
+		for (int i = 0; i < cols; i++){
 			emit modifiedData(this, colName(i));
 			if (colLabel(i) != oldHeader[i])
-				emit changedColHeader(QString(objectName()) + "_" + oldHeader[i], QString(objectName())+"_"+colLabel(i));
+				emit changedColHeader(QString(objectName()) + "_" + oldHeader[i], QString(objectName()) + "_" + colLabel(i));
 		}
 	}
 }
 
-bool Table::exportODF(const QString& fname, bool withLabels, bool exportComments, bool exportSelection)
+#ifdef XLS_IMPORT
+bool Table::exportExcel(const QString& fname, bool withLabels, bool exportComments, bool exportSelection)
 {
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
@@ -2554,7 +2720,6 @@ bool Table::exportODF(const QString& fname, bool withLabels, bool exportComments
 	int selectedCols = 0;
 	int topRow = 0, bottomRow = 0;
 	int *sCols = 0;
-	int r = rows;
 	if (exportSelection){
 		for (int i=0; i<cols; i++){
 			if (d_table->isColumnSelected(i))
@@ -2583,107 +2748,264 @@ bool Table::exportODF(const QString& fname, bool withLabels, bool exportComments
 				break;
 			}
 		}
-
-		r = abs(bottomRow - topRow) + 1;
 	}
 
 	int aux = selectedCols;
 	if (!selectedCols)
 		aux = cols;
 
-	QTextDocument *document = new QTextDocument();
-	QTextCursor cursor = QTextCursor(document);
-
-	QTextTableFormat tableFormat;
-	tableFormat.setAlignment(Qt::AlignCenter);
-	tableFormat.setCellPadding(0);
-	tableFormat.setHeaderRowCount(0);
-	tableFormat.setBorder (1);
-	tableFormat.setBorderStyle(QTextFrameFormat::BorderStyle_Solid);
-	tableFormat.setWidth(QTextLength(QTextLength::PercentageLength, 100));
+	aux = QMIN(aux, 256);
 
 	int headerRows = 0;
 	if (withLabels)
 		headerRows++;
 	if (exportComments)
 		headerRows++;
-	r += headerRows;
 
-	QTextTable *table = cursor.insertTable(r, aux, tableFormat);
-	QTextCharFormat cellFormat;
-	cellFormat.setBackground (QBrush(Qt::gray));
+	BasicExcel xls;
+	xls.New(1);
+	BasicExcelWorksheet* sheet = xls.GetWorksheet((size_t)0);
+
+	XLSFormatManager fmt_mgr(xls);
+	ExcelFont font_bold;
+	font_bold._weight = 700;
+	CellFormat fmt_bold(fmt_mgr, font_bold);
 	for (int i = 0; i < aux; i++){
 		for (int j = 0; j < headerRows; j++){
-			QTextTableCell cell = table->cellAt(j, i);
-			cell.setFormat(cellFormat);
+			BasicExcelCell* cell = sheet->Cell(j, i);
+			cell->SetFormat(fmt_bold);
 		}
 	}
 
+	int startRow = 0;
 	if (withLabels){
+		startRow++;
+
 		QStringList header = colNames();
 		QStringList ls = header.grep ( QRegExp ("\\D"));
-		if (exportSelection){
+		if (exportSelection && selectedCols){
 			for (int i = 0; i < aux; i++){
+				BasicExcelCell* cell = sheet->Cell(0, i);
 				if (ls.count()>0)
-					cursor.insertText(header[sCols[i]]);
+					cell->Set(header[sCols[i]].toStdString().c_str());
 				else
-					cursor.insertText("C" + header[sCols[i]]);
-				cursor.movePosition(QTextCursor::NextCell);
+					cell->Set(("C" + header[sCols[i]]).toStdString().c_str());
 			}
 		} else {
 			if (ls.count() > 0){
 				for (int j = 0; j < aux; j++){
-					cursor.insertText(header[j]);
-					cursor.movePosition(QTextCursor::NextCell);
+					BasicExcelCell* cell = sheet->Cell(0, j);
+					cell->Set(header[j].toStdString().c_str());
 				}
 			} else {
 				for (int j = 0; j < aux; j++){
-					cursor.insertText("C" + header[j]);
-					cursor.movePosition(QTextCursor::NextCell);
+					BasicExcelCell* cell = sheet->Cell(0, j);
+					cell->Set(("C" + header[j]).toStdString().c_str());
 				}
 			}
 		}
 	}// finished writting labels
 
 	if (exportComments){
-		if (exportSelection){
+		if (exportSelection && selectedCols){
 			for (int i = 0; i < aux; i++){
-                cursor.insertText(comments[sCols[i]]);
-                cursor.movePosition(QTextCursor::NextCell);
+				QString s = comments[sCols[i]];
+				if (s.isEmpty())
+					continue;
+				sheet->Cell(startRow, i)->Set(s.toStdString().c_str());
 			}
 		} else {
-            for (int i = 0; i < aux; i++){
-                cursor.insertText(comments[i]);
-				cursor.movePosition(QTextCursor::NextCell);
-            }
-        }
+			for (int i = 0; i < aux; i++){
+				QString s = comments[i];
+				if (s.isEmpty())
+					continue;
+				sheet->Cell(startRow, i)->Set(s.toStdString().c_str());
+			}
+		}
+		startRow++;
 	}
 
-	if (exportSelection){
+	if (exportSelection && selectedCols){
 		for (int i = topRow; i <= bottomRow; i++){
 			for (int j = 0; j < aux; j++){
-				cursor.insertText(d_table->text(i, sCols[j]));
-				cursor.movePosition(QTextCursor::NextCell);
+				int col = sCols[j];
+				QString s = d_table->text(i, col);
+				if (s.isEmpty())
+					continue;
+				if (columnType(col) == Numeric)
+					sheet->Cell(startRow, j)->Set(cell(i, col));
+				else
+					sheet->Cell(startRow, j)->Set(s.toStdString().c_str());
 			}
+			startRow++;
+			if (startRow > 65536)
+				break;
 		}
 		delete [] sCols;
 	} else {
-		for (int i = 0; i < rows; i++) {
+		for (int i = 0; i < rows; i++){
 			for (int j = 0; j < aux; j++){
-				cursor.insertText(d_table->text(i, j));
-				cursor.movePosition(QTextCursor::NextCell);
+				QString s = d_table->text(i, j);
+				if (s.isEmpty())
+					continue;
+				if (columnType(j) == Numeric)
+					sheet->Cell(startRow, j)->Set(cell(i, j));
+				else
+					sheet->Cell(startRow, j)->Set(s.toStdString().c_str());
 			}
+			startRow++;
+			if (startRow > 65536)
+				break;
 		}
 	}
 
-	QTextDocumentWriter writer(fname);
-	if (fname.endsWith(".html"))
-		writer.setFormat("HTML");
-	writer.write(document);
+	xls.SaveAs(fname);
 
 	QApplication::restoreOverrideCursor();
 	return true;
 }
+#endif
+
+bool Table::exportODF(const QString& fname, bool withLabels, bool exportComments, bool exportSelection)
+{
+        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+        int rows = d_table->numRows();
+        int cols = d_table->numCols();
+        int selectedCols = 0;
+        int topRow = 0, bottomRow = 0;
+        int *sCols = 0;
+        int r = rows;
+        if (exportSelection){
+                for (int i=0; i<cols; i++){
+                        if (d_table->isColumnSelected(i))
+                                selectedCols++;
+                }
+
+                sCols = new int[selectedCols];
+                int aux = 0;
+                for (int i=0; i<cols; i++){
+                        if (d_table->isColumnSelected(i)){
+                                sCols[aux] = i;
+                                aux++;
+                        }
+                }
+
+                for (int i=0; i<rows; i++){
+                        if (d_table->isRowSelected(i)){
+                                topRow = i;
+                                break;
+                        }
+                }
+
+                for (int i = rows - 1; i > 0; i--){
+                        if (d_table->isRowSelected(i)){
+                                bottomRow = i;
+                                break;
+                        }
+                }
+
+                r = abs(bottomRow - topRow) + 1;
+        }
+
+        int aux = selectedCols;
+        if (!selectedCols)
+                aux = cols;
+
+        QTextDocument *document = new QTextDocument();
+        QTextCursor cursor = QTextCursor(document);
+
+        QTextTableFormat tableFormat;
+        tableFormat.setAlignment(Qt::AlignCenter);
+        tableFormat.setCellPadding(0);
+        tableFormat.setHeaderRowCount(0);
+        tableFormat.setBorder (1);
+        tableFormat.setBorderStyle(QTextFrameFormat::BorderStyle_Solid);
+        tableFormat.setWidth(QTextLength(QTextLength::PercentageLength, 100));
+
+        int headerRows = 0;
+        if (withLabels)
+                headerRows++;
+        if (exportComments)
+                headerRows++;
+        r += headerRows;
+
+        QTextTable *table = cursor.insertTable(r, aux, tableFormat);
+        QTextCharFormat cellFormat;
+        cellFormat.setBackground (QBrush(Qt::gray));
+        for (int i = 0; i < aux; i++){
+                for (int j = 0; j < headerRows; j++){
+                        QTextTableCell cell = table->cellAt(j, i);
+                        cell.setFormat(cellFormat);
+                }
+        }
+
+        if (withLabels){
+                QStringList header = colNames();
+                QStringList ls = header.grep ( QRegExp ("\\D"));
+                if (exportSelection && selectedCols){
+                        for (int i = 0; i < aux; i++){
+                                if (ls.count()>0)
+                                        cursor.insertText(header[sCols[i]]);
+                                else
+                                        cursor.insertText("C" + header[sCols[i]]);
+                                cursor.movePosition(QTextCursor::NextCell);
+                        }
+                } else {
+                        if (ls.count() > 0){
+                                for (int j = 0; j < aux; j++){
+                                        cursor.insertText(header[j]);
+                                        cursor.movePosition(QTextCursor::NextCell);
+                                }
+                        } else {
+                                for (int j = 0; j < aux; j++){
+                                        cursor.insertText("C" + header[j]);
+                                        cursor.movePosition(QTextCursor::NextCell);
+                                }
+                        }
+                }
+        }// finished writting labels
+
+        if (exportComments){
+                if (exportSelection && selectedCols){
+                        for (int i = 0; i < aux; i++){
+                            cursor.insertText(comments[sCols[i]]);
+                            cursor.movePosition(QTextCursor::NextCell);
+                        }
+                } else {
+                    for (int i = 0; i < aux; i++){
+                        cursor.insertText(comments[i]);
+                        cursor.movePosition(QTextCursor::NextCell);
+                    }
+        }
+        }
+
+        if (exportSelection && selectedCols){
+                for (int i = topRow; i <= bottomRow; i++){
+                        for (int j = 0; j < aux; j++){
+                                cursor.insertText(d_table->text(i, sCols[j]));
+                                cursor.movePosition(QTextCursor::NextCell);
+                        }
+                }
+                delete [] sCols;
+        } else {
+                for (int i = 0; i < rows; i++) {
+                        for (int j = 0; j < aux; j++){
+                                cursor.insertText(d_table->text(i, j));
+                                cursor.movePosition(QTextCursor::NextCell);
+                        }
+                }
+        }
+
+        QTextDocumentWriter writer(fname);
+        if (fname.endsWith(".html"))
+                writer.setFormat("HTML");
+        writer.write(document);
+
+        QApplication::restoreOverrideCursor();
+        return true;
+}
+
 
 bool Table::exportASCII(const QString& fname, const QString& separator,
 		bool withLabels, bool exportComments, bool exportSelection)
@@ -2699,6 +3021,12 @@ bool Table::exportASCII(const QString& fname, const QString& separator,
 		f.close();
 		return exportODF(fname, withLabels, exportComments, exportSelection);
 	}
+#ifdef XLS_IMPORT
+	else if (fname.endsWith(".xls")){
+		f.close();
+		return exportExcel(fname, withLabels, exportComments, exportSelection);
+	}
+#endif
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
@@ -2760,7 +3088,7 @@ bool Table::exportASCII(const QString& fname, const QString& separator,
 	if (withLabels){
 		QStringList header = colNames();
 		QStringList ls = header.grep ( QRegExp ("\\D"));
-		if (exportSelection){
+                if (exportSelection && selectedCols){
 			for (int i = 0; i < aux; i++){
 				if (ls.count()>0)
 					t << header[sCols[i]] + sep;
@@ -2788,19 +3116,19 @@ bool Table::exportASCII(const QString& fname, const QString& separator,
 	}// finished writting labels
 
 	if (exportComments){
-		if (exportSelection){
+                if (exportSelection && selectedCols){
 			for (int i = 0; i < aux; i++)
-                t << comments[sCols[i]] + sep;
+                            t << comments[sCols[i]] + sep;
 			if (aux >= 0)
-				t << comments[sCols[aux]] + eol;
+                            t << comments[sCols[aux]] + eol;
 		} else {
-            for (int i = 0; i < aux; i++)
-                t << comments[i] + sep;
-            t << comments[aux] + eol;
-        }
+                    for (int i = 0; i < aux; i++)
+                        t << comments[i] + sep;
+                    t << comments[aux] + eol;
+                }
 	}
 
-	if (exportSelection){
+        if (exportSelection && selectedCols){
 		for (int i = topRow; i <= bottomRow; i++){
 			for (int j = 0; j < aux; j++)
 				t << d_table->text(i, sCols[j]) + sep;
@@ -2907,7 +3235,7 @@ bool Table::eventFilter(QObject *object, QEvent *e)
 				QMimeData *mimeData = new QMimeData;
 				mimeData->setText(selectedColumns().join("\n"));
 				drag->setMimeData(mimeData);
-				drag->setPixmap(QPixmap(drag_curves_xpm));
+				drag->setPixmap(QPixmap(":/drag_curves.png"));
 				drag->exec();
 				return true;
 			}
@@ -2951,7 +3279,7 @@ bool Table::eventFilter(QObject *object, QEvent *e)
 		QRect r = hheader->sectionRect(col);
 		r = QRect(r.topLeft(), QSize(r.width(), 10));
 		if (d_table->isColumnSelected(col, true) && r.contains(me->pos()))
-			setCursor(QCursor(QPixmap(append_drag_curves_xpm)));
+			setCursor(QCursor(QPixmap(":/append_drag_curves.png")));
 		else
 			setCursor(QCursor(Qt::ArrowCursor));
 		return false;
@@ -3072,13 +3400,16 @@ void Table::resizeCols(int c)
 	emit modifiedWindow(this);
 }
 
-void Table::copy(Table *m)
+void Table::copy(Table *m, bool values)
 {
 	int rows = d_table->numRows();
 	int cols = d_table->numCols();
-	for (int i=0; i<rows; i++){
-		for (int j=0; j<cols; j++)
-			d_table->setText(i, j, m->text(i, j));
+
+	if (values){
+		for (int i=0; i<rows; i++){
+			for (int j=0; j<cols; j++)
+				d_table->setText(i, j, m->text(i, j));
+		}
 	}
 
 	for (int i=0; i<cols; i++){
@@ -3149,6 +3480,12 @@ void Table::notifyChanges()
 	emit modifiedWindow(this);
 }
 
+void Table::notifyChanges(const QString& colName)
+{
+	emit modifiedData(this, colName);
+	emit modifiedWindow(this);
+}
+
 void Table::clear()
 {
 	for (int i=0; i<d_table->numCols(); i++)
@@ -3167,14 +3504,17 @@ void Table::goToRow(int row)
 	if( (row < 1) || (row > numRows()) ) return;
 
 	d_table->ensureCellVisible ( row-1, 0 );
+	d_table->clearSelection();
 	d_table->selectRow(row-1);
 }
 
 void Table::goToColumn(int col)
 {
-	if( (col < 1) || (col > numCols()) ) return;
+	if( (col < 1) || (col > numCols()) )
+		return;
 
 	d_table->ensureCellVisible (0, col - 1);
+	d_table->clearSelection();
 	d_table->selectColumn(col - 1);
 }
 
@@ -3291,6 +3631,7 @@ void Table::swapColumns(int col1, int col2)
     d_table->setColumnWidth(col2, width1);
     setHeaderColType();
 	emit modifiedWindow(this);
+	emit colIndexChanged(col1, col2);
 }
 
 void Table::moveColumnBy(int cols)
@@ -3319,6 +3660,7 @@ void Table::moveColumnBy(int cols)
 		colTypes.move(oldPos, newPos);
 		col_format.move(oldPos, newPos);
 		col_plot_type.move(oldPos, newPos);
+		emit colIndexChanged(oldPos, newPos);
 	} else
 		swapColumns(oldPos, newPos);
 
@@ -3383,6 +3725,114 @@ void Table::moveRow(bool up)
 	emit modifiedWindow(this);
 }
 
+double Table::sum(int col, int startRow, int endRow)
+{
+	if (col < 0 || col >= d_table->numCols())
+		return 0.0;
+
+	if (colTypes[col] != Numeric)
+		return 0.0;
+
+	int rows = d_table->numRows();
+	if (startRow < 0 || startRow >= rows)
+		startRow = 0;
+	if (endRow < 0 || endRow >= rows)
+		endRow = d_table->numRows() - 1;
+
+	QLocale l = locale();
+	double sum = 0.0;
+	for (int i = startRow; i <= endRow; i++){
+		QString s = d_table->text(i, col);
+		if (!s.isEmpty())
+			sum += l.toDouble(s);
+	}
+	return sum;
+}
+
+double Table::avg(int col, int startRow, int endRow)
+{
+	if (col < 0 || col >= d_table->numCols())
+		return 0.0;
+
+	if (colTypes[col] != Numeric)
+		return 0.0;
+
+	int rows = d_table->numRows();
+	if (startRow < 0 || startRow >= rows)
+		startRow = 0;
+	if (endRow < 0 || endRow >= rows)
+		endRow = d_table->numRows() - 1;
+
+	QLocale l = locale();
+	double sum = 0.0;
+	int count = 0;
+	for (int i = startRow; i <= endRow; i++){
+		QString s = d_table->text(i, col);
+		if (!s.isEmpty()){
+			sum += l.toDouble(s);
+			count++;
+		}
+	}
+
+	if (count)
+		return sum/(double)count;
+
+	return 0.0;
+}
+
+double Table::min(int col, int startRow, int endRow)
+{
+	if (col < 0 || col >= d_table->numCols())
+		return 0.0;
+
+	if (colTypes[col] != Numeric)
+		return 0.0;
+
+	int rows = d_table->numRows();
+	if (startRow < 0 || startRow >= rows)
+		startRow = 0;
+	if (endRow < 0 || endRow >= rows)
+		endRow = d_table->numRows() - 1;
+
+	QLocale l = locale();
+	double min = this->cell(startRow, col);
+	for (int i = startRow + 1; i <= endRow; i++){
+		QString s = d_table->text(i, col);
+		if (!s.isEmpty()){
+			double val = l.toDouble(s);
+			if (val < min)
+				min = val;
+		}
+	}
+	return min;
+}
+
+double Table::max(int col, int startRow, int endRow)
+{
+	if (col < 0 || col >= d_table->numCols())
+		return 0.0;
+
+	if (colTypes[col] != Numeric)
+		return 0.0;
+
+	int rows = d_table->numRows();
+	if (startRow < 0 || startRow >= rows)
+		startRow = 0;
+	if (endRow < 0 || endRow >= rows)
+		endRow = d_table->numRows() - 1;
+
+	QLocale l = locale();
+	double max = this->cell(startRow, col);
+	for (int i = startRow + 1; i <= endRow; i++){
+		QString s = d_table->text(i, col);
+		if (!s.isEmpty()){
+			double val = l.toDouble(s);
+			if (val > max)
+				max = val;
+		}
+	}
+	return max;
+}
 /*****************************************************************************
  *
  * Class MyTable
